@@ -119,6 +119,12 @@ class PegasusUPB : public INDI::DefaultDevice, public INDI::FocuserInterface, pu
          */
         bool sendCommand(const char *cmd, char *res);
 
+        /**
+         * @brief cleanupResponse Removes all spaces
+         * @param response buffer
+         */
+        void cleanupResponse(char *response);
+
         /**
          * @return Return true if sensor data different from last data
          */
@@ -262,7 +268,7 @@ class PegasusUPB : public INDI::DefaultDevice, public INDI::FocuserInterface, pu
         /// USB
         ////////////////////////////////////////////////////////////////////////////////////
         ITextVectorProperty FirmwateTP;
-        IText FirmwareT[2];
+        IText FirmwareT[2] {};
         enum
         {
             FIRMWARE_VERSION,
