@@ -23,7 +23,7 @@
 #include "locale_compat.h"
 
 #include <libnova/julian_day.h>
-#include <libnova/precession.h>
+#include <libastro.h>
 
 #include <cmath>
 #include <unistd.h>
@@ -504,8 +504,8 @@ void GuideSim::TimerHit()
 int GuideSim::DrawCcdFrame(INDI::CCDChip * targetChip)
 {
     //  CCD frame is 16 bit data
-    float exposure_time;
-    float targetFocalLength;
+    double exposure_time;
+    double targetFocalLength;
 
     uint16_t * ptr = reinterpret_cast<uint16_t *>(targetChip->getFrameBuffer());
 
@@ -590,6 +590,8 @@ int GuideSim::DrawCcdFrame(INDI::CCDChip * targetChip)
         double theta = rotationCW + 270;
         if (theta > 360)
             theta -= 360;
+        if (pierSide == 1)
+            theta -= 180;       // rotate 180 if on East
         else if (theta < -360)
             theta += 360;
 
@@ -622,7 +624,8 @@ int GuideSim::DrawCcdFrame(INDI::CCDChip * targetChip)
             epochPos.dec = currentDE;
 
             // Convert from JNow to J2000
-            ln_get_equ_prec2(&epochPos, ln_get_julian_from_sys(), JD2000, &J2000Pos);
+            LibAstro::ObservedToJ2000(&epochPos, ln_get_julian_from_sys(), &J2000Pos);
+            //ln_get_equ_prec2(&epochPos, ln_get_julian_from_sys(), JD2000, &J2000Pos);
 
             currentRA  = J2000Pos.ra / 15.0;
             currentDE = J2000Pos.dec;
@@ -1185,7 +1188,8 @@ bool GuideSim::ISNewNumber(const char * dev, const char * name, double values[],
             RA = EqPEN[AXIS_RA].value;
             Dec = EqPEN[AXIS_DE].value;
 
-            ln_get_equ_prec2(&epochPos, ln_get_julian_from_sys(), JD2000, &J2000Pos);
+            LibAstro::ObservedToJ2000(&epochPos, ln_get_julian_from_sys(), &J2000Pos);
+            //ln_get_equ_prec2(&epochPos, ln_get_julian_from_sys(), JD2000, &J2000Pos);
             currentRA  = J2000Pos.ra / 15.0;
             currentDE = J2000Pos.dec;
             usePE = true;
