@@ -72,10 +72,11 @@ void ISSnoopDevice(XMLEle * root)
 {
     deepSkyDadAf3->ISSnoopDevice(root);
 }
- 
+
 DeepSkyDadAF3::DeepSkyDadAF3()
 {
-    FI::SetCapability(FOCUSER_CAN_ABS_MOVE | FOCUSER_CAN_REL_MOVE | FOCUSER_CAN_SYNC | FOCUSER_CAN_REVERSE | FOCUSER_CAN_ABORT | FOCUSER_HAS_BACKLASH);
+    FI::SetCapability(FOCUSER_CAN_ABS_MOVE | FOCUSER_CAN_REL_MOVE | FOCUSER_CAN_SYNC | FOCUSER_CAN_REVERSE | FOCUSER_CAN_ABORT |
+                      FOCUSER_HAS_BACKLASH);
 }
 
 bool DeepSkyDadAF3::initProperties()
@@ -92,15 +93,17 @@ bool DeepSkyDadAF3::initProperties()
     IUFillSwitch(&StepModeS[S4], "S4", "1/4 Step", ISS_OFF);
     IUFillSwitch(&StepModeS[S2], "S2", "1/2 Step", ISS_OFF);
     IUFillSwitch(&StepModeS[S1], "S1", "Full Step", ISS_OFF);
-    IUFillSwitchVector(&StepModeSP, StepModeS, 9, getDeviceName(), "Step Mode", "", OPTIONS_TAB, IP_RW, ISR_1OFMANY, 0, IPS_IDLE);
-    
+    IUFillSwitchVector(&StepModeSP, StepModeS, 9, getDeviceName(), "Step Mode", "", OPTIONS_TAB, IP_RW, ISR_1OFMANY, 0,
+                       IPS_IDLE);
+
     // Speed Mode
     IUFillSwitch(&SpeedModeS[VERY_SLOW], "VERY_SLOW", "Very slow", ISS_OFF);
     IUFillSwitch(&SpeedModeS[SLOW], "SLOW", "Slow", ISS_OFF);
     IUFillSwitch(&SpeedModeS[MEDIUM], "MEDIUM", "Medium", ISS_OFF);
     IUFillSwitch(&SpeedModeS[FAST], "FAST", "Fast", ISS_OFF);
     IUFillSwitch(&SpeedModeS[VERY_FAST], "VERY_FAST", "Very fast", ISS_OFF);
-    IUFillSwitchVector(&SpeedModeSP, SpeedModeS, 5, getDeviceName(), "Speed Mode", "", OPTIONS_TAB, IP_RW, ISR_1OFMANY, 0, IPS_IDLE);
+    IUFillSwitchVector(&SpeedModeSP, SpeedModeS, 5, getDeviceName(), "Speed Mode", "", OPTIONS_TAB, IP_RW, ISR_1OFMANY, 0,
+                       IPS_IDLE);
 
     /* Relative and absolute movement */
     FocusRelPosN[0].min = 0.;
@@ -115,7 +118,7 @@ bool DeepSkyDadAF3::initProperties()
 
     FocusMaxPosN[0].min = 0.;
     FocusMaxPosN[0].max = 1000000.;
-    FocusMaxPosN[0].value = 50000.;
+    FocusMaxPosN[0].value = 1000000.;
     FocusMaxPosN[0].step = 5000.;
 
     FocusSyncN[0].min = 0.;
@@ -137,17 +140,19 @@ bool DeepSkyDadAF3::initProperties()
     IUFillNumber(&SettleBufferN[0], "SETTLE_BUFFER", "Period (ms)", "%5.0f", 0, 99999, 100, 0);
     IUFillNumberVector(&SettleBufferNP, SettleBufferN, 1, getDeviceName(), "FOCUS_SETTLE_BUFFER", "Settle buffer",
                        OPTIONS_TAB, IP_RW, 0, IPS_IDLE);
-                       
+
     // Motor move multiplier
     IUFillNumber(&MoveCurrentMultiplierN[0], "MOTOR_MOVE_MULTIPLIER", "%", "%3.0f", 1, 100, 1, 90);
-    IUFillNumberVector(&MoveCurrentMultiplierNP, MoveCurrentMultiplierN, 1, getDeviceName(), "FOCUS_MMM", "Move current multiplier",
-       OPTIONS_TAB, IP_RW, 0, IPS_IDLE);
-       
+    IUFillNumberVector(&MoveCurrentMultiplierNP, MoveCurrentMultiplierN, 1, getDeviceName(), "FOCUS_MMM",
+                       "Move current multiplier",
+                       OPTIONS_TAB, IP_RW, 0, IPS_IDLE);
+
     // Motor hold multiplier
     IUFillNumber(&HoldCurrentMultiplierN[0], "MOTOR_HOLD_MULTIPLIER", "%", "%3.0f", 1, 100, 1, 40);
-    IUFillNumberVector(&HoldCurrentMultiplierNP, HoldCurrentMultiplierN, 1, getDeviceName(), "FOCUS_MHM", "Hold current multiplier",
-       OPTIONS_TAB, IP_RW, 0, IPS_IDLE);
-    
+    IUFillNumberVector(&HoldCurrentMultiplierNP, HoldCurrentMultiplierN, 1, getDeviceName(), "FOCUS_MHM",
+                       "Hold current multiplier",
+                       OPTIONS_TAB, IP_RW, 0, IPS_IDLE);
+
     // Focuser temperature
     IUFillNumber(&TemperatureN[0], "TEMPERATURE", "Celsius", "%6.2f", -50, 70., 0., 0.);
     IUFillNumberVector(&TemperatureNP, TemperatureN, 1, getDeviceName(), "FOCUS_TEMPERATURE", "Temperature",
@@ -214,7 +219,7 @@ bool DeepSkyDadAF3::Ack()
     sleep(2);
 
     char res[DSD_RES] = {0};
-    if (!sendCommand("[GPOS]", res) && !sendCommand("[GPOS]", res)) //try twice 
+    if (!sendCommand("[GPOS]", res) && !sendCommand("[GPOS]", res)) //try twice
     {
         LOG_ERROR("ACK - getPosition failed");
         return false;
@@ -409,7 +414,7 @@ bool DeepSkyDadAF3::readMoveCurrentMultiplier()
 
 bool DeepSkyDadAF3::readHoldCurrentMultiplier()
 {
-     char res[DSD_RES] = {0};
+    char res[DSD_RES] = {0};
 
     if (sendCommand("[GMHM]", res) == false)
         return false;
@@ -432,14 +437,15 @@ bool DeepSkyDadAF3::readHoldCurrentMultiplier()
 
 bool DeepSkyDadAF3::readTemperature()
 {
-    char res[DSD_RES]= {0};
+    char res[DSD_RES] = {0};
 
     if (sendCommand("[GTMC]", res) == false)
         return false;
 
     double temp = 0;
     int rc = sscanf(res, "(%lf)", &temp);
-    if (rc > 0) {
+    if (rc > 0)
+    {
         TemperatureN[0].value = temp;
     }
     else
@@ -561,7 +567,7 @@ bool DeepSkyDadAF3::ISNewSwitch(const char * dev, const char * name, ISState * s
             IDSetSwitch(&StepModeSP, nullptr);
             return true;
         }
-        
+
         // Focus Speed Mode
         if (strcmp(SpeedModeSP.name, name) == 0)
         {
@@ -632,7 +638,7 @@ bool DeepSkyDadAF3::ISNewNumber(const char * dev, const char * name, double valu
             IDSetNumber(&SettleBufferNP, nullptr);
             return true;
         }
-        
+
         // Move current multiplier
         if (strcmp(name, MoveCurrentMultiplierNP.name) == 0)
         {
@@ -650,7 +656,7 @@ bool DeepSkyDadAF3::ISNewNumber(const char * dev, const char * name, double valu
             IDSetNumber(&MoveCurrentMultiplierNP, nullptr);
             return true;
         }
-        
+
         // Hold current multiplier
         if (strcmp(name, HoldCurrentMultiplierNP.name) == 0)
         {
@@ -688,22 +694,22 @@ bool DeepSkyDadAF3::ISNewNumber(const char * dev, const char * name, double valu
         }
 
         // Max. movement
-        if (strcmp(name, FocusMaxMoveNP.name) == 0)
-        {
-            IUUpdateNumber(&FocusMaxMoveNP, values, names, n);
-            char cmd[DSD_RES] = {0};
-            snprintf(cmd, DSD_RES, "[SMXM%d]", static_cast<int>(FocusMaxMoveN[0].value));
-            bool rc = sendCommandSet(cmd);
-            if (!rc)
-            {
-                FocusMaxMoveNP.s = IPS_ALERT;
-                return false;
-            }
+        //        if (strcmp(name, FocusMaxMoveNP.name) == 0)
+        //        {
+        //            IUUpdateNumber(&FocusMaxMoveNP, values, names, n);
+        //            char cmd[DSD_RES] = {0};
+        //            snprintf(cmd, DSD_RES, "[SMXM%d]", static_cast<int>(FocusMaxMoveN[0].value));
+        //            bool rc = sendCommandSet(cmd);
+        //            if (!rc)
+        //            {
+        //                FocusMaxMoveNP.s = IPS_ALERT;
+        //                return false;
+        //            }
 
-            FocusMaxMoveNP.s = IPS_OK;
-            IDSetNumber(&FocusMaxMoveNP, nullptr);
-            return true;
-        }
+        //            FocusMaxMoveNP.s = IPS_OK;
+        //            IDSetNumber(&FocusMaxMoveNP, nullptr);
+        //            return true;
+        //        }
 
     }
 
@@ -720,16 +726,16 @@ void DeepSkyDadAF3::GetFocusParams()
 
     if (readStepMode())
         IDSetSwitch(&StepModeSP, nullptr);
-    
+
     if (readSpeedMode())
         IDSetSwitch(&SpeedModeSP, nullptr);
 
     if (readSettleBuffer())
         IDSetNumber(&SettleBufferNP, nullptr);
-    
+
     if (readMoveCurrentMultiplier())
         IDSetNumber(&MoveCurrentMultiplierNP, nullptr);
-    
+
     if (readHoldCurrentMultiplier())
         IDSetNumber(&HoldCurrentMultiplierNP, nullptr);
 
@@ -738,7 +744,7 @@ void DeepSkyDadAF3::GetFocusParams()
 
     if (readMaxMovement())
         IDSetNumber(&FocusMaxMoveNP, nullptr);
-    
+
     if (readTemperature())
         IDSetNumber(&TemperatureNP, nullptr);
 }
@@ -842,13 +848,18 @@ void DeepSkyDadAF3::TimerHit()
             IDSetNumber(&FocusRelPosNP, nullptr);
             lastPos = FocusAbsPosN[0].value;
 
-            if(moveAborted) {
+            if(moveAborted)
+            {
                 LOG_INFO("Move aborted.");
-            } else if(backlashComp != 0) {
+            }
+            else if(backlashComp != 0)
+            {
                 LOGF_INFO("Performing backlash compensation of %i.", (int)backlashComp);
                 targetPos += backlashComp;
                 MoveFocuser(targetPos);
-            } else {
+            }
+            else
+            {
                 LOG_INFO("Focuser reached requested position.");
             }
 
@@ -856,11 +867,11 @@ void DeepSkyDadAF3::TimerHit()
             backlashComp = 0;
         }
     }
-    
+
     rc = readTemperature();
     if (rc)
     {
-        if (fabs(lastTemperature - TemperatureN[0].value) >= 0.5)
+        if (fabs(lastTemperature - TemperatureN[0].value) >= 0.1 ) //more accurate update
         {
             IDSetNumber(&TemperatureNP, nullptr);
             lastTemperature = TemperatureN[0].value;
@@ -940,3 +951,17 @@ bool DeepSkyDadAF3::SetFocuserBacklash(int32_t steps)
     return true;
 }
 
+bool DeepSkyDadAF3::SetFocuserMaxPosition(uint32_t ticks)
+{
+    char cmd[DSD_RES] = {0};
+
+    snprintf(cmd, DSD_RES, "[SMXP%d]", ticks);
+
+    if (sendCommandSet(cmd))
+    {
+        SyncPresets(ticks);
+        return true;
+    }
+
+    return false;
+}
