@@ -1,3 +1,23 @@
+/*
+    SestoSenso 2 Focuser
+    Copyright (C) 2020 Piotr Zyziuk
+    Copyright (C) 2020 Jasem Mutlaq (Added Esatto support)
+
+    This library is free software; you can redistribute it and/or
+    modify it under the terms of the GNU Lesser General Public
+    License as published by the Free Software Foundation; either
+    version 2.1 of the License, or (at your option) any later version.
+
+    This library is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+    Lesser General Public License for more details.
+
+    You should have received a copy of the GNU Lesser General Public
+    License along with this library; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+
+*/
 
 #include "sestosenso2.h"
 
@@ -56,7 +76,7 @@ void ISSnoopDevice(XMLEle *root)
 
 SestoSenso2::SestoSenso2()
 {
-    setVersion(0, 1);
+    setVersion(0, 6);
     // Can move in Absolute & Relative motions, can AbortFocuser motion.
     FI::SetCapability(FOCUSER_CAN_ABS_MOVE | FOCUSER_CAN_REL_MOVE | FOCUSER_CAN_ABORT);
 
@@ -69,34 +89,54 @@ bool SestoSenso2::initProperties()
 
     setConnectionParams();
 
-    // Firmware Information
-    IUFillText(&FirmwareT[0], "VERSION", "Version", "");
-    IUFillTextVector(&FirmwareTP, FirmwareT, 1, getDeviceName(), "FOCUS_FIRMWARE", "Firmware", MAIN_CONTROL_TAB, IP_RO, 0, IPS_IDLE);
+    // Firmware information
+    IUFillText(&FirmwareT[FIRMWARE_SN], "SERIALNUMBER", "Serial Number", "");
+    IUFillText(&FirmwareT[FIRMWARE_VERSION], "VERSION", "Version", "");
+    IUFillTextVector(&FirmwareTP, FirmwareT, 2, getDeviceName(), "FOCUS_FIRMWARE", "Firmware", CONNECTION_TAB, IP_RO, 0,
+                     IPS_IDLE);
+
+    // Voltage Information
+    IUFillNumber(&VoltageInN[0], "VOLTAGEIN", "Volts", "%.2f", 0, 100, 0., 0.);
+    IUFillNumberVector(&VoltageInNP, VoltageInN, 1, getDeviceName(), "VOLTAGE_IN", "Voltage in", MAIN_CONTROL_TAB, IP_RO, 0,
+                     IPS_IDLE);
 
     // Focuser temperature
-    IUFillNumber(&TemperatureN[0], "TEMPERATURE", "Celsius", "%6.2f", -50, 70., 0., 0.);
-    IUFillNumberVector(&TemperatureNP, TemperatureN, 1, getDeviceName(), "FOCUS_TEMPERATURE", "Motor temp.", MAIN_CONTROL_TAB, IP_RO, 0, IPS_IDLE);
+    IUFillNumber(&TemperatureN[TEMPERATURE_MOTOR], "TEMPERATURE", "Motor (c)", "%.2f", -50, 70., 0., 0.);
+    IUFillNumber(&TemperatureN[TEMPERATURE_EXTERNAL], "TEMPERATURE_ETX", "External (c)", "%.2f", -50, 70., 0., 0.);
+    IUFillNumberVector(&TemperatureNP, TemperatureN, 2, getDeviceName(), "FOCUS_TEMPERATURE", "Temperature", MAIN_CONTROL_TAB,
+                       IP_RO, 0, IPS_IDLE);
 
-    IUFillNumber(&SpeedN[0], "SPEED", "RPM", "%0.0f", 0, 7000., 1, 0);
-    IUFillNumberVector(&SpeedNP, SpeedN, 1, getDeviceName(), "FOCUS_SPEED", "Motor speed", MAIN_CONTROL_TAB, IP_RO, 0, IPS_IDLE);
+    // Current Speed
+    IUFillNumber(&SpeedN[0], "SPEED", "steps/s", "%.f", 0, 7000., 1, 0);
+    IUFillNumberVector(&SpeedNP, SpeedN, 1, getDeviceName(), "FOCUS_SPEED", "Motor speed", MAIN_CONTROL_TAB, IP_RO, 0,
+                       IPS_IDLE);
 
     // Focuser calibration
-    IUFillText(&CalibrationMessageT[0], "CALIBRATION", "Calibration stage", "");
-    IUFillTextVector(&CalibrationMessageTP, CalibrationMessageT, 1, getDeviceName(), "CALIBRATION_MESSAGE", "Calibration", MAIN_CONTROL_TAB, IP_RO, 0, IPS_IDLE);
+    IUFillText(&CalibrationMessageT[0], "CALIBRATION", "Calibration stage", "Press START to begin the Calibration.");
+    IUFillTextVector(&CalibrationMessageTP, CalibrationMessageT, 1, getDeviceName(), "CALIBRATION_MESSAGE", "Calibration",
+                     MAIN_CONTROL_TAB, IP_RO, 0, IPS_IDLE);
 
+    // Calibration
     IUFillSwitch(&CalibrationS[CALIBRATION_START], "CALIBRATION_START", "Start", ISS_OFF);
     IUFillSwitch(&CalibrationS[CALIBRATION_NEXT], "CALIBRATION_NEXT", "Next", ISS_OFF);
-    IUFillSwitchVector(&CalibrationSP, CalibrationS, 2, getDeviceName(), "FOCUS_CALIBRATION", "Calibration", MAIN_CONTROL_TAB, IP_RW, ISR_1OFMANY, 0, IPS_IDLE);
+    IUFillSwitchVector(&CalibrationSP, CalibrationS, 2, getDeviceName(), "FOCUS_CALIBRATION", "Calibration", MAIN_CONTROL_TAB,
+                       IP_RW, ISR_ATMOST1, 0, IPS_IDLE);
 
+    // Speed Moves
     IUFillSwitch(&FastMoveS[FASTMOVE_IN], "FASTMOVE_IN", "Move In", ISS_OFF);
     IUFillSwitch(&FastMoveS[FASTMOVE_OUT], "FASTMOVE_OUT", "Move out", ISS_OFF);
     IUFillSwitch(&FastMoveS[FASTMOVE_STOP], "FASTMOVE_STOP", "Stop", ISS_OFF);
+    IUFillSwitchVector(&FastMoveSP, FastMoveS, 3, getDeviceName(), "FAST_MOVE", "Calibration Move", MAIN_CONTROL_TAB, IP_RW,
+                       ISR_ATMOST1, 0, IPS_IDLE);
 
-    IUFillSwitchVector(&FastMoveSP, FastMoveS, 3, getDeviceName(), "FAST_MOVE", "Calibration Move", MAIN_CONTROL_TAB, IP_RW, ISR_1OFMANY, 0, IPS_IDLE);
-
-    //
     // Override the default Max. Position to make it Read-Only
-    IUFillNumberVector(&FocusMaxPosNP, FocusMaxPosN, 1, getDeviceName(), "FOCUS_MAX", "Max. Position", MAIN_CONTROL_TAB, IP_RO, 0, IPS_IDLE);
+    IUFillNumberVector(&FocusMaxPosNP, FocusMaxPosN, 1, getDeviceName(), "FOCUS_MAX", "Max. Position", MAIN_CONTROL_TAB, IP_RO,
+                       0, IPS_IDLE);
+
+    // Home Position
+    IUFillSwitch(&HomeS[0], "FOCUS_HOME_GO", "Go", ISS_OFF);
+    IUFillSwitchVector(&HomeSP, HomeS, 1, getDeviceName(), "FOCUS_HOME", "Home", MAIN_CONTROL_TAB, IP_RW, ISR_ATMOST1, 60,
+                       IPS_IDLE);
 
     // Relative and absolute movement
     FocusRelPosN[0].min   = 0.;
@@ -110,6 +150,9 @@ bool SestoSenso2::initProperties()
     FocusAbsPosN[0].step  = 1000;
 
     FocusMaxPosN[0].value = 2097152;
+    PresetN[0].max = FocusMaxPosN[0].value;
+    PresetN[1].max = FocusMaxPosN[0].value;
+    PresetN[2].max = FocusMaxPosN[0].value;
 
     addAuxControls();
 
@@ -120,6 +163,9 @@ bool SestoSenso2::initProperties()
 
 bool SestoSenso2::updateProperties()
 {
+    if (isConnected() && updateMaxLimit() == false)
+        LOGF_WARN("Check you have the latest %s firmware. Focuser requires calibration.", getDeviceName());
+
     INDI::Focuser::updateProperties();
 
     if (isConnected())
@@ -128,12 +174,18 @@ bool SestoSenso2::updateProperties()
             defineNumber(&TemperatureNP);
         defineNumber(&SpeedNP);
         defineText(&FirmwareTP);
-        IUSaveText(&CalibrationMessageT[0], "Press START to begin the Calibration");
+        if (updateVoltageIn())
+            defineNumber(&VoltageInNP);
+
+        //        if (m_IsSestoSenso2)
+        //        {
         defineText(&CalibrationMessageTP);
         defineSwitch(&CalibrationSP);
+        //}
+        defineSwitch(&HomeSP);
 
         if (getStartupValues())
-            LOG_INFO("SestoSenso parameters updated, focuser ready for use.");
+            LOG_INFO("Parameters updated, focuser ready for use.");
         else
             LOG_WARN("Failed to inquire parameters. Check logs.");
     }
@@ -142,9 +194,11 @@ bool SestoSenso2::updateProperties()
         if (TemperatureNP.s == IPS_OK)
             deleteProperty(TemperatureNP.name);
         deleteProperty(FirmwareTP.name);
+        deleteProperty(VoltageInNP.name);
         deleteProperty(CalibrationMessageTP.name);
         deleteProperty(CalibrationSP.name);
         deleteProperty(SpeedNP.name);
+        deleteProperty(HomeSP.name);
     }
 
     return true;
@@ -154,18 +208,18 @@ bool SestoSenso2::Handshake()
 {
     if (Ack())
     {
-        LOG_INFO("SestoSenso is online. Getting focus parameters...");
+        LOGF_INFO("%s is online. Getting focus parameters...", getDeviceName());
         return true;
     }
 
-    LOG_INFO("Error retrieving data from SestoSenso, please ensure SestoSenso controller is powered and the port is correct.");
+    LOG_INFO("Error retrieving data from device, please ensure focuser is powered and the port is correct.");
     return false;
 }
 
 bool SestoSenso2::Disconnect()
 {
-    if (isSimulation() == false)
-        command->goHome();
+    //    if (isSimulation() == false)
+    //        command->goHome();
 
     return INDI::Focuser::Disconnect();
 }
@@ -182,7 +236,7 @@ bool SestoSenso2::updateTemperature()
 
     if (isSimulation())
         strncpy(res, "23.45", SESTO_LEN);
-    else if (command->getMotorTemp(res) == CMD_FALSE)
+    else if (command->getMotorTemp(res) == false)
         return false;
 
     try
@@ -198,9 +252,26 @@ bool SestoSenso2::updateTemperature()
     if (temperature > 90)
         return false;
 
-    TemperatureN[0].value = temperature;
+    TemperatureN[TEMPERATURE_MOTOR].value = temperature;
     TemperatureNP.s = IPS_OK;
 
+    // External temperature - Optional
+    if (command->getExternalTemp(res))
+    {
+        TemperatureN[TEMPERATURE_EXTERNAL].value = -273.15;
+        try
+        {
+            temperature = std::stod(res);
+        }
+        catch(...)
+        {
+            LOGF_DEBUG("Failed to process external temperature response: %s (%d bytes)", res, strlen(res));
+        }
+
+        if (temperature < 90)
+            TemperatureN[TEMPERATURE_EXTERNAL].value = temperature;
+    }
+
     return true;
 }
 
@@ -212,7 +283,7 @@ bool SestoSenso2::updateMaxLimit()
     if (isSimulation())
         return true;
 
-    if (command->getMaxPosition(res) == CMD_FALSE)
+    if (command->getMaxPosition(res) == false)
         return false;
 
     int maxLimit = 0;
@@ -235,11 +306,15 @@ bool SestoSenso2::updateMaxLimit()
         FocusRelPosN[0].value = 0;
         FocusRelPosN[0].step  = FocusAbsPosN[0].step;
 
-        IUUpdateMinMax(&FocusAbsPosNP);
-        IUUpdateMinMax(&FocusRelPosNP);
+        PresetN[0].max = maxLimit;
+        PresetN[0].step = (FocusAbsPosN[0].max - FocusAbsPosN[0].min) / 50.0;
+        PresetN[1].max = maxLimit;
+        PresetN[1].step = (FocusAbsPosN[0].max - FocusAbsPosN[0].min) / 50.0;
+        PresetN[2].max = maxLimit;
+        PresetN[2].step = (FocusAbsPosN[0].max - FocusAbsPosN[0].min) / 50.0;
+
 
         FocusMaxPosNP.s = IPS_OK;
-        IUUpdateMinMax(&FocusMaxPosNP);
         return true;
     }
 
@@ -252,8 +327,8 @@ bool SestoSenso2::updatePosition()
 {
     char res[SESTO_LEN] = {0};
     if (isSimulation())
-        snprintf(res, SESTO_LEN, "%d", static_cast<uint32_t>(FocusAbsPosN[0].value));
-    else if (command->getAbsolutePosition(res) == CMD_FALSE)
+        snprintf(res, SESTO_LEN, "%u", static_cast<uint32_t>(FocusAbsPosN[0].value));
+    else if (command->getAbsolutePosition(res) == false)
         return false;
 
     try
@@ -270,10 +345,39 @@ bool SestoSenso2::updatePosition()
     }
 }
 
+bool SestoSenso2::updateVoltageIn()
+{
+    char res[SESTO_LEN] = {0};
+    double voltageIn = 0;
+
+    if (isSimulation())
+        strncpy(res, "12.00", SESTO_LEN);
+    else if (command->getVoltageIn(res) == false)
+        return false;
+
+    try
+    {
+        voltageIn = std::stod(res);
+    }
+    catch(...)
+    {
+        LOGF_WARN("Failed to process voltage response: %s (%d bytes)", res, strlen(res));
+        return false;
+    }
+
+    if (voltageIn > 24)
+        return false;
+
+    VoltageInN[0].value = voltageIn;
+    VoltageInNP.s = (voltageIn >= 11.0) ? IPS_OK : IPS_ALERT;
+
+    return true;
+}
+
 bool SestoSenso2::setupRunPreset()
 {
     char res[SESTO_LEN] = {0};
-    if (command->loadSlowPreset(res) == CMD_FALSE)
+    if (command->loadSlowPreset(res) == false)
     {
         return false;
     }
@@ -305,7 +409,7 @@ bool SestoSenso2::isMotionComplete()
     }
     else
     {
-        if(command->getCurrentSpeed(res) == CMD_OK)
+        if(command->getCurrentSpeed(res))
         {
             try
             {
@@ -322,7 +426,7 @@ bool SestoSenso2::isMotionComplete()
                 return true;
 
             *res = {0};
-            if(command->getAbsolutePosition(res) == CMD_OK)
+            if(command->getAbsolutePosition(res))
             {
                 try
                 {
@@ -345,11 +449,9 @@ bool SestoSenso2::ISNewSwitch(const char *dev, const char *name, ISState *states
 {
     if (dev != nullptr && strcmp(dev, getDeviceName()) == 0)
     {
-
         // Calibrate focuser
         if (!strcmp(name, CalibrationSP.name))
         {
-
             int current_switch = 0;
 
             CalibrationSP.s = IPS_BUSY;
@@ -372,10 +474,10 @@ bool SestoSenso2::ISNewSwitch(const char *dev, const char *name, ISState *states
                     //
                     // Init
                     //
-                    if (command->initCalibration() == false)
+                    if (m_IsSestoSenso2 && command->initCalibration() == false)
                         return false;
 
-                    IUSaveText(&CalibrationMessageT[0], "Set focus in MIN position and then press NEXT");
+                    IUSaveText(&CalibrationMessageT[0], "Set focus in MIN position and then press NEXT.");
                     IDSetText(&CalibrationMessageTP, nullptr);
 
                     // Set next step
@@ -392,20 +494,33 @@ bool SestoSenso2::ISNewSwitch(const char *dev, const char *name, ISState *states
             {
                 if (cStage == GoToMiddle)
                 {
-                    if (command->storeAsMinPosition() == CMD_FALSE)
-                        return false;
                     defineSwitch(&FastMoveSP);
-                    IUSaveText(&CalibrationMessageT[0], "Press MOVE OUT to move focuser out (CAUTION!)");
-                    IDSetText(&CalibrationMessageTP, nullptr);
-                    cStage = GoMinimum;
+                    if (m_IsSestoSenso2)
+                    {
+                        if (command->storeAsMinPosition() == false)
+                            return false;
+
+                        IUSaveText(&CalibrationMessageT[0], "Press MOVE OUT to move focuser out (CAUTION!)");
+                        IDSetText(&CalibrationMessageTP, nullptr);
+                        cStage = GoMinimum;
+                    }
+                    // For Esatto, start moving out immediately
+                    else
+                    {
+                        cStage = GoMaximum;
+
+                        ISState fs[1] = { ISS_ON };
+                        char *fn[1] =  { FastMoveS[FASTMOVE_OUT].name };
+                        ISNewSwitch(getDeviceName(), FastMoveSP.name, fs, fn, 1);
+                    }
                 }
                 else if (cStage == GoMinimum)
                 {
                     char res[SESTO_LEN] = {0};
-                    if (command->storeAsMaxPosition(res) == CMD_FALSE)
+                    if (m_IsSestoSenso2 && command->storeAsMaxPosition(res) == false)
                         return false;
 
-                    IUSaveText(&CalibrationMessageT[0], "Press NEXT to finish");
+                    IUSaveText(&CalibrationMessageT[0], "Press NEXT to finish.");
                     IDSetText(&CalibrationMessageTP, nullptr);
                     cStage = GoMaximum;
                 }
@@ -413,7 +528,7 @@ bool SestoSenso2::ISNewSwitch(const char *dev, const char *name, ISState *states
                 {
                     char res[SESTO_LEN] = {0};
 
-                    if (command->getMaxPosition(res) == CMD_FALSE)
+                    if (command->getMaxPosition(res) == false)
                         return false;
 
                     int maxLimit = 0;
@@ -433,9 +548,17 @@ bool SestoSenso2::ISNewSwitch(const char *dev, const char *name, ISState *states
                     FocusRelPosN[0].value = 0;
                     FocusRelPosN[0].step  = FocusAbsPosN[0].step;
 
+                    PresetN[0].max = maxLimit;
+                    PresetN[0].step = (FocusAbsPosN[0].max - FocusAbsPosN[0].min) / 50.0;
+                    PresetN[1].max = maxLimit;
+                    PresetN[1].step = (FocusAbsPosN[0].max - FocusAbsPosN[0].min) / 50.0;
+                    PresetN[2].max = maxLimit;
+                    PresetN[2].step = (FocusAbsPosN[0].max - FocusAbsPosN[0].min) / 50.0;
+
+                    FocusMaxPosNP.s = IPS_OK;
                     IUUpdateMinMax(&FocusAbsPosNP);
                     IUUpdateMinMax(&FocusRelPosNP);
-                    FocusMaxPosNP.s = IPS_OK;
+                    IUUpdateMinMax(&PresetNP);
                     IUUpdateMinMax(&FocusMaxPosNP);
 
                     IUSaveText(&CalibrationMessageT[0], "Calibration Completed.");
@@ -452,50 +575,88 @@ bool SestoSenso2::ISNewSwitch(const char *dev, const char *name, ISState *states
                 }
                 else
                 {
-                    IUSaveText(&CalibrationMessageT[0], "Calibration not in process");
+                    IUSaveText(&CalibrationMessageT[0], "Calibration not in progress.");
                     IDSetText(&CalibrationMessageTP, nullptr);
                 }
 
             }
             return true;
         }
+        // Fast motion
         else if (!strcmp(name, FastMoveSP.name))
         {
             IUUpdateSwitch(&FastMoveSP, states, names, n);
             int current_switch = IUFindOnSwitchIndex(&FastMoveSP);
+            char res[SESTO_LEN] = {0};
 
             switch (current_switch)
             {
-            case FASTMOVE_IN:
-                if (command->fastMoveIn() == CMD_FALSE)
-                {
-                    return false;
-                }
-                break;
-            case FASTMOVE_OUT:
-                if (command->goOutToFindMaxPos() == CMD_FALSE)
-                {
-                    return false;
-                }
-                IUSaveText(&CalibrationMessageT[0], "Press STOP focuser almost at MAX position.");
-                IDSetText(&CalibrationMessageTP, nullptr);
-                break;
-            case FASTMOVE_STOP:
-                if (command->stop() == CMD_FALSE)
-                {
-                    return false;
-                }
-                IUSaveText(&CalibrationMessageT[0], "Press NEXT to store max limit");
-                IDSetText(&CalibrationMessageTP, nullptr);
-                break;
-            default:
-                break;
+                case FASTMOVE_IN:
+                    if (command->fastMoveIn(res) == false)
+                    {
+                        return false;
+                    }
+                    break;
+                case FASTMOVE_OUT:
+                    if (m_IsSestoSenso2)
+                    {
+                        if (command->goOutToFindMaxPos() == false)
+                        {
+                            return false;
+                        }
+                        IUSaveText(&CalibrationMessageT[0], "Press STOP focuser almost at MAX position.");
+                    }
+                    else
+                    {
+                        if (command->fastMoveOut(res))
+                        {
+                            IUSaveText(&CalibrationMessageT[0], "Focusing out to detect hall sensor.");
+
+                            if (m_MotionProgressTimerID > 0)
+                                IERmTimer(m_MotionProgressTimerID);
+                            m_MotionProgressTimerID = IEAddTimer(500, &SestoSenso2::checkMotionProgressHelper, this);
+                        }
+                    }
+                    IDSetText(&CalibrationMessageTP, nullptr);
+                    break;
+                case FASTMOVE_STOP:
+                    if (command->stop() == false)
+                    {
+                        return false;
+                    }
+                    IUSaveText(&CalibrationMessageT[0], "Press NEXT to store max limit.");
+                    IDSetText(&CalibrationMessageTP, nullptr);
+                    break;
+                default:
+                    break;
             }
 
             FastMoveSP.s = IPS_BUSY;
             IDSetSwitch(&FastMoveSP, nullptr);
             return true;
         }
+        // Homing
+        else if (!strcmp(HomeSP.name, name))
+        {
+            char res[SESTO_LEN] = {0};
+            if (command->goHome(res))
+            {
+                HomeS[0].s = ISS_ON;
+                HomeSP.s = IPS_BUSY;
+
+                if (m_MotionProgressTimerID > 0)
+                    IERmTimer(m_MotionProgressTimerID);
+                m_MotionProgressTimerID = IEAddTimer(100, &SestoSenso2::checkMotionProgressHelper, this);
+            }
+            else
+            {
+                HomeS[0].s = ISS_OFF;
+                HomeSP.s = IPS_ALERT;
+            }
+
+            IDSetSwitch(&HomeSP, nullptr);
+            return true;
+        }
 
     }
     return INDI::Focuser::ISNewSwitch(dev, name, states, names, n);
@@ -504,11 +665,11 @@ bool SestoSenso2::ISNewSwitch(const char *dev, const char *name, ISState *states
 IPState SestoSenso2::MoveAbsFocuser(uint32_t targetTicks)
 {
     targetPos = targetTicks;
-    char res[SESTO_LEN] = {0};
 
     if (isSimulation() == false)
     {
-        if (command->go(targetTicks,res) == CMD_FALSE)
+        char res[SESTO_LEN] = {0};
+        if (command->go(targetTicks, res) == false)
             return IPS_ALERT;
     }
 
@@ -540,14 +701,28 @@ bool SestoSenso2::AbortFocuser()
     if (isSimulation())
         return true;
 
-    return command->abort();
+    bool rc = command->abort();
 
+    if (rc && HomeSP.s == IPS_BUSY)
+    {
+        HomeS[0].s = ISS_OFF;
+        HomeSP.s = IPS_IDLE;
+        IDSetSwitch(&HomeSP, nullptr);
+    }
+
+    return rc;
 }
 
 void SestoSenso2::checkMotionProgressHelper(void *context)
 {
     static_cast<SestoSenso2*>(context)->checkMotionProgressCallback();
 }
+
+void SestoSenso2::checkHallSensorHelper(void *context)
+{
+    static_cast<SestoSenso2*>(context)->checkHallSensorCallback();
+}
+
 //
 // This timer function is initiated when a GT command has been issued
 // A timer will call this function on a regular interval during the motion
@@ -568,7 +743,22 @@ void SestoSenso2::checkMotionProgressCallback()
 
 
         lastPos = FocusAbsPosN[0].value;
-        LOG_INFO("Focuser reached requested position.");
+
+        if (HomeSP.s == IPS_BUSY)
+        {
+            LOG_INFO("Focuser at home position.");
+            HomeS[0].s = ISS_OFF;
+            HomeSP.s = IPS_OK;
+            IDSetSwitch(&HomeSP, nullptr);
+        }
+        else if (CalibrationSP.s == IPS_BUSY)
+        {
+            ISState states[2] = { ISS_OFF, ISS_ON };
+            char * names[2] = { CalibrationS[CALIBRATION_START].name, CalibrationS[CALIBRATION_NEXT].name };
+            ISNewSwitch(getDeviceName(), CalibrationSP.name, states, names, CalibrationSP.nsp);
+        }
+        else
+            LOG_INFO("Focuser reached requested position.");
         return;
     }
     else
@@ -585,9 +775,31 @@ void SestoSenso2::checkMotionProgressCallback()
     m_MotionProgressTimerID = IEAddTimer(500, &SestoSenso2::checkMotionProgressHelper, this);
 }
 
+void SestoSenso2::checkHallSensorCallback()
+{
+    char res[SESTO_LEN] = {0};
+    if (command->getHallSensor(res))
+    {
+        int detected = 0;
+        if (sscanf(res, "%d", &detected) == 1)
+        {
+            if (detected == 1)
+            {
+                ISState states[2] = { ISS_OFF, ISS_ON };
+                char * names[2] = { CalibrationS[CALIBRATION_START].name, CalibrationS[CALIBRATION_NEXT].name };
+                ISNewSwitch(getDeviceName(), CalibrationSP.name, states, names, CalibrationSP.nsp);
+                return;
+            }
+        }
+    }
+
+    m_HallSensorTimerID = IEAddTimer(1000, &SestoSenso2::checkHallSensorHelper, this);
+}
+
 void SestoSenso2::TimerHit()
 {
-    if (!isConnected() || FocusAbsPosNP.s == IPS_BUSY || FocusRelPosNP.s == IPS_BUSY || CalibrationSP.s == IPS_BUSY)
+    if (!isConnected() || FocusAbsPosNP.s == IPS_BUSY || FocusRelPosNP.s == IPS_BUSY || (m_IsSestoSenso2
+            && CalibrationSP.s == IPS_BUSY))
     {
         SetTimer(POLLMS);
         return;
@@ -614,6 +826,23 @@ void SestoSenso2::TimerHit()
                 lastTemperature = TemperatureN[0].value;
             }
         }
+
+        // Also use temparature poll rate for tracking input voltage
+        rc = updateVoltageIn();
+        if (rc)
+        {
+            if (fabs(lastVoltageIn - VoltageInN[0].value) >= 0.1)
+            {
+                IDSetNumber(&VoltageInNP, nullptr);
+                lastVoltageIn = VoltageInN[0].value;
+
+                if (VoltageInN[0].value < 11.0)
+                {
+                    LOG_WARN("Please check 12v DC power supply is connected.");
+                }
+            }
+        }
+
         m_TemperatureCounter = 0;   // Reset the counter
     }
 
@@ -622,15 +851,15 @@ void SestoSenso2::TimerHit()
 
 bool SestoSenso2::getStartupValues()
 {
-    setupRunPreset();
-    bool rc1 = updatePosition();
-    if (rc1)
+    // Do not run for Esatto
+    if (m_IsSestoSenso2)
+        setupRunPreset();
+
+    bool rc = updatePosition();
+    if (rc)
         IDSetNumber(&FocusAbsPosNP, nullptr);
 
-    if (updateMaxLimit() == false)
-        LOG_WARN("Check you have the latest SestoSenso firmware. Focuser requires calibration.");
-
-    return (rc1);
+    return (rc);
 }
 
 
@@ -656,15 +885,27 @@ bool SestoSenso2::Ack()
         }
         if(command->getSerialNumber(res))
         {
-            LOGF_INFO("Hello,I'm %s !", res);
+            LOGF_INFO("Serial number: %s", res);
         }
         else
         {
             return false;
         }
     }
-    IUSaveText(&FirmwareT[0], res);
 
+    m_IsSestoSenso2 = !strstr(res, "ESATTO");
+    IUSaveText(&FirmwareT[FIRMWARE_SN], res);
+
+    if (command->getFirmwareVersion(res))
+    {
+        LOGF_INFO("Firmware version: %s", res);
+        IUSaveText(&FirmwareT[FIRMWARE_VERSION], res);
+    }
+    else
+    {
+        return false;
+    }
+    
     return true;
 }
 
@@ -698,7 +939,7 @@ bool SestoSenso2::initCommandSet()
 
 
 bool CommandSet::sendCmd(std::string cmd, std::string property, char *res)
-{  
+{
     LOGF_DEBUG("Sending comand: %s with property: %s", cmd.c_str(), property.c_str());
     tcflush(CommandSet::PortFD, TCIOFLUSH);
     std::string cmd_str;
@@ -719,7 +960,7 @@ bool CommandSet::sendCmd(std::string cmd, std::string property, char *res)
         return false;
     }
     std::string response(read_buf);
-    if(CommandSet::getValueFromResponse(response, property, res) == CMD_FALSE)
+    if(CommandSet::getValueFromResponse(response, property, res) == false)
     {
         LOGF_ERROR("Communication error: cmd %s property %s resvalue: %s", cmd.c_str(), property.c_str(), res);
         return false;
@@ -743,17 +984,17 @@ bool CommandSet::getValueFromResponse(std::string response, std::string property
     std::size_t found = response.find(",");
     if(found != std::string::npos)
     {
-        response = response.substr(0,found);
+        response = response.substr(0, found);
     }
     else
     {
         found = response.find("}");
-        response = response.substr(0,found);
+        response = response.substr(0, found);
     }
-    response = removeChars(response,'\"');
-    response = removeChars(response,',');
-    response = removeChars(response,':');
-    strcpy(value,response.c_str());
+    response = removeChars(response, '\"');
+    response = removeChars(response, ',');
+    response = removeChars(response, ':');
+    strcpy(value, response.c_str());
 
     return true;
 }
@@ -763,7 +1004,7 @@ std::string CommandSet::removeChars(std::string str, char ch)
     std::size_t found = str.find(ch);
     if(found != std::string::npos)
     {
-        str = str.erase(found,1);
+        str = str.erase(found, 1);
         str = removeChars(str, ch);
     }
 
@@ -775,6 +1016,11 @@ bool CommandSet::getSerialNumber(char *res)
     return CommandSet::sendCmd("{\"req\":{\"get\":{\"SN\":\"\"}}}", "SN", res);
 }
 
+bool CommandSet::getFirmwareVersion(char* res)
+{
+    return CommandSet::sendCmd("{\"req\":{\"get\":{\"SWVERS\":\"\"}}}", "SWAPP", res);
+}
+
 bool CommandSet::abort()
 {
     return CommandSet::sendCmd("{\"req\":{\"cmd\":{\"MOT1\" :{\"MOT_ABORT\":\"\"}}}}");
@@ -783,8 +1029,8 @@ bool CommandSet::abort()
 bool CommandSet::go(uint32_t targetTicks, char *res)
 {
     char cmd[SESTO_LEN] = {0};
-    snprintf(cmd,sizeof(cmd),"{\"req\":{\"cmd\":{\"MOT1\" :{\"GOTO\":%u}}}}", targetTicks);
-    return CommandSet::sendCmd(cmd,"GOTO",res);
+    snprintf(cmd, sizeof(cmd), "{\"req\":{\"cmd\":{\"MOT1\" :{\"GOTO\":%u}}}}", targetTicks);
+    return CommandSet::sendCmd(cmd, "GOTO", res);
 }
 
 bool CommandSet::stop()
@@ -792,19 +1038,19 @@ bool CommandSet::stop()
     return CommandSet::sendCmd("{\"req\":{\"cmd\":{\"MOT1\" :{\"MOT_STOP\":\"\"}}}}");
 }
 
-bool CommandSet::goHome()
+bool CommandSet::goHome(char *res)
 {
-    return CommandSet::sendCmd("{\"req\":{\"cmd\":{\"MOT1\" :{\"GOHOME\":\"\"}}}}","GOHOME");
+    return CommandSet::sendCmd("{\"req\":{\"cmd\":{\"MOT1\" :{\"GOHOME\":\"\"}}}}", "GOHOME", res);
 }
 
-bool CommandSet::fastMoveOut()
+bool CommandSet::fastMoveOut(char *res)
 {
-    return CommandSet::sendCmd("{\"req\":{\"cmd\":{\"MOT1\" :{\"F_OUTW\":\"\"}}}}");
+    return CommandSet::sendCmd("{\"req\":{\"cmd\":{\"MOT1\" :{\"F_OUTW\":\"\"}}}}", "F_OUTW", res);
 }
 
-bool CommandSet::fastMoveIn()
+bool CommandSet::fastMoveIn(char *res)
 {
-    return CommandSet::sendCmd("{\"req\":{\"cmd\":{\"MOT1\" :{\"F_INW\":\"\"}}}}");
+    return CommandSet::sendCmd("{\"req\":{\"cmd\":{\"MOT1\" :{\"F_INW\":\"\"}}}}", "F_INW", res);
 }
 
 bool CommandSet::getMaxPosition(char *res)
@@ -812,6 +1058,11 @@ bool CommandSet::getMaxPosition(char *res)
     return CommandSet::sendCmd("{\"req\":{\"get\":{\"MOT1\":\"\"}}}", "CAL_MAXPOS", res);
 }
 
+bool CommandSet::getHallSensor(char *res)
+{
+    return CommandSet::sendCmd("{\"req\":{\"get\":{\"MOT1\":\"\"}}}", "HSENDET", res);
+}
+
 bool CommandSet::storeAsMaxPosition(char *res)
 {
     return CommandSet::sendCmd("{\"req\":{\"cmd\": {\"MOT1\": {\"CAL_FOCUSER\": \"StoreAsMaxPos\"}}}}"), res;
@@ -844,7 +1095,7 @@ bool CommandSet::getCurrentSpeed(char *res)
 
 bool CommandSet::loadSlowPreset(char *res)
 {
-    return sendCmd("{\"req\":{\"cmd\":{\"RUNPRESET\":\"slow\"}}}","RUNPRESET",res);
+    return sendCmd("{\"req\":{\"cmd\":{\"RUNPRESET\":\"slow\"}}}", "RUNPRESET", res);
 }
 
 bool CommandSet::getMotorTemp(char *res)
@@ -852,8 +1103,12 @@ bool CommandSet::getMotorTemp(char *res)
     return sendCmd("{\"req\":{\"get\":{\"MOT1\":\"\"}}}", "NTC_T", res);
 }
 
+bool CommandSet::getExternalTemp(char *res)
+{
+    return sendCmd("{\"req\":{\"get\":{\"EXT_T\":\"\"}}}", "EXT_T", res);
+}
 
-
-
-
-
+bool CommandSet::getVoltageIn(char *res)
+{
+    return sendCmd("{\"req\":{\"get\":{\"VIN_12V\":\"\"}}}", "VIN_12V", res);
+}
