@@ -122,6 +122,13 @@ bool SteelDriveII::initProperties()
     IUFillNumber(&TemperatureSensorN[TEMP_AVG], "TEMP_AVG", "Average (C)", "%.2f", -60, 60, 0, 0);
     IUFillNumberVector(&TemperatureSensorNP, TemperatureSensorN, 3, getDeviceName(), "TC_SENSOR", "Sensor", COMPENSATION_TAB, IP_RO, 60, IPS_IDLE);
 
+    // Stepper Drive
+    IUFillNumber(&StepperDriveN[CURRENT_MOVE], "STEPPER_DRIVE_CURRENT_MOVE", "Inverse Current Move", "%.f", 0, 127, 1, 25);
+    IUFillNumber(&StepperDriveN[CURRENT_HOLD], "STEPPER_DRIVE_CURRENT_HOLD", "Inverse Current Hold", "%.f", 0, 127, 1, 100);
+    IUFillNumberVector(
+        &StepperDriveNP, StepperDriveN, NARRAY(StepperDriveN),
+        getDeviceName(), "STEPPER_DRIVE", "Stepper Drive", OPTIONS_TAB, IP_RW, 60, IPS_IDLE
+    );
 
     addAuxControls();
     serialConnection->setDefaultBaudRate(Connection::Serial::B_19200);
@@ -148,6 +155,7 @@ bool SteelDriveII::updateProperties()
         defineSwitch(&TemperatureStateSP);
         defineNumber(&TemperatureSettingsNP);
         defineNumber(&TemperatureSensorNP);
+        defineNumber(&StepperDriveNP);
     }
     else
     {
@@ -158,6 +166,7 @@ bool SteelDriveII::updateProperties()
         deleteProperty(TemperatureStateSP.name);
         deleteProperty(TemperatureSettingsNP.name);
         deleteProperty(TemperatureSensorNP.name);
+        deleteProperty(StepperDriveNP.name);
     }
 
     return true;
@@ -334,6 +343,31 @@ bool SteelDriveII::ISNewNumber(const char *dev, const char *name, double values[
             IDSetNumber(&TemperatureSettingsNP, nullptr);
             return true;
         }
+
+        if (!strcmp(StepperDriveNP.name, name))
+        {
+            StepperDriveNP.s = IPS_OK;
+
+            if (StepperDriveN[CURRENT_MOVE].value != values[CURRENT_MOVE])
+            {
+                if (setParameter("CURRENT_MOVE", to_string(values[CURRENT_MOVE], 0)))
+                    StepperDriveN[CURRENT_MOVE].value = values[CURRENT_MOVE];
+                else
+                    StepperDriveNP.s = IPS_ALERT;
+            }
+
+            if (StepperDriveN[CURRENT_HOLD].value != values[CURRENT_HOLD])
+            {
+                if (setParameter("CURRENT_HOLD", to_string(values[CURRENT_HOLD], 0)))
+                    StepperDriveN[CURRENT_HOLD].value = values[CURRENT_HOLD];
+                else
+                    StepperDriveNP.s = IPS_ALERT;
+            }
+
+            IDSetNumber(&StepperDriveNP, nullptr);
+            return true;
+        }
+
     }
 
     return INDI::Focuser::ISNewNumber(dev, name, values, names, n);
@@ -541,6 +575,17 @@ void SteelDriveII::getStartupValues()
         TemperatureStateS[TC_PAUSED].s = (value == "0") ? ISS_OFF : ISS_ON;
     }
 
+    StepperDriveNP.s = IPS_OK;
+    if (getParameter("CURRENT_MOVE", value))
+        StepperDriveN[CURRENT_MOVE].value = std::stod(value);
+    else
+        StepperDriveNP.s = IPS_ALERT;
+
+    if (getParameter("CURRENT_HOLD", value))
+        StepperDriveN[CURRENT_HOLD].value = std::stod(value);
+    else
+        StepperDriveNP.s = IPS_ALERT;
+
     getSummary();
 
     FocusMaxPosN[0].value = std::stoul(m_Summary[LIMIT]);
@@ -563,7 +608,7 @@ bool SteelDriveII::getSummary()
         return false;
 
     std::vector<std::string> params = split(res, ";");
-    if (params.size() != 10)
+    if (params.size() < 10)
         return false;
 
     for (int i = 0; i < 10; i++)
