@@ -37,7 +37,8 @@ Version with experimental pulse guide support. GC 04.12.2015
 
 #include <sys/stat.h>
 
-# include "indilogger.h"
+#include "indilogger.h"
+#include "indiutility.h"
 
 //#include <time.h>
 
@@ -734,6 +735,8 @@ bool CelestronGPS::MoveWE(INDI_DIR_WE dir, TelescopeMotionCommand command)
 
 bool CelestronGPS::ReadScopeStatus()
 {
+    INDI::Telescope::TelescopePierSide pierSide = PIER_UNKNOWN;
+
     if (isSimulation())
         mountSim();
 
@@ -743,6 +746,52 @@ bool CelestronGPS::ReadScopeStatus()
         return false;
     }
 
+    if (HasPierSide())
+    {
+        // read the pier side close to reading the Radec so they should match
+        char sop;
+        char psc = 'u';
+        if (driver.get_pier_side(&sop))
+        {
+            // manage version and hemisphere nonsense
+            // HC versions less than 5.24 reverse the side of pier if the mount
+            // is in the Southern hemisphere.  StarSense doesn't
+            if (LocationN[LOCATION_LATITUDE].value < 0)
+            {
+                if (fwInfo.controllerVersion <= 5.24 && fwInfo.controllerVariant != ISSTARSENSE)
+                {
+                    // swap the char reported
+                    if (sop == 'E')
+                        sop = 'W';
+                    else if (sop == 'W')
+                        sop = 'E';
+                }
+            }
+            // The Celestron and INDI pointing states are opposite
+            if (sop == 'W')
+            {
+                pierSide = PIER_EAST;
+                psc = 'E';
+            }
+            else if (sop == 'E')
+            {
+                pierSide = PIER_WEST;
+                psc = 'W';
+            }
+            // pier side and Ha don't match at +-90 deg dec
+            if (currentDEC > 89.999 || currentDEC < -89.999)
+            {
+                pierSide = PIER_UNKNOWN;
+                psc = 'U';
+            }
+        }
+
+        LOGF_DEBUG("latitude %g, sop %c, PierSide %c",
+                   LocationN[LOCATION_LATITUDE].value,
+                   sop, psc);
+    }
+
+
     // aligning
     if (slewToIndex)
     {
@@ -839,6 +888,10 @@ bool CelestronGPS::ReadScopeStatus()
             break;
     }
 
+    // update pier side and RaDec close together to minimise the possibility of
+    // a mismatch causing an Ha limit error during a pier flip slew.
+    if (HasPierSide())
+        setPierSide(pierSide);
     NewRaDec(currentRA, currentDEC);
 
     // is PEC Handling required
@@ -964,46 +1017,6 @@ bool CelestronGPS::ReadScopeStatus()
         }
     }
 
-    if (!HasPierSide())
-        return true;
-
-    char sop;
-    INDI::Telescope::TelescopePierSide pierSide = PIER_UNKNOWN;
-    char psc = 'U';
-    if (driver.get_pier_side(&sop))
-    {
-        // manage version and hemisphere nonsense
-        // HC versions less than 5.24 reverse the side of pier if the mount
-        // is in the Southern hemisphere.  StarSense doesn't
-        if (LocationN[LOCATION_LATITUDE].value < 0)
-        {
-            if (fwInfo.controllerVersion <= 5.24 && fwInfo.controllerVariant != ISSTARSENSE)
-            {
-                // swap the char reported
-                if (sop == 'E')
-                    sop = 'W';
-                else if (sop == 'W')
-                    sop = 'E';
-            }
-        }
-        // The Celestron and INDI pointing states are opposite
-        if (sop == 'W')
-        {
-            pierSide = PIER_EAST;
-            psc = 'E';
-        }
-        else if (sop == 'E')
-        {
-            pierSide = PIER_WEST;
-            psc = 'W';
-        }
-    }
-
-    LOGF_DEBUG("latitude %g, sop %c, PierSide %c",
-               LocationN[LOCATION_LATITUDE].value,
-               sop, psc);
-    setPierSide(pierSide);
-
     return true;
 }
 
@@ -1270,6 +1283,12 @@ bool CelestronGPS::ISNewText(const char *dev, const char *name, char **texts, ch
 
         if (name && std::string(name) == "PEC_LOAD")
         {
+
+            IUUpdateText(&PecFileNameTP, texts, names, n);
+            IDSetText(&PecFileNameTP, nullptr);
+
+            LOGF_DEBUG("PEC Set %s", PecFileNameT[0].text);
+
             PecData pecData;
 
             // load from file
@@ -1969,31 +1988,6 @@ void CelestronGPS::checkAlignment()
         LOG_WARN("Mount is NOT aligned. You must align the mount first before you can use it. Disconnect, align the mount, and reconnect again.");
 }
 
-// Create dir recursively, reinventing - well copying - the wheel
-static int _mkdir(const char *dir, mode_t mode)
-{
-    char tmp[1024];
-    char *p = nullptr;
-    size_t len;
-
-    snprintf(tmp, sizeof(tmp), "%s", dir);
-    len = strlen(tmp);
-    if (tmp[len - 1] == '/')
-        tmp[len - 1] = 0;
-    for (p = tmp + 1; *p; p++)
-        if (*p == '/')
-        {
-            *p = 0;
-            if (mkdir(tmp, mode) == -1 && errno != EEXIST)
-                return -1;
-            *p = '/';
-        }
-    if (mkdir(tmp, mode) == -1 && errno != EEXIST)
-        return -1;
-
-    return 0;
-}
-
 bool CelestronGPS::savePecData()
 {
     // generate the file name:
@@ -2011,9 +2005,9 @@ bool CelestronGPS::savePecData()
     char dir[MAXRBUF];
     snprintf(dir, MAXRBUF, "%s/PEC_Data/%s", getenv("HOME"), ts_date);
 
-    if (_mkdir(dir, 0755) < 0)
+    if (INDI::mkpath(dir, 0755) == -1)
     {
-        LOGF_DEBUG("_mkdir %s failed", dir);
+        LOGF_ERROR("Error creating directory %s (%s)", dir, strerror(errno));
         return false;
     }
 
