@@ -94,7 +94,8 @@ IEQPro::IEQPro()
     DBG_SCOPE = INDI::Logger::getInstance().addDebugLevel("Scope Verbose", "SCOPE");
 
     SetTelescopeCapability(TELESCOPE_CAN_PARK | TELESCOPE_CAN_SYNC | TELESCOPE_CAN_GOTO | TELESCOPE_CAN_ABORT |
-                           TELESCOPE_HAS_TIME | TELESCOPE_HAS_LOCATION | TELESCOPE_HAS_TRACK_MODE | TELESCOPE_CAN_CONTROL_TRACK | TELESCOPE_HAS_TRACK_RATE,
+                           TELESCOPE_HAS_TIME | TELESCOPE_HAS_LOCATION | TELESCOPE_HAS_TRACK_MODE | TELESCOPE_CAN_CONTROL_TRACK |
+                           TELESCOPE_HAS_TRACK_RATE,
                            9);
 }
 
@@ -204,10 +205,13 @@ bool IEQPro::initProperties()
 
 bool IEQPro::updateProperties()
 {
-    INDI::Telescope::updateProperties();
 
     if (isConnected())
     {
+        getStartupData();
+
+        INDI::Telescope::updateProperties();
+
         // Remove find home if we do not support it.
         if (!canFindHome)
             HomeSP.nsp = 2;
@@ -224,11 +228,11 @@ bool IEQPro::updateProperties()
         defineSwitch(&GPSStatusSP);
         defineSwitch(&TimeSourceSP);
         defineSwitch(&HemisphereSP);
-
-        getStartupData();
     }
     else
     {
+        INDI::Telescope::updateProperties();
+
         HomeSP.nsp = 3;
         deleteProperty(HomeSP.name);
 
@@ -337,6 +341,16 @@ void IEQPro::getStartupData()
         SetAxis2ParkDefault(LocationN[LOCATION_LATITUDE].value);
     }
 
+    // can we read pier side?
+    IEQ_PIER_SIDE pierSide = IEQ_PIER_UNKNOWN;
+    if (driver->getPierSide(&pierSide) && pierSide != IEQ_PIER_UNKNOWN)
+    {
+        // add the pier side capability
+        auto cap = GetTelescopeCapability();
+        cap |= TELESCOPE_HAS_PIER_SIDE;
+        SetTelescopeCapability(cap, 9);
+    }
+
     //    if (isSimulation())
     //    {
     //        if (isParked())
@@ -403,6 +417,12 @@ bool IEQPro::ISNewSwitch(const char *dev, const char *name, ISState *states, cha
                     return true;
 
                 case IEQ_GOTO_HOME:
+                    if (TrackState == SCOPE_PARKED)
+                    {
+                        LOG_ERROR("Please unpark the mount before issuing any motion commands.");
+                        return false;
+                    }
+
                     if (driver->gotoHome() == false)
                     {
                         HomeSP.s = IPS_ALERT;
@@ -542,6 +562,29 @@ bool IEQPro::ReadScopeStatus()
         scopeInfo = newInfo;
     }
 
+    if (HasPierSide())
+    {
+        IEQ_PIER_SIDE pierSide;
+        if (driver->getPierSide(&pierSide))
+        {
+            TelescopePierSide tps = PIER_UNKNOWN;
+            switch (pierSide)
+            {
+                case IEQ_PIER_UNKNOWN:
+                case IEQ_PIER_UNCERTAIN:
+                    tps = PIER_UNKNOWN;
+                    break;
+                case IEQ_PIER_EAST:
+                    tps = PIER_EAST;
+                    break;
+                case IEQ_PIER_WEST:
+                    tps = PIER_WEST;
+                    break;
+            }
+            setPierSide(tps);
+        }
+    }
+
     rc = driver->getCoords(&currentRA, &currentDEC);
 
     if (rc)
@@ -688,14 +731,11 @@ bool IEQPro::Park()
     ln_hrz_posn horizontalPos;
     // Libnova south = 0, west = 90, north = 180, east = 270
 
-    horizontalPos.az = parkAz + 180;
-    if (horizontalPos.az > 360)
-        horizontalPos.az -= 360;
+    horizontalPos.az = parkAz;
     horizontalPos.alt = parkAlt;
-
     ln_equ_posn equatorialPos;
 
-    ln_get_equ_from_hrz(&horizontalPos, &observer, ln_get_julian_from_sys(), &equatorialPos);
+    get_equ_from_hrz(&horizontalPos, &observer, ln_get_julian_from_sys(), &equatorialPos);
 
     if (Goto(equatorialPos.ra / 15.0, equatorialPos.dec))
     {
@@ -1025,11 +1065,8 @@ bool IEQPro::SetCurrentPark()
     ln_equ_posn equatorialPos;
     equatorialPos.ra  = currentRA * 15;
     equatorialPos.dec = currentDEC;
-    ln_get_hrz_from_equ(&equatorialPos, &observer, ln_get_julian_from_sys(), &horizontalPos);
-
-    double parkAZ = horizontalPos.az - 180;
-    if (parkAZ < 0)
-        parkAZ += 360;
+    get_hrz_from_equ(&equatorialPos, &observer, ln_get_julian_from_sys(), &horizontalPos);
+    double parkAZ = horizontalPos.az;
     double parkAlt = horizontalPos.alt;
 
     char AzStr[16], AltStr[16];
