@@ -92,7 +92,7 @@ void ISSnoopDevice(XMLEle *root)
  */
 LX200_TeenAstro::LX200_TeenAstro()
 {
-    setVersion(1, 1);           // don't forget to update drivers.xml
+    setVersion(1, 2);           // don't forget to update drivers.xml
 
     DBG_SCOPE = INDI::Logger::getInstance().addDebugLevel("Scope Verbose", "SCOPE");
 
@@ -128,15 +128,6 @@ bool LX200_TeenAstro::initProperties()
     SetParkDataType(PARK_RA_DEC);
 
     // ============== MAIN_CONTROL_TAB
-    IUFillSwitch(&UsePulseCmdS[0], "On", "", ISS_ON);
-    IUFillSwitch(&UsePulseCmdS[1], "Off", "", ISS_OFF);
-    IUFillSwitchVector(&UsePulseCmdSP, UsePulseCmdS, 2, getDeviceName(), "Pulse Guiding", "", MAIN_CONTROL_TAB, IP_RW,
-                       ISR_1OFMANY, 0, IPS_IDLE);
-
-    IUFillSwitch(&RefractionS[0], "On", "", ISS_OFF);
-    IUFillSwitch(&RefractionS[1], "Off", "", ISS_ON);
-    IUFillSwitchVector(&RefractionSP, RefractionS, 2, getDeviceName(), "Refraction Tracking", "", MAIN_CONTROL_TAB, IP_RW,
-                       ISR_1OFMANY, 0, IPS_IDLE);
 
     // Tracking Mode
      AddTrackMode("TRACK_SIDEREAL", "Sidereal", true);
@@ -151,23 +142,13 @@ bool LX200_TeenAstro::initProperties()
     // ============== MOTION_TAB
     // Motion speed of axis when pressing NSWE buttons
     IUFillSwitch(&SlewRateS[0], "Guide", "Guide Speed", ISS_OFF);
-    IUFillSwitch(&SlewRateS[1], "0.5", "0.5x", ISS_OFF);
-    IUFillSwitch(&SlewRateS[2], "1", "1x", ISS_OFF);
-    IUFillSwitch(&SlewRateS[3], "2", "2x", ISS_OFF);
-    IUFillSwitch(&SlewRateS[4], "4", "4x", ISS_OFF);
-    IUFillSwitch(&SlewRateS[5], "16", "16x", ISS_OFF);
-    IUFillSwitch(&SlewRateS[6], "32", "32x", ISS_OFF);
-    IUFillSwitch(&SlewRateS[7], "64", "64x", ISS_ON);
-    IUFillSwitch(&SlewRateS[8], "0.5 Max", "0.5 Max", ISS_OFF);
-    IUFillSwitch(&SlewRateS[9], "Max", "Max", ISS_OFF);
-    IUFillSwitchVector(&SlewRateSP, SlewRateS, 10, getDeviceName(), "TELESCOPE_SLEW_RATE", "Centering Rate", 
+    IUFillSwitch(&SlewRateS[1], "Slow", "Slow", ISS_OFF);
+    IUFillSwitch(&SlewRateS[2], "Medium", "Medium", ISS_OFF);
+    IUFillSwitch(&SlewRateS[3], "Fast", "Fast", ISS_ON);
+    IUFillSwitch(&SlewRateS[4], "Max", "Max", ISS_OFF);
+    IUFillSwitchVector(&SlewRateSP, SlewRateS, 5, getDeviceName(), "TELESCOPE_SLEW_RATE", "Centering Rate", 
                         MOTION_TAB, IP_RW, ISR_1OFMANY, 0, IPS_IDLE);
 
-    IUFillNumber(&BacklashRAN[0], "Backlash DEC", "DE", "%g", 0, 999, 1, 15);
-    IUFillNumberVector(&BacklashRANP, BacklashRAN, 1, getDeviceName(), "Backlash RA", "", MOTION_TAB, IP_RW, 0, IPS_IDLE);
-    IUFillNumber(&BacklashDecN[0], "Backlash RA", "RA", "%g", 0, 999, 1, 15);
-    IUFillNumberVector(&BacklashDecNP, BacklashDecN, 1, getDeviceName(), "Backlash Dec", "", MOTION_TAB, IP_RW, 0, IPS_IDLE);
-
     // ============== GUIDE_TAB
     // Motion speed of axis when guiding
     IUFillSwitch(&GuideRateS[0], "25", "0.25x", ISS_OFF);
@@ -243,12 +224,6 @@ bool LX200_TeenAstro::updateProperties()
         // Motion Control
         defineSwitch(&SlewRateSP);
         defineSwitch(&GuideRateSP);
-//        defineSwitch(&FrequencyAdjustSP);
-//        defineNumber(&TrackFreqNP);
-        defineNumber(&BacklashRANP);
-        defineNumber(&BacklashDecNP);
-        defineSwitch(&UsePulseCmdSP);
-        defineSwitch(&RefractionSP);
 
         // Site Management
         defineSwitch(&ParkOptionSP);
@@ -275,10 +250,6 @@ bool LX200_TeenAstro::updateProperties()
         // Motion Control
         deleteProperty(SlewRateSP.name);
         deleteProperty(GuideRateSP.name);
-        deleteProperty(RefractionSP.name);
-        deleteProperty(BacklashRANP.name);
-        deleteProperty(BacklashDecNP.name);
-        deleteProperty(UsePulseCmdSP.name);
         deleteProperty(SiteSP.name);
         deleteProperty(SiteNameTP.name);
 
@@ -347,7 +318,7 @@ bool LX200_TeenAstro::ReadScopeStatus()
     }
 
     // update mount status
-    getCommandString(PortFD, OSStat, ":GU#");       // :GU# returns a string containg controller status
+    getCommandString(PortFD, OSStat, statusCommand);       // returns a string containg controller status
     if (OSStat[15] != '0')
     {
         updateMountStatus(OSStat[15]);              // error
@@ -371,6 +342,23 @@ void LX200_TeenAstro::handleStatusChange(void)
 {
     LOGF_DEBUG ("Status Change: %s", OSStat);        
 
+    if (OSStat[0] != OldOSStat[0])
+    {
+        if (OSStat[0] == '0')
+        {
+            TrackState = SCOPE_IDLE;
+        }
+        else if (OSStat[0] == '1')
+        {
+            TrackState = SCOPE_TRACKING;
+        }
+        else if (OSStat[0] == '2' || OSStat[0] == '3' )
+        {
+            TrackState = SCOPE_SLEWING;
+        }
+    }
+
+
     // Byte 2 is park status
     if (OSStat[2] != OldOSStat[2])
     {
@@ -381,7 +369,7 @@ void LX200_TeenAstro::handleStatusChange(void)
         else
         {
             SetParked(false);
-            SetTrackEnabled(false);     //disable since TeenAstro enables it by default            
+//            SetTrackEnabled(false);     //disable since TeenAstro enables it by default            
         }
     }
     // Byte 13 is pier side
@@ -498,8 +486,6 @@ bool LX200_TeenAstro::SetTrackMode(uint8_t mode)
 
     bool rc = (selectTrackingMode(PortFD, mode) == 0);
 
-//    getTrackFreq(PortFD, &TrackFreqN[0].value);
-//    IDSetNumber(&TrackingFreqNP, nullptr);
     return rc;
 }
 
@@ -630,46 +616,14 @@ bool LX200_TeenAstro::Park()
 
 
 /*
- *  updateLocation: standard LX200 commands
+ *  updateLocation: not used - use hand controller to update
  */
 bool LX200_TeenAstro::updateLocation(double latitude, double longitude, double elevation)
 {
-    if (isSimulation())
-        return true;
+    INDI_UNUSED(latitude);
+    INDI_UNUSED(longitude);
+    INDI_UNUSED(elevation);
 
-    // We count longitude as positive to the east, Meade is positive to the west.
-    double onstep_long = 360 - longitude;
-    while (onstep_long < 0)
-        onstep_long += 360;
-    while (onstep_long > 360)
-        onstep_long -= 360;
-
-#if 0
-    // Don't allow the driver to change the mount's predefined sites - too confusing...
-    if (setSiteLongitude(PortFD, onstep_long) < 0)
-    {
-        LOG_ERROR("Error setting site longitude coordinates");
-        return false;
-    }
-
-    if (setSiteLatitude(PortFD, latitude) < 0)
-    {
-        LOG_ERROR("Error setting site latitude coordinates");
-        return false;
-    }
-
-    if (!setSiteElevation(elevation))
-    {
-        LOG_ERROR("Error setting site elevation");
-        return false;
-    }
-#endif
-
-    char l[32] = {0}, L[32] = {0};
-    fs_sexa(l, latitude, 3, 3600);
-    fs_sexa(L, longitude, 4, 3600);
-
-    LOGF_INFO("Site location updated to Lat %.32s - Long %.32s - Elev. %d", l, L, elevation);
     return true;
 }
 
@@ -678,19 +632,19 @@ bool LX200_TeenAstro::updateLocation(double latitude, double longitude, double e
  */
 void LX200_TeenAstro::getBasicData()
 {
-    char OSbacklashDEC[RB_MAX_LEN];
-    char OSbacklashRA[RB_MAX_LEN];
     int currentSiteIndex, slewRateIndex;
 
     if (!isSimulation())
     {
-        checkLX200Format(PortFD);
+        checkLX200EquatorialFormat(PortFD);
         char buffer[128];
         getVersionDate(PortFD, buffer);
         IUSaveText(&VersionT[0], buffer);
         getVersionTime(PortFD, buffer);
         IUSaveText(&VersionT[1], buffer);
         getVersionNumber(PortFD, buffer);
+        statusCommand = ":GXI#";
+        guideSpeedCommand = ":SXR0:%s#";
         IUSaveText(&VersionT[2], buffer);
         getProductName(PortFD, buffer);
         IUSaveText(&VersionT[3], buffer);
@@ -718,10 +672,11 @@ void LX200_TeenAstro::getBasicData()
         // Get initial state and set switches
         for (unsigned i=0;i<sizeof(OldOSStat);i++)
             OldOSStat[i] = 'x';                         // reset old OS stat to force re-evaluation
-        getCommandString(PortFD, OSStat, ":GU#");       // :GU# returns a string containg controller status
+        getCommandString(PortFD, OSStat, statusCommand);       // returns a string containing controller status
         handleStatusChange();
         LOGF_INFO("Initial Status: %s", OSStat);
-       // get current slew rate
+
+        // get current slew rate
         if (getSlewRate(&slewRateIndex))
         {
             LOGF_INFO("current slew rate : %d", slewRateIndex);
@@ -734,36 +689,12 @@ void LX200_TeenAstro::getBasicData()
              LOG_ERROR("Error reading current slew rate");
         }
 
-        // Get actual Backlash values
-        getCommandString(PortFD, OSbacklashDEC, ":%BD#");
-        getCommandString(PortFD, OSbacklashRA, ":%BR#");
-        BacklashDecNP.np[0].value = atof(OSbacklashDEC);
-        BacklashRANP.np[0].value = atof(OSbacklashRA);
-        BacklashRANP.s = IPS_OK;
-        BacklashDecNP.s = IPS_OK;
-        IDSetNumber(&BacklashDecNP, nullptr);
-        IDSetNumber(&BacklashRANP, nullptr);
-
         // Turn off tracking. (too much interaction with telescope.cpp if we try to keep the mount's current track state)
-        SetTrackEnabled(false);
+        if (TrackState != SCOPE_TRACKING)
+        {
+            SetTrackEnabled(false);
+        }
 
-        // Following parameters are read from config file and restored
-        loadConfig(true, RefractionSP.name);
-        bool refrac = IUFindOnSwitchIndex(&RefractionSP);
-        enableRefractionTracking(refrac);
-        IDSetSwitch(&RefractionSP, nullptr);
-
-        // Set pulse guiding 
-        loadConfig(true, UsePulseCmdSP.name);
-        usePulseCommand = IUFindOnSwitchIndex(&UsePulseCmdSP);
-        IDSetSwitch(&UsePulseCmdSP, nullptr);
-
-        // set guide rate 
-        loadConfig(true, GuideRateSP.name);
-        int index = IUFindOnSwitchIndex(&GuideRateSP);
-        SetGuideRate(index);
-        GuideRateSP.s = IPS_OK;
-        IDSetSwitch(&GuideRateSP, nullptr);
  
         if (InitPark())
         {
@@ -801,34 +732,6 @@ bool LX200_TeenAstro::ISNewNumber(const char *dev, const char *name, double valu
             IDSetNumber(&SlewAccuracyNP, nullptr);
             return true;
         }
-        if (!strcmp(name, BacklashRANP.name))
-        {
-            char cmd[20];
-
-            IUUpdateNumber(&BacklashRANP, values, names, n);
-            INumber *bklshra = IUFindNumber(&BacklashRANP, names[0]);
-            snprintf(cmd, sizeof(cmd), ":$BR%d#",  static_cast<int>(bklshra->value));
-            sendCommand(cmd);
-
-            BacklashRANP.np[0].value = bklshra->value;
-            BacklashRANP.s           = IPS_OK;
-            IDSetNumber(&BacklashRANP, nullptr);
-            return true;
-        }
-        if (!strcmp(name, BacklashDecNP.name))
-        {
-            char cmd[20];
-
-            IUUpdateNumber(&BacklashDecNP, values, names, n);
-            INumber *bklshdec = IUFindNumber(&BacklashDecNP, names[0]);
-            snprintf(cmd, sizeof(cmd), ":$BD%d#", static_cast<int>(bklshdec->value));
-            sendCommand(cmd);
-
-            BacklashDecNP.np[0].value = bklshdec->value;
-            BacklashDecNP.s           = IPS_OK;
-            IDSetNumber(&BacklashDecNP, nullptr);
-            return true;
-        }
 
         // GUIDE process Guider properties.
         processGuiderProperties(name, values, names, n);
@@ -874,30 +777,6 @@ bool LX200_TeenAstro::ISNewSwitch(const char *dev, const char *name, ISState *st
             GuideRateSP.s = IPS_OK;
             SetGuideRate(index);
             IDSetSwitch(&GuideRateSP, nullptr);
-        }
-        // Pulse-Guide command support
-        if (!strcmp(name, UsePulseCmdSP.name))
-        {
-            IUResetSwitch(&UsePulseCmdSP);
-            IUUpdateSwitch(&UsePulseCmdSP, states, names, n);
-
-            usePulseCommand = (UsePulseCmdS[0].s == ISS_ON);
-            UsePulseCmdSP.s = usePulseCommand ? IPS_OK : IPS_IDLE;
-            IDSetSwitch(&UsePulseCmdSP, nullptr);
-            LOGF_INFO("Pulse guiding is %s.", usePulseCommand ? "enabled" : "disabled");
-            return true;
-        }
-        // Refraction tracking
-        if (!strcmp(name, RefractionSP.name))
-        {
-            IUResetSwitch(&RefractionSP);
-            IUUpdateSwitch(&RefractionSP, states, names, n);
-
-            bool refraction = (RefractionS[0].s == ISS_ON);
-            RefractionSP.s = refraction ? IPS_OK : IPS_IDLE;
-            IDSetSwitch(&RefractionSP, nullptr);
-            enableRefractionTracking(refraction);
-            return (refraction);
         }
          // Sites
         if (!strcmp(name, SiteSP.name))
@@ -1098,6 +977,7 @@ bool LX200_TeenAstro::sendScopeTime()
 bool LX200_TeenAstro::sendScopeLocation()
 {
     int dd = 0, mm = 0, elev = 0;
+    double ssf = 0.0;
 
     LOG_INFO("Send location");
     return true;
@@ -1112,7 +992,7 @@ bool LX200_TeenAstro::sendScopeLocation()
         return true;
     }
 
-    if (getSiteLatitude(PortFD, &dd, &mm) < 0)
+    if (getSiteLatitude(PortFD, &dd, &mm, &ssf) < 0)
     {
         LOG_WARN("Failed to get site latitude from device.");
         return false;
@@ -1124,7 +1004,7 @@ bool LX200_TeenAstro::sendScopeLocation()
         else
             LocationNP.np[LOCATION_LATITUDE].value = dd - mm / 60.0;
     }
-    if (getSiteLongitude(PortFD, &dd, &mm) < 0)
+    if (getSiteLongitude(PortFD, &dd, &mm, &ssf) < 0)
     {
         LOG_WARN("Failed to get site longitude from device.");
         return false;
@@ -1175,19 +1055,12 @@ bool LX200_TeenAstro::getSiteIndex(int *ndxP)
 
 /*
  * getSlewRate - not in Meade standard
- * uses character 4 in string returned by :GU#
- * returns the index in table SlewRateS[]
  */
 bool LX200_TeenAstro::getSlewRate(int *ndxP)
 {
-//    getCommandString(PortFD, OSStat, ":GU#");
-    char code = OSStat[4];
-    if (code >='0' && code <= '9')
-    {
-        *ndxP = OSStat[4] - '0';
-        return true;
-    }    
-    return false;
+    if (getCommandInt(PortFD, ndxP, ":GXRD#") !=0)
+        return false;
+    return true;
 }
 
 /*
@@ -1222,8 +1095,9 @@ bool LX200_TeenAstro::setSiteElevation(double elevation)
 bool LX200_TeenAstro::getLocation()
 {
     int dd = 0, mm = 0, elev = 0;
+    double ssf = 0.0;
 
-    if (getSiteLatitude(PortFD, &dd, &mm) < 0)
+    if (getSiteLatitude(PortFD, &dd, &mm, &ssf) < 0)
     {
         LOG_WARN("Failed to get site latitude from device.");
         return false;
@@ -1236,7 +1110,7 @@ bool LX200_TeenAstro::getLocation()
             LocationNP.np[LOCATION_LATITUDE].value = dd - mm / 60.0;
     }
 
-    if (getSiteLongitude(PortFD, &dd, &mm) < 0)
+    if (getSiteLongitude(PortFD, &dd, &mm, &ssf) < 0)
     {
         LOG_WARN("Failed to get site longitude from device.");
         return false;
@@ -1262,13 +1136,13 @@ bool LX200_TeenAstro::getLocation()
 }
 
 /*
- * Set Guide Rate - OnStep command :SX90:ddd# where ddd is guide rate * 100
+ * Set Guide Rate -  :SXR0:dddd# (v1.2 and above) where ddd is guide rate * 100
  */
 bool LX200_TeenAstro::SetGuideRate(int index)
 {
     char cmdString[20];
 
-    snprintf (cmdString, sizeof(cmdString), ":SX90:%s#", GuideRateS[index].name);  // GuideRateS is {25,50,100}
+    snprintf (cmdString, sizeof(cmdString), guideSpeedCommand, GuideRateS[index].name);  // GuideRateS is {25,50,100}
     sendCommand(cmdString);
 
     return true;
@@ -1280,42 +1154,26 @@ bool LX200_TeenAstro::SetGuideRate(int index)
  */
 IPState LX200_TeenAstro::GuideNorth(uint32_t ms)
 {
-    if (usePulseCommand)
-    {
-        SendPulseCmd(LX200_NORTH, ms);
-        return IPS_OK;
-    }
-    return IPS_ALERT;
+    SendPulseCmd(LX200_NORTH, ms);
+    return IPS_OK;
 }
 
 IPState LX200_TeenAstro::GuideSouth(uint32_t ms)
 {
-    if (usePulseCommand)
-    {
-        SendPulseCmd(LX200_SOUTH, ms);
-        return IPS_OK;
-    }
-    return IPS_ALERT;
+    SendPulseCmd(LX200_SOUTH, ms);
+    return IPS_OK;
 }
 
 IPState LX200_TeenAstro::GuideEast(uint32_t ms)
 {
-    if (usePulseCommand)
-    {
-        SendPulseCmd(LX200_EAST, ms);
-        return IPS_OK;
-    }
-    return IPS_ALERT;
+    SendPulseCmd(LX200_EAST, ms);
+    return IPS_OK;
 }
 
 IPState LX200_TeenAstro::GuideWest(uint32_t ms)
 {
-    if (usePulseCommand)
-    {
-        SendPulseCmd(LX200_WEST, ms);
-        return IPS_OK;
-    }
-    return IPS_ALERT;
+    SendPulseCmd(LX200_WEST, ms);
+    return IPS_OK;
 }
 
 void LX200_TeenAstro::SendPulseCmd(int8_t direction, uint32_t duration_msec)
@@ -1379,10 +1237,8 @@ bool LX200_TeenAstro::Move(TDirection dir, TelescopeMotionCommand cmd)
  */
 bool LX200_TeenAstro::saveConfigItems(FILE *fp)
 {
-    IUSaveConfigSwitch(fp, &UsePulseCmdSP);
     IUSaveConfigSwitch(fp, &SlewRateSP);
     IUSaveConfigSwitch(fp, &GuideRateSP);
-    IUSaveConfigSwitch(fp, &RefractionSP);
 
     return INDI::Telescope::saveConfigItems(fp);
 }
@@ -1487,33 +1343,17 @@ bool LX200_TeenAstro::SetTrackEnabled(bool enabled)
 }
 
 /*
- * selectSlewrate - select among TeenAstro's 10 predefined rates
+ * selectSlewrate - select among TeenAstro's 5 predefined rates
  */
 bool LX200_TeenAstro::selectSlewRate(int index)
 {
     char cmd[20];
 
-    snprintf(cmd, sizeof(cmd), ":R%d#", index);
+    snprintf(cmd, sizeof(cmd), ":SXRD:%d#", index);
     sendCommand(cmd);
     return true;
 }
 
-/*
- * enable Refraction tracking
- */
-void LX200_TeenAstro::enableRefractionTracking(bool enable)
-{
-    if (enable)
-    {
-        sendCommand(":Tr#");
-        LOG_INFO("Refraction tracking enabled");
-    }
-    else
-    {
-        sendCommand(":Tn#");
-        LOG_INFO("Refraction tracking disabled");
-    }
-}
 
 /*
  * Used instead of getCommandString when response is not terminated with '#'
@@ -1528,7 +1368,6 @@ void LX200_TeenAstro::sendCommand(const char *cmd)
     int rc = write(PortFD, cmd, strlen(cmd));
     rc = tty_read(PortFD, &resp, 1, ONSTEP_TIMEOUT, &nbytes_read);
     INDI_UNUSED(rc);
-    tcflush(PortFD, TCIFLUSH);
 }
 
 
