@@ -39,7 +39,8 @@
 LX200AstroPhysics::LX200AstroPhysics() : LX200Generic()
 {
     setLX200Capability(LX200_HAS_PULSE_GUIDING);
-    SetTelescopeCapability(GetTelescopeCapability() | TELESCOPE_HAS_PIER_SIDE | TELESCOPE_HAS_PEC | TELESCOPE_CAN_CONTROL_TRACK | TELESCOPE_HAS_TRACK_RATE, 4);
+    SetTelescopeCapability(GetTelescopeCapability() | TELESCOPE_HAS_PIER_SIDE | TELESCOPE_HAS_PEC | TELESCOPE_CAN_CONTROL_TRACK
+                           | TELESCOPE_HAS_TRACK_RATE, 4);
 
     sendLocationOnStartup = false;
     sendTimeOnStartup = false;
@@ -84,7 +85,8 @@ bool LX200AstroPhysics::initProperties()
     IUFillSwitch(&SlewRateS[1], "64", "64x", ISS_ON);
     IUFillSwitch(&SlewRateS[2], "600", "600x", ISS_OFF);
     IUFillSwitch(&SlewRateS[3], "1200", "1200x", ISS_OFF);
-    IUFillSwitchVector(&SlewRateSP, SlewRateS, 4, getDeviceName(), "TELESCOPE_SLEW_RATE", "Slew Rate", MOTION_TAB, IP_RW, ISR_1OFMANY, 0, IPS_IDLE);
+    IUFillSwitchVector(&SlewRateSP, SlewRateS, 4, getDeviceName(), "TELESCOPE_SLEW_RATE", "Slew Rate", MOTION_TAB, IP_RW,
+                       ISR_1OFMANY, 0, IPS_IDLE);
 
     // Slew speed when performing regular GOTO
     IUFillSwitch(&APSlewSpeedS[0], "600", "600x", ISS_ON);
@@ -549,7 +551,7 @@ bool LX200AstroPhysics::setBasicDataPart0()
     }
 
     // Detect and set fomat. It should be LONG.
-    checkLX200Format(PortFD);
+    checkLX200EquatorialFormat(PortFD);
 
     return true;
 }
@@ -867,9 +869,7 @@ bool LX200AstroPhysics::Park()
         ln_hrz_posn horizontalPos;
         // Libnova south = 0, west = 90, north = 180, east = 270
 
-        horizontalPos.az = parkAz + 180;
-        if (horizontalPos.az > 360)
-            horizontalPos.az -= 360;
+        horizontalPos.az = parkAz;
         horizontalPos.alt = parkAlt;
 
         ln_equ_posn equatorialPos;
@@ -936,14 +936,12 @@ bool LX200AstroPhysics::UnPark()
         ln_hrz_posn horizontalPos;
         // Libnova south = 0, west = 90, north = 180, east = 270
 
-        horizontalPos.az = parkAz + 180;
-        if (horizontalPos.az > 360)
-            horizontalPos.az -= 360;
+        horizontalPos.az = parkAz;
         horizontalPos.alt = parkAlt;
 
         ln_equ_posn equatorialPos;
 
-        ln_get_equ_from_hrz(&horizontalPos, &observer, ln_get_julian_from_sys(), &equatorialPos);
+        get_equ_from_hrz(&horizontalPos, &observer, ln_get_julian_from_sys(), &equatorialPos);
 
         currentRA = equatorialPos.ra / 15.0;
         currentDEC = equatorialPos.dec;
@@ -982,11 +980,8 @@ bool LX200AstroPhysics::SetCurrentPark()
     ln_equ_posn equatorialPos;
     equatorialPos.ra  = currentRA * 15;
     equatorialPos.dec = currentDEC;
-    ln_get_hrz_from_equ(&equatorialPos, &observer, ln_get_julian_from_sys(), &horizontalPos);
-
-    double parkAZ = horizontalPos.az - 180;
-    if (parkAZ < 0)
-        parkAZ += 360;
+    get_hrz_from_equ(&equatorialPos, &observer, ln_get_julian_from_sys(), &horizontalPos);
+    double parkAZ = horizontalPos.az;
     double parkAlt = horizontalPos.alt;
 
     char AzStr[16], AltStr[16];
