@@ -46,12 +46,12 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110 - 1301  USA
 #define RB_MAX_LEN    64
 
 
-int controller_format;
+int eq_format; /* For possible values see enum TEquatorialFormat */
+int geo_format = LX200_GEO_SHORT_FORMAT; /* For possible values see enum TGeographicFormat */
 char lx200Name[MAXINDIDEVICE];
 /* ESN DEBUG */
 unsigned int DBG_SCOPE = 8;
 
-
 /* Add mutex to communications */
 std::mutex lx200CommsLock;
 
@@ -80,9 +80,9 @@ int getCommandInt(int fd, int *value, const char *cmd);
 /* Get tracking frequency */
 int getTrackFreq(int fd, double *value);
 /* Get site Latitude */
-int getSiteLatitude(int fd, int *dd, int *mm);
+int getSiteLatitude(int fd, int *dd, int *mm, double *ssf);
 /* Get site Longitude */
-int getSiteLongitude(int fd, int *ddd, int *mm);
+int getSiteLongitude(int fd, int *ddd, int *mm, double *ssf);
 /* Get Calender data */
 int getCalendarDate(int fd, char *date);
 /* Get site Name */
@@ -119,7 +119,7 @@ int setUTCOffset(int fd, double hours);
 /* Set Track Freq */
 int setTrackFreq(int fd, double trackF);
 /* Set current site longitude */
-int setSiteLongitude(int fd, double Long);
+int setSiteLongitude(int fd, double CartographicLongitude);
 /* Set current site latitude */
 int setSiteLatitude(int fd, double Lat);
 /* Set Object Azimuth */
@@ -161,9 +161,11 @@ int SendPulseCmd(int fd, int direction, uint32_t duration_msec);
  Other Commands
  **************************************************************************/
 /* Determines LX200 RA/DEC format, tries to set to long if found short */
-int checkLX200Format(int fd);
-/* return the controller_format enum value */
-int getLX200Format();
+int checkLX200EquatorialFormat(int fd);
+/* return the eq_format enum value */
+int getLX200EquatorialFormat();
+/* return the geo_format enum value */
+int getLX200GeographicFormat();
 /* Select a site from the LX200 controller */
 int selectSite(int fd, int siteNum);
 /* Select a catalog object */
@@ -534,8 +536,17 @@ int getSiteName(int fd, char *siteName, int siteNum)
     return 0;
 }
 
-int getSiteLatitude(int fd, int *dd, int *mm)
+int getSiteLatitude(int fd, int *dd, int *mm, double *ssf)
 {
+    // :Gt# from 10Micron docs explaining the extensions to the standard LX200 protocol.
+    // Get current site latitude.
+    // Returns the latitude of the current site formatted as follows:
+    // Emulation and precision              Return value
+    // Any emulation, low precision         sDD*MM# (sign, degrees, minutes)
+    // LX200 emulation, high precision      sDD*MM# (sign, degrees, minutes)
+    // Extended emulation, high precision   sDD*MM:SS# (sign, degrees, arcminutes, arcseconds)
+    // Any emulation, ultra precision       sDD:MM:SS.S# (sign, degrees, arcminutes, arcseconds, tenths of arcsecond)
+    // Positive implies north latitude.
     DEBUGFDEVICE(lx200Name, DBG_SCOPE, "<%s>", __FUNCTION__);
     char read_buffer[RB_MAX_LEN]={0};
     int error_type;
@@ -562,19 +573,55 @@ int getSiteLatitude(int fd, int *dd, int *mm)
 
     DEBUGFDEVICE(lx200Name, DBG_SCOPE, "RES <%s>", read_buffer);
 
-    if (sscanf(read_buffer, "%d%*c%d", dd, mm) < 2)
+    *ssf = 0.0;
+    if (sscanf(read_buffer, "%d%*c%d:%lf", dd, mm, ssf) < 2)
     {
-        DEBUGDEVICE(lx200Name, DBG_SCOPE, "Unable to parse response");
+        DEBUGDEVICE(lx200Name, DBG_SCOPE, "Unable to parse :Gt# response");
         return -1;
     }
 
-    DEBUGFDEVICE(lx200Name, DBG_SCOPE, "VAL [%d,%d]", *dd, *mm);
+    DEBUGFDEVICE(lx200Name, DBG_SCOPE, "VAL [%d,%d,%.1lf]", *dd, *mm, *ssf);
+
+    int new_geo_format;
+    switch (nbytes_read) {
+    case 9:
+    case 10:
+        new_geo_format = LX200_GEO_LONG_FORMAT;
+        break;
+    case 11:
+    case 12:
+        new_geo_format = LX200_GEO_LONGER_FORMAT;
+        break;
+    default:
+        new_geo_format = LX200_GEO_SHORT_FORMAT;
+        break;
+    }
+    if (new_geo_format != geo_format)
+    {
+        DEBUGFDEVICE(lx200Name, DBG_SCOPE, "Updated geographic precision from setting %d to %d", geo_format, new_geo_format);
+        geo_format = new_geo_format;
+    }
 
     return 0;
 }
 
-int getSiteLongitude(int fd, int *ddd, int *mm)
+// Meade classic handset defines longitude as 0 to 360 WESTWARD. However,
+// Meade API expresses East Longitudes as negative, West Longitudes as positive.
+// Source: https://www.meade.com/support/LX200CommandSet.pdf from 2002 at :Gg#
+// (And also 10Micron has East Longitudes expressed as negative.)
+// Also note that this is the opposite of cartography where East is positive.
+int getSiteLongitude(int fd, int *ddd, int *mm, double *ssf)
 {
+    // :Gg# from 10Micron docs explaining the extensions to the standard LX200 protocol.
+    // Get current site longitude.
+    // Note: East Longitudes are expressed as negative.
+    // Returns the current site longitude formatted as follows:
+    // Emulation and precision               Return value
+    // Any emulation, low precision or LX200 sDDD*MM# (sign, degrees, arcminutes)
+    //  emulation, high precision
+    // Extended emulation, high precision    sDDD*MM:SS# (sign, degrees, arcminutes, arcseconds)
+    // Any emulation, ultra precision        sDDD:MM:SS.S# (sign, degrees, arcminutes, arcseconds, tenths of arcsecond)
+
     DEBUGFDEVICE(lx200Name, DBG_SCOPE, "<%s>", __FUNCTION__);
     char read_buffer[RB_MAX_LEN]={0};
     int error_type;
@@ -599,13 +646,35 @@ int getSiteLongitude(int fd, int *ddd, int *mm)
 
     DEBUGFDEVICE(lx200Name, DBG_SCOPE, "RES <%s>", read_buffer);
 
-    if (sscanf(read_buffer, "%d%*c%d", ddd, mm) < 2)
+    *ssf = 0.0;
+    if (sscanf(read_buffer, "%d%*c%d:%lf", ddd, mm, ssf) < 2)
     {
-        DEBUGDEVICE(lx200Name, DBG_SCOPE, "Unable to parse response");
+        DEBUGDEVICE(lx200Name, DBG_SCOPE, "Unable to parse :Gg# response");
         return -1;
     }
+    *ddd *= -1.0; // Convert LX200Longitude to CartographicLongitude
 
-    DEBUGFDEVICE(lx200Name, DBG_SCOPE, "VAL [%d,%d]", *ddd, *mm);
+    DEBUGFDEVICE(lx200Name, DBG_SCOPE, "VAL in CartographicLongitude format [%d,%d,%.1lf]", *ddd, *mm, *ssf);
+
+    int new_geo_format;
+    switch (nbytes_read) {
+    case 10:
+    case 11:
+        new_geo_format = LX200_GEO_LONG_FORMAT;
+        break;
+    case 12:
+    case 13:
+        new_geo_format = LX200_GEO_LONGER_FORMAT;
+        break;
+    default:
+        new_geo_format = LX200_GEO_SHORT_FORMAT;
+        break;
+    }
+    if (new_geo_format != geo_format)
+    {
+        DEBUGFDEVICE(lx200Name, DBG_SCOPE, "Updated geographic precision from setting %d to %d", geo_format, new_geo_format);
+        geo_format = new_geo_format;
+    }
 
     return 0;
 }
@@ -841,25 +910,25 @@ int setObjectRA(int fd, double ra)
 /* Add mutex */
 /*    std::unique_lock<std::mutex> guard(lx200CommsLock);  */
 
-    switch (controller_format)
+    switch (eq_format)
     {
-        case LX200_LONG_FORMAT:
+        case LX200_EQ_LONG_FORMAT:
             getSexComponents(ra, &h, &m, &s);
             snprintf(read_buffer, sizeof(read_buffer), ":Sr %02d:%02d:%02d#", h, m, s);
             break;
-        case LX200_LONGER_FORMAT:
+        case LX200_EQ_LONGER_FORMAT:
             double d_s;
             getSexComponentsIID(ra, &h, &m, &d_s);
             snprintf(read_buffer, sizeof(read_buffer), ":Sr %02d:%02d:%05.02f#", h, m, d_s);
             break;
-        case LX200_SHORT_FORMAT:
+        case LX200_EQ_SHORT_FORMAT:
             int frac_m;
             getSexComponents(ra, &h, &m, &s);
             frac_m = (s / 60.0) * 10.;
             snprintf(read_buffer, sizeof(read_buffer), ":Sr %02d:%02d.%01d#", h, m, frac_m);
             break;
         default:
-            DEBUGFDEVICE(lx200Name, DBG_SCOPE, "Unknown controller_format <%d>", controller_format);
+            DEBUGFDEVICE(lx200Name, DBG_SCOPE, "Unknown controller_format <%d>", eq_format);
             return -1;            
     }
 
@@ -876,9 +945,9 @@ int setObjectDEC(int fd, double dec)
     int d, m, s;
     char read_buffer[22];
 
-    switch (controller_format)
+    switch (eq_format)
     {
-        case LX200_LONG_FORMAT:
+        case LX200_EQ_LONG_FORMAT:
             getSexComponents(dec, &d, &m, &s);
             /* case with negative zero */
             if (!d && dec < 0)
@@ -886,7 +955,7 @@ int setObjectDEC(int fd, double dec)
             else
                 snprintf(read_buffer, sizeof(read_buffer), ":Sd %+03d:%02d:%02d#", d, m, s);
             break;
-        case LX200_LONGER_FORMAT:
+        case LX200_EQ_LONGER_FORMAT:
             double d_s;
             getSexComponentsIID(dec, &d, &m, &d_s);
             /* case with negative zero */
@@ -895,7 +964,7 @@ int setObjectDEC(int fd, double dec)
             else
                 snprintf(read_buffer, sizeof(read_buffer), ":Sd %+03d:%02d:%05.02f#", d, m, d_s);
             break;
-        case LX200_SHORT_FORMAT:
+        case LX200_EQ_SHORT_FORMAT:
             getSexComponents(dec, &d, &m, &s);
             /* case with negative zero */
             if (!d && dec < 0)
@@ -904,7 +973,7 @@ int setObjectDEC(int fd, double dec)
                 snprintf(read_buffer, sizeof(read_buffer), ":Sd %+03d*%02d#", d, m);
             break;
         default:
-            DEBUGFDEVICE(lx200Name, DBG_SCOPE, "Unknown controller_format <%d>", controller_format);
+            DEBUGFDEVICE(lx200Name, DBG_SCOPE, "Unknown controller_format <%d>", eq_format);
             return -1;            
     }
 
@@ -1016,19 +1085,38 @@ int setUTCOffset(int fd, double hours)
     return (setStandardProcedure(fd, read_buffer));
 }
 
-// Meade defines longitude as 0 to 360 WESTWARD
-int setSiteLongitude(int fd, double Long)
+// Meade classic handset defines longitude as 0 to 360 WESTWARD. However,
+// Meade API expresses East Longitudes as negative, West Longitudes as positive.
+// Source: https://www.meade.com/support/LX200CommandSet.pdf from 2002 at :Gg#
+// (And also 10Micron has East Longitudes expressed as negative.)
+// Also note that this is the opposite of cartography where East is positive.
+int setSiteLongitude(int fd, double CartographicLongitude)
 {
     DEBUGFDEVICE(lx200Name, DBG_SCOPE, "<%s>", __FUNCTION__);
     int d, m, s;
     char read_buffer[RB_MAX_LEN]={0};
+    double LX200Longitude = -1.0 * CartographicLongitude;
 
 /* Add mutex */
 /*    std::unique_lock<std::mutex> guard(lx200CommsLock); */
-
-    getSexComponents(Long, &d, &m, &s);
-
-    snprintf(read_buffer, sizeof(read_buffer), ":Sg%03d:%02d#", d, m);
+    switch (geo_format) {
+    case LX200_GEO_SHORT_FORMAT: // d m
+        getSexComponents(LX200Longitude, &d, &m, &s);
+        snprintf(read_buffer, sizeof(read_buffer), ":Sg%03d:%02d#", d, m);
+        break;
+    case LX200_GEO_LONG_FORMAT: // d m s
+        getSexComponents(LX200Longitude, &d, &m, &s);
+        snprintf(read_buffer, sizeof(read_buffer), ":Sg%03d:%02d:%02d#", d, m, s);
+        break;
+    case LX200_GEO_LONGER_FORMAT: // d m s.f with f being tenths
+        double s_f;
+        getSexComponentsIID(LX200Longitude, &d, &m, &s_f);
+        snprintf(read_buffer, sizeof(read_buffer), ":Sg%03d:%02d:%04.01lf#", d, m, s_f);
+        break;
+    default:
+        DEBUGFDEVICE(lx200Name, DBG_SCOPE, "Unknown geographic format <%d>", geo_format);
+        return -1;
+    }
 
     return (setStandardProcedure(fd, read_buffer));
 }
@@ -1042,9 +1130,24 @@ int setSiteLatitude(int fd, double Lat)
 /* Add mutex */
 /*    std::unique_lock<std::mutex> guard(lx200CommsLock); */
 
-    getSexComponents(Lat, &d, &m, &s);
-
-    snprintf(read_buffer, sizeof(read_buffer), ":St%+03d:%02d:%02d#", d, m, s);
+    switch (geo_format) {
+    case LX200_GEO_SHORT_FORMAT: // d m
+        getSexComponents(Lat, &d, &m, &s);
+        snprintf(read_buffer, sizeof(read_buffer), ":St%+03d:%02d#", d, m);
+        break;
+    case LX200_GEO_LONG_FORMAT: // d m s
+        getSexComponents(Lat, &d, &m, &s);
+        snprintf(read_buffer, sizeof(read_buffer), ":St%+03d:%02d:%02d#", d, m, s);
+        break;
+    case LX200_GEO_LONGER_FORMAT: // d m s.f with f being tenths
+        double s_f;
+        getSexComponentsIID(Lat, &d, &m, &s_f);
+        snprintf(read_buffer, sizeof(read_buffer), ":St%+03d:%02d:%04.01lf#", d, m, s_f);
+        break;
+    default:
+        DEBUGFDEVICE(lx200Name, DBG_SCOPE, "Unknown geographic format <%d>", geo_format);
+        return -1;
+    }
 
     return (setStandardProcedure(fd, read_buffer));
 }
@@ -1561,15 +1664,20 @@ int selectSubCatalog(int fd, int catalog, int subCatalog)
     return (setStandardProcedure(fd, read_buffer));
 }
 
-int getLX200Format()
+int getLX200EquatorialFormat()
 {
-    return controller_format;
+    return eq_format;
 }
 
-int checkLX200Format(int fd)
+int getLX200GeographicFormat()
+{
+    return geo_format;
+}
+
+int checkLX200EquatorialFormat(int fd)
 {
     char read_buffer[RB_MAX_LEN] = {0};
-    controller_format = LX200_LONG_FORMAT;
+    eq_format = LX200_EQ_LONG_FORMAT;
     int error_type;
     int nbytes_write = 0, nbytes_read = 0;
 
@@ -1595,23 +1703,31 @@ int checkLX200Format(int fd)
 
     DEBUGFDEVICE(lx200Name, DBG_SCOPE, "RES <%s>", read_buffer);
 
+    // 10micron returns on U2 15:46:18.03 . Prevent setting it to a lower precision later by detecting this mode here.
+    if (nbytes_read >= 11 && read_buffer[8] == '.')
+    {
+        eq_format = LX200_EQ_LONGER_FORMAT;
+        DEBUGDEVICE(lx200Name, DBG_SCOPE, "Equatorial coordinate format is ultra high precision.");
+        return 0;
+    }
+
     /* If it's short format, try to toggle to high precision format */
     if (read_buffer[5] == '.')
     {
-        DEBUGDEVICE(lx200Name, DBG_SCOPE, "Detected low precision format, attempting to switch to high precision.");
+        DEBUGDEVICE(lx200Name, DBG_SCOPE, "Detected low precision equatorial format, attempting to switch to high precision.");
         if ((error_type = tty_write_string(fd, ":U#", &nbytes_write)) != TTY_OK)
             return error_type;
     }
     else if (read_buffer[8] == '.')
     {
-        controller_format = LX200_LONGER_FORMAT;
-        DEBUGDEVICE(lx200Name, DBG_SCOPE, "Coordinate format is ultra high precision.");
+        eq_format = LX200_EQ_LONGER_FORMAT;
+        DEBUGDEVICE(lx200Name, DBG_SCOPE, "Equatorial coordinate format is ultra high precision.");
         return 0;
     }
     else
     {
-        controller_format = LX200_LONG_FORMAT;
-        DEBUGDEVICE(lx200Name, DBG_SCOPE, "Coordinate format is high precision.");
+        eq_format = LX200_EQ_LONG_FORMAT;
+        DEBUGDEVICE(lx200Name, DBG_SCOPE, "Equatorial coordinate format is high precision.");
         return 0;
     }
 
@@ -1636,13 +1752,13 @@ int checkLX200Format(int fd)
 
     if (read_buffer[5] == '.')
     {
-        controller_format = LX200_SHORT_FORMAT;
-        DEBUGDEVICE(lx200Name, DBG_SCOPE, "Coordinate format is low precision.");
+        eq_format = LX200_EQ_SHORT_FORMAT;
+        DEBUGDEVICE(lx200Name, DBG_SCOPE, "Equatorial coordinate format is low precision.");
     }
     else
     {
-        controller_format = LX200_LONG_FORMAT;
-        DEBUGDEVICE(lx200Name, DBG_SCOPE, "Coordinate format is high precision.");
+        eq_format = LX200_EQ_LONG_FORMAT;
+        DEBUGDEVICE(lx200Name, DBG_SCOPE, "Equatorial coordinate format is high precision.");
     }
 
     tcflush(fd, TCIFLUSH);
