@@ -20,6 +20,7 @@
 #include "connectionplugins/connectioninterface.h"
 #include "connectionplugins/connectiontcp.h"
 #include "indicom.h"
+#include "libastro.h"
 
 #include <libnova/transform.h>
 #include <libnova/precession.h>
@@ -32,6 +33,7 @@
 #include <memory>
 #include <termios.h>
 #include <cstring>
+#include <assert.h>
 
 constexpr uint16_t SynscanDriver::SIM_SLEW_RATE[];
 
@@ -91,14 +93,16 @@ bool SynscanDriver::initProperties()
     //////////////////////////////////////////////////////////////////////////////////////////////////
     IUFillNumber(&CustomSlewRateN[AXIS_RA], "AXIS1", "RA/AZ (arcsecs/s)", "%.2f", 0.05, 800, 10, 0);
     IUFillNumber(&CustomSlewRateN[AXIS_DE], "AXIS2", "DE/AL (arcsecs/s)", "%.2f", 0.05, 800, 10, 0);
-    IUFillNumberVector(&CustomSlewRateNP, CustomSlewRateN, 2, getDeviceName(), "CUSTOM_SLEW_RATE", "Custom Slew", MOTION_TAB, IP_RW, 60, IPS_IDLE);
+    IUFillNumberVector(&CustomSlewRateNP, CustomSlewRateN, 2, getDeviceName(), "CUSTOM_SLEW_RATE", "Custom Slew", MOTION_TAB,
+                       IP_RW, 60, IPS_IDLE);
 
     //////////////////////////////////////////////////////////////////////////////////////////////////
     /// Guide Rate
     //////////////////////////////////////////////////////////////////////////////////////////////////
     IUFillNumber(&GuideRateN[AXIS_RA], "GUIDE_RATE_WE", "W/E Rate", "%.2f", 0, 1, 0.1, 0.5);
     IUFillNumber(&GuideRateN[AXIS_DE], "GUIDE_RATE_NS", "N/S Rate", "%.2f", 0, 1, 0.1, 0.5);
-    IUFillNumberVector(&GuideRateNP, GuideRateN, 2, getDeviceName(), "GUIDE_RATE", "Guiding Rate", GUIDE_TAB, IP_RW, 0, IPS_IDLE);
+    IUFillNumberVector(&GuideRateNP, GuideRateN, 2, getDeviceName(), "GUIDE_RATE", "Guiding Rate", GUIDE_TAB, IP_RW, 0,
+                       IPS_IDLE);
 
     //////////////////////////////////////////////////////////////////////////////////////////////////
     /// Horizontal Coords
@@ -112,6 +116,11 @@ bool SynscanDriver::initProperties()
     AddTrackMode("TRACK_EQ", "Equatorial", true);
     AddTrackMode("TRACK_PEC", "PEC Mode");
 
+    IUFillSwitch(&GotoModeS[0], "ALTAZ", "Alt/Az", ISS_OFF);
+    IUFillSwitch(&GotoModeS[1], "RADEC", "Ra/Dec", ISS_ON);
+    IUFillSwitchVector(&GotoModeSP, GotoModeS, NARRAY(GotoModeS), getDeviceName(), "GOTOMODE", "Goto mode",
+                       MAIN_CONTROL_TAB, IP_RW, ISR_1OFMANY, 0, IPS_IDLE);
+
     SetParkDataType(PARK_AZ_ALT);
 
     // Initialize guiding properties.
@@ -140,6 +149,11 @@ bool SynscanDriver::updateProperties()
         defineNumber(&GuideWENP);
         defineNumber(&GuideRateNP);
 
+        if (m_isAltAz)
+        {
+            defineSwitch(&GotoModeSP);
+        }
+
         if (InitPark())
         {
             SetAxis1ParkDefault(359);
@@ -161,6 +175,10 @@ bool SynscanDriver::updateProperties()
         deleteProperty(GuideNSNP.name);
         deleteProperty(GuideWENP.name);
         deleteProperty(GuideRateNP.name);
+        if (m_isAltAz)
+        {
+            deleteProperty(GotoModeSP.name);
+        }
     }
 
     return true;
@@ -295,6 +313,31 @@ bool SynscanDriver::ISNewNumber(const char * dev, const char * name, double valu
     return INDI::Telescope::ISNewNumber(dev, name, values, names, n);
 }
 
+bool SynscanDriver::ISNewSwitch(const char *dev, const char *name, ISState *states, char *names[], int n)
+{
+    if (dev && !strcmp(dev, getDeviceName()))
+    {
+        ISwitchVectorProperty *svp = getSwitch(name);
+
+        if (!strcmp(svp->name, GotoModeSP.name))
+        {
+            IUUpdateSwitch(svp, states, names, n);
+            ISwitch *sp = IUFindOnSwitch(svp);
+
+            assert(sp != nullptr);
+
+            if (!strcmp(sp->name, GotoModeS[0].name))
+                SetAltAzMode(true);
+            else
+                SetAltAzMode(false);
+            return true;
+        }
+
+    }
+
+    return INDI::Telescope::ISNewSwitch(dev, name, states, names, n);
+}
+
 bool SynscanDriver::echo()
 {
     char res[SYN_RES] = {0};
@@ -382,6 +425,7 @@ bool SynscanDriver::readModel()
         {5, "AZ-EQ6 GOTO Series"},
         {6, "AZ-EQ5 GOTO Series"},
         {160, "AllView GOTO Series"},
+        {161, "Virtuoso Alt/Az mount"},
         {165, "AZ-GTi GOTO Series"}
     };
 
@@ -407,7 +451,8 @@ bool SynscanDriver::readModel()
     m_isAltAz = m_MountModel > 4;
 
     LOGF_INFO("Driver is running in %s mode.", m_isAltAz ? "Alt-Az" : "Equatorial");
-    LOGF_INFO("Detected mount: %s. Mount must be aligned from the handcontroller before using the driver.", StatusT[MI_MOUNT_MODEL].text);
+    LOGF_INFO("Detected mount: %s. Mount must be aligned from the handcontroller before using the driver.",
+              StatusT[MI_MOUNT_MODEL].text);
 
     return true;
 }
@@ -479,7 +524,7 @@ bool SynscanDriver::ReadScopeStatus()
     J2000Pos.dec = rangeDec(de);
 
     // Synscan reports J2000 coordinates so we need to convert from J2000 to JNow
-    ln_get_equ_prec2(&J2000Pos, JD2000, ln_get_julian_from_sys(), &epochPos);
+    LibAstro::J2000toObserved(&J2000Pos, ln_get_julian_from_sys(), &epochPos);
 
     CurrentRA = epochPos.ra / 15.0;
     CurrentDE = epochPos.dec;
@@ -545,6 +590,36 @@ bool SynscanDriver::SetTrackMode(uint8_t mode)
     return sendCommand(cmd, res);
 }
 
+bool SynscanDriver::SetAltAzMode(bool enable)
+{
+    IUResetSwitch(&GotoModeSP);
+
+    if (enable)
+    {
+        ISwitch *sp = IUFindSwitch(&GotoModeSP, "ALTAZ");
+        if (sp)
+        {
+            LOG_INFO("Using AltAz goto.");
+            sp->s = ISS_ON;
+        }
+        goto_AltAz = true;
+    }
+    else
+    {
+        ISwitch *sp = IUFindSwitch(&GotoModeSP, "RADEC");
+        if (sp)
+        {
+            sp->s = ISS_ON;
+            LOG_INFO("Using Ra/Dec goto.");
+        }
+        goto_AltAz = false;
+    }
+
+    GotoModeSP.s = IPS_OK;
+    IDSetSwitch(&GotoModeSP, nullptr);
+    return true;
+}
+
 bool SynscanDriver::Goto(double ra, double dec)
 {
     char cmd[SYN_RES] = {0}, res[SYN_RES] = {0};
@@ -562,7 +637,7 @@ bool SynscanDriver::Goto(double ra, double dec)
     epochPos.dec = dec;
 
     // For Alt/Az mounts, we must issue Goto Alt/Az
-    if (m_isAltAz)
+    if (goto_AltAz && m_isAltAz) // only if enabled to use AltAz goto
     {
         struct ln_lnlat_posn lnobserver;
         struct ln_hrz_posn lnaltaz;
@@ -571,20 +646,22 @@ bool SynscanDriver::Goto(double ra, double dec)
         if (lnobserver.lng > 180)
             lnobserver.lng -= 360;
         lnobserver.lat = LocationN[LOCATION_LATITUDE].value;
-        ln_get_hrz_from_equ(&epochPos, &lnobserver, ln_get_julian_from_sys(), &lnaltaz);
+        get_hrz_from_equ(&epochPos, &lnobserver, ln_get_julian_from_sys(), &lnaltaz);
         /* libnova measures azimuth from south towards west */
-        double az = range360(lnaltaz.az + 180);
+        double az = lnaltaz.az;
         double al = lnaltaz.alt;
 
         return GotoAzAlt(az, al);
     }
 
     // Synscan accepts J2000 coordinates so we need to convert from JNow to J2000
-    ln_get_equ_prec2(&epochPos, ln_get_julian_from_sys(), JD2000, &J2000Pos);
+    LibAstro::ObservedToJ2000(&epochPos, ln_get_julian_from_sys(), &J2000Pos);
 
+    double dec_pos = J2000Pos.dec;
+    if (J2000Pos.dec < 0) dec_pos = dec_pos + 360;
     // Mount deals in J2000 coords.
     uint32_t n1 = J2000Pos.ra  / 360  * 0x100000000;
-    uint32_t n2 = J2000Pos.dec / 360 * 0x100000000;
+    uint32_t n2 = dec_pos / 360 * 0x100000000;
 
     LOGF_DEBUG("Goto - JNow RA: %g JNow DE: %g J2000 RA: %g J2000 DE: %g", ra, dec, J2000Pos.ra / 15.0, J2000Pos.dec);
 
@@ -620,12 +697,10 @@ bool SynscanDriver::GotoAzAlt(double az, double alt)
             observer.lng -= 360;
         observer.lat = LocationN[LOCATION_LATITUDE].value;
 
-        horizontalPos.az = az + 180;
-        if (horizontalPos.az > 360)
-            horizontalPos.az -= 360;
+        horizontalPos.az = az;
         horizontalPos.alt = alt;
 
-        ln_get_equ_from_hrz(&horizontalPos, &observer, ln_get_julian_from_sys(), &equatorialPos);
+        get_equ_from_hrz(&horizontalPos, &observer, ln_get_julian_from_sys(), &equatorialPos);
         return Goto(equatorialPos.ra / 15.0, equatorialPos.dec);
     }
 
@@ -1090,9 +1165,9 @@ bool SynscanDriver::updateLocation(double latitude, double longitude, double ele
     cmd[5] = p2.lng.degrees;
     cmd[6] = p2.lng.minutes;
     cmd[7] = rint(p2.lng.seconds);
-    cmd[9] = IsWest ? 1 : 0;
+    cmd[8] = IsWest ? 1 : 0;
 
-    return sendCommand(cmd, res, 10);
+    return sendCommand(cmd, res, 9);
 }
 
 bool SynscanDriver::Sync(double ra, double dec)
@@ -1112,7 +1187,7 @@ bool SynscanDriver::Sync(double ra, double dec)
     epochPos.dec = dec;
 
     // Synscan accepts J2000 coordinates so we need to convert from JNow to J2000
-    ln_get_equ_prec2(&epochPos, ln_get_julian_from_sys(), JD2000, &J2000Pos);
+    LibAstro::ObservedToJ2000(&epochPos, ln_get_julian_from_sys(), &J2000Pos);
 
     // Mount deals in J2000 coords.
     uint32_t n1 = J2000Pos.ra  / 360 * 0x100000000;
@@ -1136,11 +1211,7 @@ ln_hrz_posn SynscanDriver::getAltAzPosition(double ra, double dec)
 
     Eq.ra  = ra * 360.0 / 24.0;
     Eq.dec = dec;
-    ln_get_hrz_from_equ(&Eq, &Location, ln_get_julian_from_sys(), &AltAz);
-    AltAz.az -= 180;
-    if (AltAz.az < 0)
-        AltAz.az += 360;
-
+    get_hrz_from_equ(&Eq, &Location, ln_get_julian_from_sys(), &AltAz);
     return AltAz;
 }
 
