@@ -17,6 +17,7 @@
 *******************************************************************************/
 
 #include "telescope_simulator.h"
+#include "scopesim_helper.h"
 
 #include "indicom.h"
 
@@ -79,7 +80,8 @@ ScopeSim::ScopeSim()
     DBG_SCOPE = static_cast<uint32_t>(INDI::Logger::getInstance().addDebugLevel("Scope Verbose", "SCOPE"));
 
     SetTelescopeCapability(TELESCOPE_CAN_PARK | TELESCOPE_CAN_SYNC | TELESCOPE_CAN_GOTO | TELESCOPE_CAN_ABORT |
-                           TELESCOPE_HAS_TIME | TELESCOPE_HAS_LOCATION | TELESCOPE_HAS_TRACK_MODE | TELESCOPE_CAN_CONTROL_TRACK | TELESCOPE_HAS_TRACK_RATE,
+                           TELESCOPE_HAS_TIME | TELESCOPE_HAS_LOCATION | TELESCOPE_HAS_TRACK_MODE | TELESCOPE_CAN_CONTROL_TRACK |
+                           TELESCOPE_HAS_TRACK_RATE,
                            4);
 
     /* initialize random seed: */
@@ -87,6 +89,7 @@ ScopeSim::ScopeSim()
 
     // initalise axis positions, for GEM pointing at pole, counterweight down
     axisPrimary.setDegrees(90.0);
+    axisPrimary.TrackRate(Axis::SIDEREAL);
     axisSecondary.setDegrees(90.0);
 }
 
@@ -132,7 +135,7 @@ bool ScopeSim::initProperties()
                        "Simulation", IP_RO, 0, IPS_IDLE);
 #endif
 
-       /* How fast do we guide compared to sidereal rate */
+    /* How fast do we guide compared to sidereal rate */
     IUFillNumber(&GuideRateN[RA_AXIS], "GUIDE_RATE_WE", "W/E Rate", "%g", 0, 1, 0.1, 0.5);
     IUFillNumber(&GuideRateN[DEC_AXIS], "GUIDE_RATE_NS", "N/S Rate", "%g", 0, 1, 0.1, 0.5);
     IUFillNumberVector(&GuideRateNP, GuideRateN, 2, getDeviceName(), "GUIDE_RATE", "Guiding Rate", MOTION_TAB, IP_RW, 0,
@@ -145,8 +148,10 @@ bool ScopeSim::initProperties()
     IUFillSwitchVector(&SlewRateSP, SlewRateS, 4, getDeviceName(), "TELESCOPE_SLEW_RATE", "Slew Rate", MOTION_TAB,
                        IP_RW, ISR_1OFMANY, 0, IPS_IDLE);
 
-    // Add Tracking Modes
+    // Add Tracking Modes, the order must match the order of the TelescopeTrackMode enum
     AddTrackMode("TRACK_SIDEREAL", "Sidereal", true);
+    AddTrackMode("TRACK_SOLAR", "Solar");
+    AddTrackMode("TRACK_LUNAR", "Lunar");
     AddTrackMode("TRACK_CUSTOM", "Custom");
 
     // Let's simulate it to be an F/7.5 120mm telescope
@@ -155,7 +160,8 @@ bool ScopeSim::initProperties()
     ScopeParametersN[2].value = 120;
     ScopeParametersN[3].value = 900;
 
-    SetParkDataType(PARK_RA_DEC);
+    // RA is a rotating frame, while HA or Alt/Az is not
+    SetParkDataType(PARK_HA_DEC);
 
     initGuiderProperties(getDeviceName(), MOTION_TAB);
 
@@ -185,7 +191,6 @@ void ScopeSim::ISGetProperties(const char *dev)
     defineNumber(&flipHourAngleNP);
     loadConfig(true, flipHourAngleNP.name);
 #endif
-
     /*
     if (isConnected())
     {
@@ -214,25 +219,34 @@ bool ScopeSim::updateProperties()
 
         if (InitPark())
         {
-            // If loading parking data is successful, we just set the default parking values.
-            SetAxis1ParkDefault(currentRA);
-            SetAxis2ParkDefault(currentDEC);
 
             if (isParked())
             {
-                currentRA = ParkPositionN[AXIS_RA].value;
+	        // at this point there is a valid ParkData.xml available
+	        double longitude, latitude;
+	        IUGetConfigNumber(getDeviceName(), "GEOGRAPHIC_COORD", "LONG", &longitude);
+	        IUGetConfigNumber(getDeviceName(), "GEOGRAPHIC_COORD", "LAT", &latitude);
+	        alignment.latitude = Angle(latitude);
+		alignment.longitude = Angle(longitude);
+
+	        currentRA = (alignment.lst() - Angle(ParkPositionN[AXIS_RA].value, Angle::ANGLE_UNITS::HOURS)).Hours();
                 currentDEC = ParkPositionN[AXIS_DE].value;
+                Sync(currentRA, currentDEC);
+
             }
+            // If loading parking data is successful, we just set the default parking values.
+            SetAxis1ParkDefault(-6.);
+            SetAxis2ParkDefault(0.);
         }
         else
         {
             // Otherwise, we set all parking data to default in case no parking data is found.
-            SetAxis1Park(currentRA);
-            SetAxis2Park(currentDEC);
-            SetAxis1ParkDefault(currentRA);
-            SetAxis2ParkDefault(currentDEC);
+            SetAxis1Park(-6.);
+            SetAxis2Park(0.);
+            SetAxis1ParkDefault(-6.);
+            SetAxis2ParkDefault(0.);
         }
-        
+
         sendTimeFromSystem();
     }
     else
@@ -286,7 +300,7 @@ bool ScopeSim::ReadScopeStatus()
             {
                 SetParked(true);
                 EqNP.s = IPS_IDLE;
-                LOG_INFO("Telescope slew is complete. Parking...");
+                LOG_INFO("Telescope slew is complete. Parked");
             }
             break;
         case SCOPE_SLEWING:
@@ -298,6 +312,11 @@ bool ScopeSim::ReadScopeStatus()
                 SetTrackEnabled(true);
                 EqNP.s = IPS_IDLE;
                 LOG_INFO("Telescope slew is complete. Tracking...");
+
+                // check the slew accuracy
+                auto dRa = targetRA - currentRA;
+                auto dDec = targetDEC - currentDEC;
+                LOGF_DEBUG("slew accuracy %f, %f", dRa * 15 * 3600, dDec * 3600);
             }
             break;
         default:
@@ -321,19 +340,25 @@ bool ScopeSim::ReadScopeStatus()
     }
 
 #ifdef USE_SIM_TAB
-    mountAxisN[0].value = axisPrimary.position.Degrees();
-    mountAxisN[1].value = axisSecondary.position.Degrees();
-    IDSetNumber(&mountAxisNP, nullptr);
+    double axisRA = axisPrimary.position.Degrees();
+    double axisDE = axisSecondary.position.Degrees();
+    // No need to spam log until we have some actual changes.
+    if (std::fabs(mountAxisN[AXIS_RA].value - axisRA) > 0.0001 ||
+            std::fabs(mountAxisN[AXIS_DE].value - axisDE) > 0.0001)
+    {
+        mountAxisN[AXIS_RA].value = axisRA;
+        mountAxisN[AXIS_DE].value = axisDE;
+
+        LOGF_EXTRA1("%s: %f, ra %f", axisPrimary.axisName, axisPrimary.position.Degrees(), ra.Hours());
+        LOGF_EXTRA1("%s: %f, dec %f", axisSecondary.axisName, axisSecondary.position.Degrees(), dec.Degrees());
+
+        IDSetNumber(&mountAxisNP, nullptr);
+    }
 #endif
 
-    LOGF_EXTRA1("%s: %f, ra %f", axisPrimary.axisName, axisPrimary.position.Degrees(), ra.Hours());
-    LOGF_EXTRA1("%s: %f, dec %f", axisSecondary.axisName, axisSecondary.position.Degrees(), dec.Degrees());
-
     char RAStr[64], DecStr[64];
-
     fs_sexa(RAStr, currentRA, 2, 3600);
     fs_sexa(DecStr, currentDEC, 2, 3600);
-
     DEBUGF(DBG_SCOPE, "Current RA: %s Current DEC: %s", RAStr, DecStr);
 
     NewRaDec(currentRA, currentDEC);
@@ -350,6 +375,7 @@ bool ScopeSim::Goto(double r, double d)
 bool ScopeSim::Sync(double ra, double dec)
 {
     Angle a1, a2;
+    // set the mount axes to the position that will cause it to report the sync position
     alignment.apparentRaDecToMount(Angle(ra * 15.0), Angle(dec), &a1, &a2);
     axisPrimary.setDegrees(a1.Degrees());
     axisSecondary.setDegrees(a2.Degrees());
@@ -371,7 +397,8 @@ bool ScopeSim::Sync(double ra, double dec)
 
 bool ScopeSim::Park()
 {
-    StartSlew(GetAxis1Park(), GetAxis2Park(), SCOPE_PARKING);
+    double ra = (alignment.lst() - Angle(GetAxis1Park() * 15.)).Degrees()/15.;
+    StartSlew(ra, GetAxis2Park(), SCOPE_PARKING);
     return true;
 }
 
@@ -424,7 +451,7 @@ bool ScopeSim::ISNewNumber(const char *dev, const char *name, double values[], c
         {
             IUUpdateNumber(&GuideRateNP, values, names, n);
             GuideRateNP.s = IPS_OK;
-            //IDSetNumber(&GuideRateNP, nullptr);
+            IDSetNumber(&GuideRateNP, nullptr);
             return true;
         }
 
@@ -441,8 +468,8 @@ bool ScopeSim::ISNewNumber(const char *dev, const char *name, double values[], c
             mountModelNP.s = IPS_OK;
             IDSetNumber(&mountModelNP, nullptr);
             alignment.setCorrections(mountModelN[0].value, mountModelN[1].value,
-                    mountModelN[2].value, mountModelN[3].value,
-                    mountModelN[4].value, mountModelN[5].value);
+                                     mountModelN[2].value, mountModelN[3].value,
+                                     mountModelN[4].value, mountModelN[5].value);
 
             return true;
         }
@@ -474,6 +501,7 @@ bool ScopeSim::ISNewSwitch(const char *dev, const char *name, ISState *states, c
                 return false;
 
             mountTypeSP.s = IPS_OK;
+            IDSetSwitch(&mountTypeSP, nullptr);
             updateMountAndPierSide();
             return true;
         }
@@ -483,11 +511,11 @@ bool ScopeSim::ISNewSwitch(const char *dev, const char *name, ISState *states, c
                 return false;
 
             simPierSideSP.s = IPS_OK;
+            IDSetSwitch(&simPierSideSP, nullptr);
             updateMountAndPierSide();
             return true;
         }
 #endif
-
         // Slew mode
         if (strcmp(name, SlewRateSP.name) == 0)
         {
@@ -506,8 +534,8 @@ bool ScopeSim::ISNewSwitch(const char *dev, const char *name, ISState *states, c
 
 bool ScopeSim::Abort()
 {
-    axisPrimary.AbortSlew();
-    axisSecondary.AbortSlew();
+    axisPrimary.Abort();
+    axisSecondary.Abort();
     return true;
 }
 
@@ -521,7 +549,7 @@ bool ScopeSim::MoveNS(INDI_DIR_NS dir, TelescopeMotionCommand command)
     mcRate = static_cast<int>(IUFindOnSwitchIndex(&SlewRateSP)) + 1;
 
     int rate = (dir == INDI_DIR_NS::DIRECTION_NORTH) ? mcRate : -mcRate;
-    //LOGF_DEBUG("MoveNS dir %s, motion %s, rate %d", dir == DIRECTION_NORTH ? "N" : "S" , command == 0 ? "start" : "stop", rate);
+    LOGF_DEBUG("MoveNS dir %s, motion %s, rate %d", dir == DIRECTION_NORTH ? "N" : "S", command == 0 ? "start" : "stop", rate);
 
     axisSecondary.mcRate = command == MOTION_START ? rate : 0;
 
@@ -537,8 +565,8 @@ bool ScopeSim::MoveWE(INDI_DIR_WE dir, TelescopeMotionCommand command)
     }
 
     mcRate = static_cast<int>(IUFindOnSwitchIndex(&SlewRateSP)) + 1;
-    int rate = (dir == INDI_DIR_WE::DIRECTION_EAST) ? mcRate : -mcRate;
-    //LOGF_DEBUG("MoveWE dir %d, motion %s, rate %d", dir == DIRECTION_EAST ? "E" : "W", command == 0 ? "start" : "stop", rate);
+    int rate = (dir == INDI_DIR_WE::DIRECTION_EAST) ? -mcRate : mcRate;
+    LOGF_DEBUG("MoveWE dir %d, motion %s, rate %d", dir == DIRECTION_EAST ? "E" : "W", command == 0 ? "start" : "stop", rate);
 
     axisPrimary.mcRate = command == MOTION_START ? rate : 0;
     return true;
@@ -578,7 +606,9 @@ IPState ScopeSim::GuideWest(uint32_t ms)
 
 bool ScopeSim::SetCurrentPark()
 {
-    SetAxis1Park(currentRA);
+
+    double ha  = (alignment.lst() - Angle(currentRA, Angle::ANGLE_UNITS::HOURS)).Hours();
+    SetAxis1Park(ha);
     SetAxis2Park(currentDEC);
 
     return true;
@@ -586,33 +616,48 @@ bool ScopeSim::SetCurrentPark()
 
 bool ScopeSim::SetDefaultPark()
 {
-    // By default set RA to HA
-    SetAxis1Park(get_local_sidereal_time(LocationN[LOCATION_LONGITUDE].value));
-
-    // Set DEC to 90 or -90 depending on the hemisphere
-    SetAxis2Park((LocationN[LOCATION_LATITUDE].value > 0) ? 90 : -90);
+    // mount points to East (couter weights down) at the horizon
+    // works for both hemispheres
+    SetAxis1Park(-6.);
+    SetAxis2Park(0.);
 
     return true;
 }
 
 bool ScopeSim::SetTrackMode(uint8_t mode)
 {
-    INDI_UNUSED(mode);
-    axisPrimary.TrackRate(Axis::SIDEREAL);
-    return true;
+    switch (static_cast<TelescopeTrackMode>(mode))
+    {
+        case TRACK_SIDEREAL:
+            axisPrimary.TrackRate(Axis::SIDEREAL);
+            axisSecondary.TrackRate(Axis::OFF);
+            return true;
+        case TRACK_SOLAR:
+            axisPrimary.TrackRate(Axis::SOLAR);
+            axisSecondary.TrackRate(Axis::OFF);
+            return true;
+        case TRACK_LUNAR:
+            axisPrimary.TrackRate(Axis::LUNAR);
+            axisSecondary.TrackRate(Axis::OFF);
+            return true;
+        case TRACK_CUSTOM:
+            SetTrackRate(TrackRateN[AXIS_RA].value, TrackRateN[AXIS_DE].value);
+            return true;
+    }
+    return false;
 }
 
 bool ScopeSim::SetTrackEnabled(bool enabled)
 {
-    Axis::AXIS_TRACK_RATE rate = enabled ? Axis::AXIS_TRACK_RATE::SIDEREAL : Axis::AXIS_TRACK_RATE::OFF;
-    axisPrimary.TrackRate(rate);
+    axisPrimary.Tracking(enabled);
+    axisSecondary.Tracking(enabled);
     return true;
 }
 
 bool ScopeSim::SetTrackRate(double raRate, double deRate)
 {
-    INDI_UNUSED(raRate);
-    INDI_UNUSED(deRate);
+    axisPrimary.TrackingRateDegSec = Angle(raRate / 3600.0);
+    axisSecondary.TrackingRateDegSec = Angle(deRate / 3600.0);
     return true;
 }
 
@@ -639,19 +684,34 @@ bool ScopeSim::updateLocation(double latitude, double longitude, double elevatio
 
     INDI_UNUSED(elevation);
     return true;
- }
+}
 
-void ScopeSim::updateMountAndPierSide()
+bool ScopeSim::updateMountAndPierSide()
 {
 #ifdef USE_SIM_TAB
     int mountType = IUFindOnSwitchIndex(&mountTypeSP);
     int pierSide = IUFindOnSwitchIndex(&simPierSideSP);
-    if (mountType < 0 || pierSide < 0) return;
+    if (mountType < 0 || pierSide < 0)
+        return false;
+
+    // If nothing changed, return
+    if (mountType == m_MountType && pierSide == m_PierSide)
+        return true;
+
+    m_MountType = mountType;
+    m_PierSide = pierSide;
+
     LOGF_INFO("update mount and pier side: Pier Side %s, mount type %d", pierSide == 0 ? "Off" : "On", mountType);
 #else
     int mountType = 2;
     int pierSide = 1;
 #endif
+    if ( mountType == Alignment::MOUNT_TYPE::ALTAZ)
+    {
+        LOG_INFO("AltAz mount type not implemented yet");
+        return false;
+    }
+
     alignment.mountType = static_cast<Alignment::MOUNT_TYPE>(mountType);
     // update the pier side capability depending on the mount type
     uint32_t cap = GetTelescopeCapability();
@@ -664,6 +724,8 @@ void ScopeSim::updateMountAndPierSide()
         cap &= ~static_cast<uint32_t>(TELESCOPE_HAS_PIER_SIDE);
     }
     SetTelescopeCapability(cap, 4);
+
+    return true;
 }
 
 /////////////////////////////////////////////////////////////////////
