@@ -22,10 +22,11 @@ void dsp_buffer_shift(dsp_stream_p stream)
 {
     if(stream->dims == 0)
         return;
-    double* tmp = (double*)malloc(sizeof(double) * stream->len);
-    for(int x = 0; x < stream->len/2; x++) {
+    dsp_t* tmp = (dsp_t*)malloc(sizeof(dsp_t) * stream->len);
+    int x, d;
+    for(x = 0; x < stream->len/2; x++) {
         int* pos = dsp_stream_get_position(stream, x);
-        for(int d = 0; d < stream->dims; d++) {
+        for(d = 0; d < stream->dims; d++) {
             if(pos[d]<stream->sizes[d] / 2) {
                 pos[d] += stream->sizes[d] / 2;
             } else {
@@ -36,29 +37,21 @@ void dsp_buffer_shift(dsp_stream_p stream)
         tmp[dsp_stream_set_position(stream, pos)] = stream->buf[x];
         free(pos);
     }
-    memcpy(stream->buf, tmp, stream->len * sizeof(double));
+    memcpy(stream->buf, tmp, stream->len * sizeof(dsp_t));
     free(tmp);
 }
 
-void dsp_buffer_clear(dsp_stream_p stream)
-{
-    int k;
-    for(k = 0; k < stream->len; k++)
-        stream->buf[k] = 0;
-
-}
-
 void dsp_buffer_removemean(dsp_stream_p stream)
 {
     int k;
 
-    double mean = dsp_stats_mean(stream->buf, stream->len);
+    dsp_t mean = dsp_stats_mean(stream->buf, stream->len);
     for(k = 0; k < stream->len; k++)
         stream->buf[k] = stream->buf[k] - mean;
 
 }
 
-void dsp_buffer_sub(dsp_stream_p stream, double* in, int inlen)
+void dsp_buffer_sub(dsp_stream_p stream, dsp_t* in, int inlen)
 {
     int len = Min(stream->len, inlen);
 
@@ -69,18 +62,18 @@ void dsp_buffer_sub(dsp_stream_p stream, double* in, int inlen)
 
 }
 
-void dsp_buffer_sum(dsp_stream_p stream, double* in, int inlen)
+void dsp_buffer_sum(dsp_stream_p stream, dsp_t* in, int inlen)
 {
     int len = Min(stream->len, inlen);
 
 	int k;
 	for(k = 0; k < len; k++) {
-        stream->buf[k] = stream->buf[k] + in[k];
+        stream->buf[k] += in[k];
 	}
 
 }
 
-void dsp_buffer_div(dsp_stream_p stream, double* in, int inlen)
+void dsp_buffer_div(dsp_stream_p stream, dsp_t* in, int inlen)
 {
     int len = Min(stream->len, inlen);
 
@@ -91,7 +84,7 @@ void dsp_buffer_div(dsp_stream_p stream, double* in, int inlen)
 
 }
 
-void dsp_buffer_mul(dsp_stream_p stream, double* in, int inlen)
+void dsp_buffer_mul(dsp_stream_p stream, dsp_t* in, int inlen)
 {
     int len = Min(stream->len, inlen);
 
@@ -102,7 +95,7 @@ void dsp_buffer_mul(dsp_stream_p stream, double* in, int inlen)
 
 }
 
-void dsp_buffer_pow(dsp_stream_p stream, double* in, int inlen)
+void dsp_buffer_pow(dsp_stream_p stream, dsp_t* in, int inlen)
 {
     int len = Min(stream->len, inlen);
 
@@ -113,7 +106,7 @@ void dsp_buffer_pow(dsp_stream_p stream, double* in, int inlen)
 
 }
 
-void dsp_buffer_log(dsp_stream_p stream, double* in, int inlen)
+void dsp_buffer_log(dsp_stream_p stream, dsp_t* in, int inlen)
 {
     int len = Min(stream->len, inlen);
 
@@ -159,7 +152,7 @@ void dsp_buffer_1div(dsp_stream_p stream, double val)
     int k;
 
     for(k = 0; k < stream->len; k++) {
-        stream->buf[k] = val / stream->buf[k];
+        stream->buf[k] = (dsp_t)(val / (dsp_t)stream->buf[k]);
     }
 
 }
@@ -169,7 +162,7 @@ void dsp_buffer_div1(dsp_stream_p stream, double val)
     int k;
 
     for(k = 0; k < stream->len; k++) {
-        stream->buf[k] /= val;
+        stream->buf[k] = (dsp_t)((dsp_t)stream->buf[k] / val);
     }
 
 }
@@ -179,7 +172,7 @@ void dsp_buffer_mul1(dsp_stream_p stream, double val)
     int k;
 
     for(k = 0; k < stream->len; k++) {
-        stream->buf[k] = stream->buf[k] * val;
+        stream->buf[k] = (dsp_t)((dsp_t)stream->buf[k] * val);
     }
 
 }
@@ -189,7 +182,7 @@ void dsp_buffer_pow1(dsp_stream_p stream, double val)
     int k;
 
     for(k = 0; k < stream->len; k++) {
-        stream->buf[k] = pow(stream->buf[k], val);
+        stream->buf[k] = (dsp_t)pow((dsp_t)stream->buf[k], val);
     }
 
 }
@@ -199,15 +192,15 @@ void dsp_buffer_log1(dsp_stream_p stream, double val)
     int k;
 
     for(k = 0; k < stream->len; k++) {
-        stream->buf[k] = Log(stream->buf[k], val);
+        stream->buf[k] = (dsp_t)Log((dsp_t)stream->buf[k], val);
     }
 
 }
 
 static int compare( const void* a, const void* b)
 {
-     double int_a = * ( (double*) a );
-     double int_b = * ( (double*) b );
+     dsp_t int_a = * ( (dsp_t*) a );
+     dsp_t int_b = * ( (dsp_t*) b );
 
      if ( int_a == int_b ) return 0;
      else if ( int_a < int_b ) return -1;
@@ -218,19 +211,20 @@ void dsp_buffer_median(dsp_stream_p stream, int size, int median)
 {
 	int k;
     int mid = (size / 2) + (size % 2);
-    double* sorted = (double*)malloc(size * sizeof(double));
+    dsp_t* sorted = (dsp_t*)malloc(size * sizeof(dsp_t));
     for(k = mid; k < stream->len; k++) {
-        memcpy (sorted, stream->buf + (k - mid), size * sizeof(double));
-        qsort(sorted, size, sizeof(double), compare);
+        memcpy (sorted, stream->buf + (k - mid), size * sizeof(dsp_t));
+        qsort(sorted, size, sizeof(dsp_t), compare);
         stream->buf[k] = sorted[median];
 	}
 
 }
 
-void dsp_buffer_deviate(dsp_stream_p stream, double* deviation, double mindeviation, double maxdeviation)
+void dsp_buffer_deviate(dsp_stream_p stream, dsp_t* deviation, dsp_t mindeviation, dsp_t maxdeviation)
 {
     dsp_stream_p tmp = dsp_stream_copy(stream);
-    for(int k = 1; k < stream->len; k++) {
+    int k;
+    for(k = 1; k < stream->len; k++) {
         stream->buf[(int)Max(0, Min(stream->len, ((deviation[k] - mindeviation) * (maxdeviation - mindeviation) + mindeviation) + k))] = tmp->buf[k];
     }
     dsp_stream_free(tmp);
