@@ -24,6 +24,7 @@
 #include <cstdlib>
 #include <cstring>
 #include <assert.h>
+#include <algorithm>
 
 const char *COMMUNICATION_TAB = "Communication";
 const char *MAIN_CONTROL_TAB  = "Main Control";
@@ -36,6 +37,7 @@ const char *FILTER_TAB        = "Filter Wheel";
 const char *FOCUS_TAB         = "Focuser";
 const char *GUIDE_TAB         = "Guide";
 const char *ALIGNMENT_TAB     = "Alignment";
+const char *SATELLITE_TAB     = "Satellite";
 const char *INFO_TAB          = "General Info";
 
 void timerfunc(void *t)
@@ -57,24 +59,15 @@ namespace INDI
 
 DefaultDevice::DefaultDevice()
 {
-    pDebug      = false;
-    pSimulation = false;
-    isInit      = false;
-
-    majorVersion        = 1;
-    minorVersion        = 0;
-    interfaceDescriptor = GENERAL_INTERFACE;
     memset(&ConnectionModeSP, 0, sizeof(ConnectionModeSP));
 }
 
-DefaultDevice::~DefaultDevice()
-{
-}
-
 bool DefaultDevice::loadConfig(bool silent, const char *property)
 {
-    char errmsg[MAXRBUF];
+    char errmsg[MAXRBUF] = {0};
+    m_ConfigLoading = true;
     bool pResult = IUReadConfig(nullptr, deviceID, property, silent ? 1 : 0, errmsg) == 0 ? true : false;
+    m_ConfigLoading = false;
 
     if (!silent)
     {
@@ -83,10 +76,15 @@ bool DefaultDevice::loadConfig(bool silent, const char *property)
             LOG_DEBUG("Configuration successfully loaded.");
         }
         else
-            LOG_INFO("No previous configuration found. To save driver configuration, click Save Configuration in Options tab");
+            LOG_INFO("No previous configuration found. To save driver configuration, click Save Configuration in Options tab.");
     }
 
-    IUSaveDefaultConfig(nullptr, nullptr, deviceID);
+    // Determine default config file name
+    // Need to be done only once per device.
+    if (m_DefaultConfigLoaded == false)
+    {
+        m_DefaultConfigLoaded = IUSaveDefaultConfig(nullptr, nullptr, deviceID) == 0;
+    }
 
     return pResult;
 }
@@ -106,44 +104,16 @@ bool DefaultDevice::saveConfigItems(FILE *fp)
 
 bool DefaultDevice::saveAllConfigItems(FILE *fp)
 {
-    std::vector<INDI::Property *>::iterator orderi;
-
-    ISwitchVectorProperty *svp = nullptr;
-    INumberVectorProperty *nvp = nullptr;
-    ITextVectorProperty *tvp   = nullptr;
-    IBLOBVectorProperty *bvp   = nullptr;
-
-    for (orderi = pAll.begin(); orderi != pAll.end(); ++orderi)
+    for (INDI::Property *oneProperty : pAll)
     {
-        INDI_PROPERTY_TYPE pType = (*orderi)->getType();
-        void *pPtr  = (*orderi)->getProperty();
-
-        switch (pType)
+        if (oneProperty->getType() == INDI_SWITCH)
         {
-            case INDI_NUMBER:
-                nvp = static_cast<INumberVectorProperty *>(pPtr);
-                //IDLog("Trying to save config for number %s\n", nvp->name);
-                IUSaveConfigNumber(fp, nvp);
-                break;
-            case INDI_TEXT:
-                tvp = static_cast<ITextVectorProperty *>(pPtr);
-                IUSaveConfigText(fp, tvp);
-                break;
-            case INDI_SWITCH:
-                svp = static_cast<ISwitchVectorProperty *>(pPtr);
-                /* Never save CONNECTION property. Don't save switches with no switches on if the rule is one of many */
-                if (!strcmp(svp->name, INDI::SP::CONNECTION) || (svp->r == ISR_1OFMANY && !IUFindOnSwitch(svp)))
-                    continue;
-                IUSaveConfigSwitch(fp, svp);
-                break;
-            case INDI_BLOB:
-                bvp = static_cast<IBLOBVectorProperty *>(pPtr);
-                IUSaveConfigBLOB(fp, bvp);
-                break;
-            case INDI_LIGHT:
-            case INDI_UNKNOWN:
-                break;
+            ISwitchVectorProperty * svp = oneProperty->getSwitch();
+            /* Never save CONNECTION property. Don't save switches with no switches on if the rule is one of many */
+            if (!strcmp(svp->name, INDI::SP::CONNECTION) || (svp->r == ISR_1OFMANY && !IUFindOnSwitch(svp)))
+                continue;
         }
+        oneProperty->save(fp);
     }
     return true;
 }
@@ -163,8 +133,8 @@ bool DefaultDevice::purgeConfig()
 
 bool DefaultDevice::saveConfig(bool silent, const char *property)
 {
-    //std::vector<orderPtr>::iterator orderi;
-    char errmsg[MAXRBUF];
+    silent = false;
+    char errmsg[MAXRBUF] = {0};
 
     FILE *fp = nullptr;
 
@@ -185,9 +155,13 @@ bool DefaultDevice::saveConfig(bool silent, const char *property)
 
         IUSaveConfigTag(fp, 1, getDeviceName(), silent ? 1 : 0);
 
+        fflush(fp);
         fclose(fp);
 
-        IUSaveDefaultConfig(nullptr, nullptr, deviceID);
+        if (m_DefaultConfigLoaded == false)
+        {
+            m_DefaultConfigLoaded = IUSaveDefaultConfig(nullptr, nullptr, deviceID) == 0;
+        }
 
         LOG_DEBUG("Configuration successfully saved.");
     }
@@ -304,6 +278,7 @@ bool DefaultDevice::saveConfig(bool silent, const char *property)
         {
             fp = IUGetConfigFP(nullptr, deviceID, "w", errmsg);
             prXMLEle(fp, root, 0);
+            fflush(fp);
             fclose(fp);
             delXMLEle(root);
             LOGF_DEBUG("Configuration successfully saved for %s.", property);
@@ -565,14 +540,9 @@ bool DefaultDevice::ISNewText(const char *dev, const char *name, char *texts[],
 bool DefaultDevice::ISNewBLOB(const char *dev, const char *name, int sizes[], int blobsizes[], char *blobs[],
                               char *formats[], char *names[], int n)
 {
-    INDI_UNUSED(dev);
-    INDI_UNUSED(name);
-    INDI_UNUSED(sizes);
-    INDI_UNUSED(blobsizes);
-    INDI_UNUSED(blobs);
-    INDI_UNUSED(formats);
-    INDI_UNUSED(names);
-    INDI_UNUSED(n);
+    for (Connection::Interface *oneConnection : connections)
+        oneConnection->ISNewBLOB(dev, name, sizes, blobsizes, blobs, formats, names, n);
+
     return false;
 }
 
@@ -585,13 +555,13 @@ bool DefaultDevice::ISSnoopDevice(XMLEle *root)
 void DefaultDevice::addDebugControl()
 {
     registerProperty(&DebugSP, INDI_SWITCH);
-    pDebug = false;
+    m_Debug = false;
 }
 
 void DefaultDevice::addSimulationControl()
 {
     registerProperty(&SimulationSP, INDI_SWITCH);
-    pSimulation = false;
+    m_Simulation = false;
 }
 
 void DefaultDevice::addConfigurationControl()
@@ -614,7 +584,7 @@ void DefaultDevice::addAuxControls()
 
 void DefaultDevice::setDebug(bool enable)
 {
-    if (pDebug == enable)
+    if (m_Debug == enable)
     {
         DebugSP.s = IPS_OK;
         IDSetSwitch(&DebugSP, nullptr);
@@ -642,7 +612,7 @@ void DefaultDevice::setDebug(bool enable)
         }
     }
 
-    pDebug = enable;
+    m_Debug = enable;
 
     // Inform logger
     if (Logger::updateProperties(enable) == false)
@@ -655,7 +625,7 @@ void DefaultDevice::setDebug(bool enable)
 
 void DefaultDevice::setSimulation(bool enable)
 {
-    if (pSimulation == enable)
+    if (m_Simulation == enable)
     {
         SimulationSP.s = IPS_OK;
         IDSetSwitch(&SimulationSP, nullptr);
@@ -683,7 +653,7 @@ void DefaultDevice::setSimulation(bool enable)
         }
     }
 
-    pSimulation = enable;
+    m_Simulation = enable;
     simulationTriggered(enable);
     SimulationSP.s = IPS_OK;
     IDSetSwitch(&SimulationSP, nullptr);
@@ -691,12 +661,12 @@ void DefaultDevice::setSimulation(bool enable)
 
 bool DefaultDevice::isDebug()
 {
-    return pDebug;
+    return m_Debug;
 }
 
 bool DefaultDevice::isSimulation()
 {
-    return pSimulation;
+    return m_Simulation;
 }
 
 void DefaultDevice::debugTriggered(bool enable)
@@ -735,32 +705,10 @@ void DefaultDevice::ISGetProperties(const char *dev)
 
     for (INDI::Property *oneProperty : pAll)
     {
-        INDI_PROPERTY_TYPE pType = oneProperty->getType();
-        void *pPtr = oneProperty->getProperty();
-
         if (defineDynamicProperties == false && oneProperty->isDynamic())
             continue;
 
-        switch (pType)
-        {
-            case INDI_NUMBER:
-                IDDefNumber(static_cast<INumberVectorProperty *>(pPtr), nullptr);
-                break;
-            case INDI_TEXT:
-                IDDefText(static_cast<ITextVectorProperty *>(pPtr), nullptr);
-                break;
-            case INDI_SWITCH:
-                IDDefSwitch(static_cast<ISwitchVectorProperty *>(pPtr), nullptr);
-                break;
-            case INDI_LIGHT:
-                IDDefLight(static_cast<ILightVectorProperty *>(pPtr), nullptr);
-                break;
-            case INDI_BLOB:
-                IDDefBLOB(static_cast<IBLOBVectorProperty *>(pPtr), nullptr);
-                break;
-            case INDI_UNKNOWN:
-                break;
-        }
+        oneProperty->define();
     }
 
     // Remember debug & logging settings
@@ -784,14 +732,36 @@ void DefaultDevice::ISGetProperties(const char *dev)
                 IUFillSwitch(sp++, oneConnection->name().c_str(), oneConnection->label().c_str(), ISS_OFF);
             }
 
-            activeConnection     = connections[0];
-            ConnectionModeS[0].s = ISS_ON;
             IUFillSwitchVector(&ConnectionModeSP, ConnectionModeS, connections.size(), getDeviceName(),
                                "CONNECTION_MODE", "Connection Mode", CONNECTION_TAB, IP_RW, ISR_1OFMANY, 60, IPS_IDLE);
 
+            // Try to read config first
+            int activeConnectionIndex = -1;
+            if (IUGetConfigOnSwitchIndex(getDeviceName(), ConnectionModeSP.name, &activeConnectionIndex) == 0)
+            {
+                ConnectionModeS[activeConnectionIndex].s = ISS_ON;
+                activeConnection = connections[activeConnectionIndex];
+            }
+            // Check if we already have an active connection set.
+            else if (activeConnection != nullptr)
+            {
+                std::vector<Connection::Interface *>::iterator it = std::find(connections.begin(), connections.end(), activeConnection);
+                if (it != connections.end())
+                {
+                    int index = std::distance(connections.begin(), it);
+                    if (index >= 0)
+                        ConnectionModeS[index].s = ISS_ON;
+                }
+            }
+            // Otherwise use connection 0
+            else
+            {
+                ConnectionModeS[0].s = ISS_ON;
+                activeConnection = connections[0];
+            }
+
             defineSwitch(&ConnectionModeSP);
             activeConnection->Activated();
-            loadConfig(true, "CONNECTION_MODE");
         }
     }
 
@@ -800,67 +770,21 @@ void DefaultDevice::ISGetProperties(const char *dev)
 
 void DefaultDevice::resetProperties()
 {
-    std::vector<INDI::Property *>::iterator orderi;
-
-    for (orderi = pAll.begin(); orderi != pAll.end(); ++orderi)
+    for (INDI::Property *oneProperty : pAll)
     {
-        INDI_PROPERTY_TYPE pType = (*orderi)->getType();
-        void *pPtr  = (*orderi)->getProperty();
-
-        switch (pType)
-        {
-            case INDI_NUMBER:
-                static_cast<INumberVectorProperty *>(pPtr)->s = IPS_IDLE;
-                IDSetNumber(static_cast<INumberVectorProperty *>(pPtr), nullptr);
-                break;
-            case INDI_TEXT:
-                static_cast<ITextVectorProperty *>(pPtr)->s = IPS_IDLE;
-                IDSetText(static_cast<ITextVectorProperty *>(pPtr), nullptr);
-                break;
-            case INDI_SWITCH:
-                static_cast<ISwitchVectorProperty *>(pPtr)->s = IPS_IDLE;
-                IDSetSwitch(static_cast<ISwitchVectorProperty *>(pPtr), nullptr);
-                break;
-            case INDI_LIGHT:
-                static_cast<ILightVectorProperty *>(pPtr)->s = IPS_IDLE;
-                IDSetLight(static_cast<ILightVectorProperty *>(pPtr), nullptr);
-                break;
-            case INDI_BLOB:
-                static_cast<IBLOBVectorProperty *>(pPtr)->s = IPS_IDLE;
-                IDSetBLOB(static_cast<IBLOBVectorProperty *>(pPtr), nullptr);
-                break;
-            case INDI_UNKNOWN:
-                break;
-        }
+        oneProperty->setState(IPS_IDLE);
+        oneProperty->apply();
     }
 }
 
 void DefaultDevice::setConnected(bool status, IPState state, const char *msg)
 {
-    ISwitch *sp                = nullptr;
     ISwitchVectorProperty *svp = getSwitch(INDI::SP::CONNECTION);
     if (!svp)
         return;
 
-    IUResetSwitch(svp);
-
-    // Connect
-    if (status)
-    {
-        sp = IUFindSwitch(svp, "CONNECT");
-        if (!sp)
-            return;
-        sp->s = ISS_ON;
-    }
-    // Disconnect
-    else
-    {
-        sp = IUFindSwitch(svp, "DISCONNECT");
-        if (!sp)
-            return;
-        sp->s = ISS_ON;
-    }
-
+    svp->sp[INDI_ENABLED].s = status ? ISS_ON : ISS_OFF;
+    svp->sp[INDI_DISABLED].s = status ? ISS_OFF : ISS_ON;
     svp->s = state;
 
     if (msg == nullptr)
@@ -910,6 +834,11 @@ void DefaultDevice::setDriverInterface(uint16_t value)
     IUSaveText(&DriverInfoT[3], interfaceStr);
 }
 
+void DefaultDevice::syncDriverInfo()
+{
+    IDSetText(&DriverInfoTP, nullptr);
+}
+
 bool DefaultDevice::initProperties()
 {
     char versionStr[16];
@@ -918,8 +847,8 @@ bool DefaultDevice::initProperties()
     snprintf(versionStr, 16, "%d.%d", majorVersion, minorVersion);
     snprintf(interfaceStr, 16, "%d", interfaceDescriptor);
 
-    IUFillSwitch(&ConnectionS[0], "CONNECT", "Connect", ISS_OFF);
-    IUFillSwitch(&ConnectionS[1], "DISCONNECT", "Disconnect", ISS_ON);
+    IUFillSwitch(&ConnectionS[INDI_ENABLED], "CONNECT", "Connect", ISS_OFF);
+    IUFillSwitch(&ConnectionS[INDI_DISABLED], "DISCONNECT", "Disconnect", ISS_ON);
     IUFillSwitchVector(&ConnectionSP, ConnectionS, 2, getDeviceName(), INDI::SP::CONNECTION, "Connection", "Main Control",
                        IP_RW, ISR_1OFMANY, 60, IPS_IDLE);
     registerProperty(&ConnectionSP, INDI_SWITCH);
@@ -932,13 +861,13 @@ bool DefaultDevice::initProperties()
                      IP_RO, 60, IPS_IDLE);
     registerProperty(&DriverInfoTP, INDI_TEXT);
 
-    IUFillSwitch(&DebugS[0], "ENABLE", "Enable", ISS_OFF);
-    IUFillSwitch(&DebugS[1], "DISABLE", "Disable", ISS_ON);
+    IUFillSwitch(&DebugS[INDI_ENABLED], "ENABLE", "Enable", ISS_OFF);
+    IUFillSwitch(&DebugS[INDI_DISABLED], "DISABLE", "Disable", ISS_ON);
     IUFillSwitchVector(&DebugSP, DebugS, NARRAY(DebugS), getDeviceName(), "DEBUG", "Debug", "Options", IP_RW,
                        ISR_1OFMANY, 0, IPS_IDLE);
 
-    IUFillSwitch(&SimulationS[0], "ENABLE", "Enable", ISS_OFF);
-    IUFillSwitch(&SimulationS[1], "DISABLE", "Disable", ISS_ON);
+    IUFillSwitch(&SimulationS[INDI_ENABLED], "ENABLE", "Enable", ISS_OFF);
+    IUFillSwitch(&SimulationS[INDI_DISABLED], "DISABLE", "Disable", ISS_ON);
     IUFillSwitchVector(&SimulationSP, SimulationS, NARRAY(SimulationS), getDeviceName(), "SIMULATION", "Simulation",
                        "Options", IP_RW, ISR_1OFMANY, 0, IPS_IDLE);
 
@@ -950,7 +879,8 @@ bool DefaultDevice::initProperties()
                        "Configuration", "Options", IP_RW, ISR_ATMOST1, 0, IPS_IDLE);
 
     IUFillNumber(&PollPeriodN[0], "PERIOD_MS", "Period (ms)", "%.f", 10, 600000, 1000, POLLMS);
-    IUFillNumberVector(&PollPeriodNP, PollPeriodN, 1, getDeviceName(), "POLLING_PERIOD", "Polling", "Options", IP_RW, 0, IPS_IDLE);
+    IUFillNumberVector(&PollPeriodNP, PollPeriodN, 1, getDeviceName(), "POLLING_PERIOD", "Polling", "Options", IP_RW, 0,
+                       IPS_IDLE);
 
     INDI::Logger::initProperties(this);
 
@@ -1105,4 +1035,39 @@ void DefaultDevice::setPollingPeriodRange(uint32_t minimum, uint32_t maximum)
     IUUpdateMinMax(&PollPeriodNP);
 }
 
+void DefaultDevice::setActiveConnection(Connection::Interface *existingConnection)
+{
+    if (existingConnection == activeConnection)
+        return;
+
+    for (Connection::Interface *oneConnection : connections)
+    {
+        if (oneConnection == activeConnection)
+        {
+            oneConnection->Deactivated();
+            break;
+        }
+    }
+
+    activeConnection = existingConnection;
+    if (ConnectionModeS)
+    {
+        std::vector<Connection::Interface *>::iterator it = std::find(connections.begin(), connections.end(), activeConnection);
+        if (it != connections.end())
+        {
+            int index = std::distance(connections.begin(), it);
+            if (index >= 0)
+            {
+                IUResetSwitch(&ConnectionModeSP);
+                ConnectionModeS[index].s = ISS_ON;
+                ConnectionModeSP.s = IPS_OK;
+                // If property is registerned then send back response to client
+                INDI::Property *connectionProperty = getProperty(ConnectionModeSP.name, INDI_SWITCH);
+                if (connectionProperty && connectionProperty->getRegistered())
+                    IDSetSwitch(&ConnectionModeSP, nullptr);
+            }
+        }
+    }
+}
+
 }
