@@ -19,10 +19,14 @@
 
 #include "defaultdevice.h"
 #include "indisensorinterface.h"
+#include "connectionplugins/connectionserial.h"
+#include "connectionplugins/connectiontcp.h"
 
 #include "indicom.h"
+#include "libastro.h"
 #include "stream/streammanager.h"
 #include "locale_compat.h"
+#include "indiutility.h"
 
 #include <fitsio.h>
 
@@ -39,31 +43,6 @@
 #include <zlib.h>
 #include <sys/stat.h>
 
-// Create dir recursively
-static int _det_mkdir(const char *dir, mode_t mode)
-{
-    char tmp[PATH_MAX];
-    char *p = nullptr;
-    size_t len;
-
-    snprintf(tmp, sizeof(tmp), "%s", dir);
-    len = strlen(tmp);
-    if (tmp[len - 1] == '/')
-        tmp[len - 1] = 0;
-    for (p = tmp + 1; *p; p++)
-        if (*p == '/')
-        {
-            *p = 0;
-            if (mkdir(tmp, mode) == -1 && errno != EEXIST)
-                return -1;
-            *p = '/';
-        }
-    if (mkdir(tmp, mode) == -1 && errno != EEXIST)
-        return -1;
-
-    return 0;
-}
-
 namespace INDI
 {
 
@@ -172,49 +151,24 @@ void SensorInterface::processProperties(const char *dev)
 
 bool SensorInterface::processSnoopDevice(XMLEle *root)
 {
-    XMLEle *ep           = nullptr;
-    const char *propName = findXMLAttValu(root, "name");
-
-    if (IUSnoopNumber(root, &EqNP) == 0)
+    if (!IUSnoopNumber(root, &EqNP))
     {
-        double newra, newdec;
-        newra  = EqN[0].value;
-        newdec = EqN[1].value;
-        if ((newra != RA) || (newdec != Dec))
-        {
-            //IDLog("RA %4.2f  Dec %4.2f Snooped RA %4.2f  Dec %4.2f\n",RA,Dec,newra,newdec);
-            RA  = newra;
-            Dec = newdec;
-        }
+        RA = EqNP.np[0].value;
+        Dec = EqNP.np[1].value;
+        //IDLog("Snooped RA %4.2f  Dec %4.2f\n", RA, Dec);
     }
-    else if (!strcmp(propName, "TELESCOPE_INFO"))
+    if (!IUSnoopNumber(root, &LocationNP))
     {
-        for (ep = nextXMLEle(root, 1); ep != nullptr; ep = nextXMLEle(root, 0))
-        {
-            const char *name = findXMLAttValu(ep, "name");
-
-            if (!strcmp(name, "TELESCOPE_APERTURE"))
-            {
-                primaryAperture = atof(pcdataXMLEle(ep));
-            }
-            else if (!strcmp(name, "TELESCOPE_FOCAL_LENGTH"))
-            {
-                primaryFocalLength = atof(pcdataXMLEle(ep));
-            }
-        }
+        Lat = LocationNP.np[0].value;
+        Lon = LocationNP.np[1].value;
+        El = LocationNP.np[2].value;
+        //IDLog("Snooped Lat %4.2f  Lon %4.2f  El %4.2f\n", RA, Dec, El);
     }
-    else if (!strcmp(propName, "SKY_QUALITY"))
+    if (!IUSnoopNumber(root, &ScopeParametersNP))
     {
-        for (ep = nextXMLEle(root, 1); ep != nullptr; ep = nextXMLEle(root, 0))
-        {
-            const char *name = findXMLAttValu(ep, "name");
-
-            if (!strcmp(name, "SKY_BRIGHTNESS"))
-            {
-                MPSAS = atof(pcdataXMLEle(ep));
-                break;
-            }
-        }
+        primaryAperture = ScopeParametersNP.np[0].value;
+        primaryFocalLength = ScopeParametersNP.np[1].value;
+        //IDLog("Snooped primaryAperture %4.2f  primaryFocalLength %4.2f\n", primaryAperture, primaryFocalLength);
     }
 
     return INDI::DefaultDevice::ISSnoopDevice(root);
@@ -235,11 +189,13 @@ bool SensorInterface::processText(const char *dev, const char *name, char *texts
 
             // Update the property name!
             strncpy(EqNP.device, ActiveDeviceT[0].text, MAXINDIDEVICE);
+            strncpy(LocationNP.device, ActiveDeviceT[0].text, MAXINDIDEVICE);
+            strncpy(ScopeParametersNP.device, ActiveDeviceT[0].text, MAXINDIDEVICE);
+
             IDSnoopDevice(ActiveDeviceT[0].text, "EQUATORIAL_EOD_COORD");
+            IDSnoopDevice(ActiveDeviceT[0].text, "GEOGRAPHIC_COORD");
             IDSnoopDevice(ActiveDeviceT[0].text, "TELESCOPE_INFO");
-            IDSnoopDevice(ActiveDeviceT[2].text, "FILTER_SLOT");
-            IDSnoopDevice(ActiveDeviceT[2].text, "FILTER_NAME");
-            IDSnoopDevice(ActiveDeviceT[3].text, "SKY_QUALITY");
+            IDSnoopDevice(ActiveDeviceT[1].text, "GEOGRAPHIC_COORD");
 
             // Tell children active devices was updated.
             activeDevicesUpdated();
@@ -414,7 +370,7 @@ bool SensorInterface::processSwitch(const char *dev, const char *name, ISState *
 }
 
 bool SensorInterface::processBLOB(const char *dev, const char *name, int sizes[], int blobsizes[], char *blobs[],
-                                    char *formats[], char *names[], int n)
+                                  char *formats[], char *names[], int n)
 {
     if (HasDSP())
         DSP->ISNewBLOB(dev, name, sizes, blobsizes, blobs, formats, names, n);
@@ -491,24 +447,54 @@ bool SensorInterface::initProperties()
 
     // Snooped Devices
     IUFillText(&ActiveDeviceT[0], "ACTIVE_TELESCOPE", "Telescope", "Telescope Simulator");
-    IUFillText(&ActiveDeviceT[1], "ACTIVE_FOCUSER", "Focuser", "Focuser Simulator");
-    IUFillText(&ActiveDeviceT[2], "ACTIVE_FILTER", "Filter", "Filter Simulator");
-    IUFillText(&ActiveDeviceT[3], "ACTIVE_SKYQUALITY", "Sky Quality", "SQM");
-    IUFillTextVector(&ActiveDeviceTP, ActiveDeviceT, 4, getDeviceName(), "ACTIVE_DEVICES", "Snoop devices", OPTIONS_TAB,
+    IUFillText(&ActiveDeviceT[1], "ACTIVE_GPS", "GPS", "GPS Simulator");
+    IUFillTextVector(&ActiveDeviceTP, ActiveDeviceT, 2, getDeviceName(), "ACTIVE_DEVICES", "Snoop devices", OPTIONS_TAB,
                      IP_RW, 60, IPS_IDLE);
 
-    // Snooped RA/DEC Property
-    IUFillNumber(&EqN[0], "RA", "Ra (hh:mm:ss)", "%010.6m", 0, 24, 0, 0);
-    IUFillNumber(&EqN[1], "DEC", "Dec (dd:mm:ss)", "%010.6m", -90, 90, 0, 0);
-    IUFillNumberVector(&EqNP, EqN, 2, ActiveDeviceT[0].text, "EQUATORIAL_EOD_COORD", "EQ Coord", "Main Control", IP_RW,
-                       60, IPS_IDLE);
-
     // Snoop properties of interest
+    IUFillNumber(&EqN[0], "RA", "RA (hh:mm:ss)", "%010.6m", 0, 24, 0, 0);
+    IUFillNumber(&EqN[1], "DEC", "DEC (dd:mm:ss)", "%010.6m", -90, 90, 0, 0);
+    IUFillNumberVector(&EqNP, EqN, 2, getDeviceName(), "EQUATORIAL_EOD_COORD", "Eq. Coordinates", MAIN_CONTROL_TAB,
+                       IP_RW, 60, IPS_IDLE);
+
+    IUFillNumber(&LocationN[0], "LAT", "Lat (dd:mm:ss)", "%010.6m", -90, 90, 0, 0.0);
+    IUFillNumber(&LocationN[1], "LONG", "Lon (dd:mm:ss)", "%010.6m", 0, 360, 0, 0.0);
+    IUFillNumber(&LocationN[2], "ELEV", "Elevation (m)", "%g", -200, 10000, 0, 0);
+    IUFillNumberVector(&LocationNP, LocationN, 3, getDeviceName(), "GEOGRAPHIC_COORD", "Location", MAIN_CONTROL_TAB,
+                       IP_RO, 60, IPS_IDLE);
+
+    IUFillNumber(&ScopeParametersN[0], "TELESCOPE_APERTURE", "Aperture (mm)", "%g", 10, 5000, 0, 0.0);
+    IUFillNumber(&ScopeParametersN[1], "TELESCOPE_FOCAL_LENGTH", "Focal Length (mm)", "%g", 10, 10000, 0, 0.0);
+    IUFillNumber(&ScopeParametersN[2], "GUIDER_APERTURE", "Guider Aperture (mm)", "%g", 10, 5000, 0, 0.0);
+    IUFillNumber(&ScopeParametersN[3], "GUIDER_FOCAL_LENGTH", "Guider Focal Length (mm)", "%g", 10, 10000, 0, 0.0);
+    IUFillNumberVector(&ScopeParametersNP, ScopeParametersN, 4, getDeviceName(), "TELESCOPE_INFO", "Scope Properties",
+                       OPTIONS_TAB, IP_RW, 60, IPS_OK);
+
     IDSnoopDevice(ActiveDeviceT[0].text, "EQUATORIAL_EOD_COORD");
+    IDSnoopDevice(ActiveDeviceT[0].text, "GEOGRAPHIC_COORD");
     IDSnoopDevice(ActiveDeviceT[0].text, "TELESCOPE_INFO");
-    IDSnoopDevice(ActiveDeviceT[2].text, "FILTER_SLOT");
-    IDSnoopDevice(ActiveDeviceT[2].text, "FILTER_NAME");
-    IDSnoopDevice(ActiveDeviceT[3].text, "SKY_QUALITY");
+    IDSnoopDevice(ActiveDeviceT[1].text, "GEOGRAPHIC_COORD");
+
+    if (sensorConnection & CONNECTION_SERIAL)
+    {
+        serialConnection = new Connection::Serial(this);
+        serialConnection->registerHandshake([&]()
+        {
+            return callHandshake();
+        });
+        registerConnection(serialConnection);
+    }
+
+    if (sensorConnection & CONNECTION_TCP)
+    {
+        tcpConnection = new Connection::TCP(this);
+        tcpConnection->registerHandshake([&]()
+        {
+            return callHandshake();
+        });
+
+        registerConnection(tcpConnection);
+    }
 
     return true;
 }
@@ -519,22 +505,17 @@ void SensorInterface::SetCapability(uint32_t cap)
 
     setDriverInterface(getDriverInterface());
 
-    if (HasStreaming() && Streamer.get() == nullptr)
-    {
-        Streamer.reset(new StreamManager(this));
-        Streamer->initProperties();
-    }
-
-    if (HasDSP() && DSP.get() == nullptr)
-        DSP.reset(new DSP::Manager(this));
+    HasStreaming();
+    HasDSP();
 }
 
 void SensorInterface::setMinMaxStep(const char *property, const char *element, double min, double max, double step,
-                                   bool sendToClient)
+                                    bool sendToClient)
 {
     INumberVectorProperty *vp = nullptr;
 
-    if (!strcmp(property, FramedIntegrationNP.name)) {
+    if (!strcmp(property, FramedIntegrationNP.name))
+    {
         vp = &FramedIntegrationNP;
 
         INumber *np = IUFindNumber(vp, element);
@@ -563,7 +544,7 @@ void SensorInterface::setBufferSize(int nbuf, bool allocMem)
 
     // DSP
     if (HasDSP())
-        DSP->setSizes(1, new int[1]{ BufferSize * 8 / getBPS() });
+        DSP->setSizes(1, new int[1] { BufferSize * 8 / getBPS() });
 
     if (allocMem == false)
         return;
@@ -588,7 +569,10 @@ void SensorInterface::setIntegrationLeft(double duration)
 void SensorInterface::setIntegrationTime(double duration)
 {
     integrationTime = duration;
-    clock_gettime(CLOCK_REALTIME, &startIntegrationTime);
+    // JM 2020-04-28: FIXME
+    // This does not compile on MacOS, so commenting now.
+    // IP 2020-04-29: trying with some adaptation
+    startIntegrationTime = time_ns();
 }
 
 const char *SensorInterface::getIntegrationStartTime()
@@ -597,12 +581,10 @@ const char *SensorInterface::getIntegrationStartTime()
 
     char iso8601[32];
     struct tm *tp;
-    time_t t = (time_t)startIntegrationTime.tv_sec;
-    long n    = startIntegrationTime.tv_nsec % 1000000000;
+    time_t t = (time_t)startIntegrationTime;
 
     tp = gmtime(&t);
     strftime(iso8601, sizeof(iso8601), "%Y-%m-%dT%H:%M:%S", tp);
-    snprintf(ts, 32, "%s.%09ld", iso8601, n);
     return (ts);
 }
 
@@ -636,11 +618,14 @@ bool SensorInterface::AbortIntegration()
 
 void SensorInterface::addFITSKeywords(fitsfile *fptr, uint8_t* buf, int len)
 {
+#ifndef WITH_MINMAX
     INDI_UNUSED(buf);
     INDI_UNUSED(len);
+#endif
     int status = 0;
     char dev_name[32];
     char exp_start[32];
+    char timestamp[32];
     double integrationTime;
 
     char *orig = setlocale(LC_NUMERIC, "C");
@@ -667,11 +652,13 @@ void SensorInterface::addFITSKeywords(fitsfile *fptr, uint8_t* buf, int len)
 
     strncpy(dev_name, getDeviceName(), 32);
     strncpy(exp_start, getIntegrationStartTime(), 32);
+    snprintf(timestamp, 32, "%lf", startIntegrationTime);
 
     fits_update_key_s(fptr, TDOUBLE, "EXPTIME", &(integrationTime), "Total Integration Time (s)", &status);
 
     if (HasCooler())
-        fits_update_key_s(fptr, TDOUBLE, "SENSOR-TEMP", &(TemperatureN[0].value), "PrimarySensorInterface Temperature (Celsius)", &status);
+        fits_update_key_s(fptr, TDOUBLE, "SENSOR-TEMP", &(TemperatureN[0].value), "PrimarySensorInterface Temperature (Celsius)",
+                          &status);
 
 #ifdef WITH_MINMAX
     if (getNAxis() == 2)
@@ -692,82 +679,70 @@ void SensorInterface::addFITSKeywords(fitsfile *fptr, uint8_t* buf, int len)
         fits_update_key_s(fptr, TDOUBLE, "MPSAS", &MPSAS, "Sky Quality (mag per arcsec^2)", &status);
     }
 
-    INumberVectorProperty *nv = this->getNumber("GEOGRAPHIC_COORDS");
-    if(nv != nullptr)
+    if (Lat != -1000 && Lon != -1000 && El != -1000)
     {
-        Lat = nv->np[0].value;
-        Lon = nv->np[1].value;
-        El = nv->np[2].value;
+        char lat_str[MAXINDIFORMAT];
+        char lon_str[MAXINDIFORMAT];
+        char el_str[MAXINDIFORMAT];
+        fs_sexa(lat_str, Lat, 2, 360000);
+        fs_sexa(lat_str, Lon, 2, 360000);
+        snprintf(el_str, MAXINDIFORMAT, "%lf", El);
+        fits_update_key_s(fptr, TSTRING, "LATITUDE", lat_str, "Location Latitude", &status);
+        fits_update_key_s(fptr, TSTRING, "LONGITUDE", lon_str, "Location Longitude", &status);
+        fits_update_key_s(fptr, TSTRING, "ELEVATION", el_str, "Location Elevation", &status);
+    }
+    if (RA != -1000 && Dec != -1000)
+    {
+        ln_equ_posn epochPos { 0, 0 }, J2000Pos { 0, 0 };
+        epochPos.ra  = RA * 15.0;
+        epochPos.dec = Dec;
 
-        if (Lat != -1000 && Lon != -1000 && El != -1000)
+        // Convert from JNow to J2000
+        //TODO use exp_start instead of julian from system
+        //ln_get_equ_prec2(&epochPos, ln_get_julian_from_sys(), JD2000, &J2000Pos);
+        LibAstro::ObservedToJ2000(&epochPos, ln_get_julian_from_sys(), &J2000Pos);
+
+        double raJ2000  = J2000Pos.ra / 15.0;
+        double decJ2000 = J2000Pos.dec;
+        char ra_str[32], de_str[32];
+
+        fs_sexa(ra_str, raJ2000, 2, 360000);
+        fs_sexa(de_str, decJ2000, 2, 360000);
+
+        char *raPtr = ra_str, *dePtr = de_str;
+        while (*raPtr != '\0')
         {
-            char lat_str[MAXINDIFORMAT];
-            char lon_str[MAXINDIFORMAT];
-            char el_str[MAXINDIFORMAT];
-            fs_sexa(lat_str, Lat, 2, 360000);
-            fs_sexa(lat_str, Lon, 2, 360000);
-            snprintf(el_str, MAXINDIFORMAT, "%lf", El);
-            fits_update_key_s(fptr, TSTRING, "LATITUDE", lat_str, "Location Latitude", &status);
-            fits_update_key_s(fptr, TSTRING, "LONGITUDE", lon_str, "Location Longitude", &status);
-            fits_update_key_s(fptr, TSTRING, "ELEVATION", el_str, "Location Elevation", &status);
+            if (*raPtr == ':')
+                *raPtr = ' ';
+            raPtr++;
         }
+        while (*dePtr != '\0')
+        {
+            if (*dePtr == ':')
+                *dePtr = ' ';
+            dePtr++;
+        }
+
+        fits_update_key_s(fptr, TSTRING, "OBJCTRA", ra_str, "Object RA", &status);
+        fits_update_key_s(fptr, TSTRING, "OBJCTDEC", de_str, "Object DEC", &status);
+
+        int epoch = 2000;
+
+        //fits_update_key_s(fptr, TINT, "EPOCH", &epoch, "Epoch", &status);
+        fits_update_key_s(fptr, TINT, "EQUINOX", &epoch, "Equinox", &status);
     }
 
-    nv = this->getNumber("EQUATORIAL_EOD_COORDS");
-    if(nv != nullptr)
-    {
-        RA = nv->np[0].value;
-        Dec = nv->np[1].value;
-
-        if (RA != -1000 && Dec != -1000)
-        {
-            ln_equ_posn epochPos { 0, 0 }, J2000Pos { 0, 0 };
-            epochPos.ra  = RA * 15.0;
-            epochPos.dec = Dec;
-
-            // Convert from JNow to J2000
-            //TODO use exp_start instead of julian from system
-            ln_get_equ_prec2(&epochPos, ln_get_julian_from_sys(), JD2000, &J2000Pos);
-
-            double raJ2000  = J2000Pos.ra / 15.0;
-            double decJ2000 = J2000Pos.dec;
-            char ra_str[32], de_str[32];
-
-            fs_sexa(ra_str, raJ2000, 2, 360000);
-            fs_sexa(de_str, decJ2000, 2, 360000);
-
-            char *raPtr = ra_str, *dePtr = de_str;
-            while (*raPtr != '\0')
-            {
-                if (*raPtr == ':')
-                    *raPtr = ' ';
-                raPtr++;
-            }
-            while (*dePtr != '\0')
-            {
-                if (*dePtr == ':')
-                    *dePtr = ' ';
-                dePtr++;
-            }
-
-            fits_update_key_s(fptr, TSTRING, "OBJCTRA", ra_str, "Object RA", &status);
-            fits_update_key_s(fptr, TSTRING, "OBJCTDEC", de_str, "Object DEC", &status);
-
-            int epoch = 2000;
-
-            //fits_update_key_s(fptr, TINT, "EPOCH", &epoch, "Epoch", &status);
-            fits_update_key_s(fptr, TINT, "EQUINOX", &epoch, "Equinox", &status);
-        }
-    }
+    fits_update_key_s(fptr, TSTRING, "TIMESTAMP", timestamp, "Timestamp of start of integration", &status);
 
     fits_update_key_s(fptr, TSTRING, "DATE-OBS", exp_start, "UTC start date of observation", &status);
+
     fits_write_comment(fptr, "Generated by INDI", &status);
 
     setlocale(LC_NUMERIC, orig);
 }
 
 void SensorInterface::fits_update_key_s(fitsfile *fptr, int type, std::string name, void *p, std::string explanation,
-                                 int *status)
+                                        int *status)
 {
     // this function is for removing warnings about deprecated string conversion to char* (from arg 5)
     fits_update_key(fptr, type, name.c_str(), p, const_cast<char *>(explanation.c_str()), status);
@@ -896,10 +871,11 @@ bool SensorInterface::IntegrationComplete()
     // Reset POLLMS to default value
     POLLMS = getPollingPeriod();
 
-    if(HasDSP()) {
+    if(HasDSP())
+    {
         uint8_t* buf = (uint8_t*)malloc(getBufferSize());
         memcpy(buf, getBuffer(), getBufferSize());
-        DSP->processBLOB(buf, 1, new int[1]{ getBufferSize()*8/getBPS() }, getBPS());
+        DSP->processBLOB(buf, 1, new int[1] { getBufferSize() * 8 / getBPS() }, getBPS());
         free(buf);
     }
     // Run async
@@ -958,7 +934,7 @@ bool SensorInterface::IntegrationCompletePrivate()
 }
 
 bool SensorInterface::uploadFile(const void *fitsData, size_t totalBytes, bool sendIntegration,
-                          bool saveIntegration)
+                                 bool saveIntegration)
 {
 
     DEBUGF(Logger::DBG_DEBUG, "Uploading file. Ext: %s, Size: %d, sendIntegration? %s, saveIntegration? %s",
@@ -1209,9 +1185,9 @@ int SensorInterface::getFileIndex(const char *dir, const char *prefix, const cha
 
     if (stat(dir, &st) == -1)
     {
-        DEBUGF(Logger::DBG_DEBUG, "Creating directory %s...", dir);
-        if (_det_mkdir(dir, 0755) == -1)
-            DEBUGF(Logger::DBG_ERROR, "Error creating directory %s (%s)", dir, strerror(errno));
+        LOGF_DEBUG("Creating directory %s...", dir);
+        if (INDI::mkpath(dir, 0755) == -1)
+            LOGF_ERROR("Error creating directory %s (%s)", dir, strerror(errno));
     }
 
     dpdf = opendir(dir);
@@ -1257,7 +1233,38 @@ void SensorInterface::setBPS(int bps)
 
     // DSP
     if (HasDSP())
-        DSP->setSizes(1, new int[1]{ getBufferSize() * 8 / BPS });
+        DSP->setSizes(1, new int[1] { getBufferSize() * 8 / BPS });
 }
 
+
+bool SensorInterface::Handshake()
+{
+    return false;
+}
+
+bool SensorInterface::callHandshake()
+{
+    if (sensorConnection > 0)
+    {
+        if (getActiveConnection() == serialConnection)
+            PortFD = serialConnection->getPortFD();
+        else if (getActiveConnection() == tcpConnection)
+            PortFD = tcpConnection->getPortFD();
+    }
+
+    return Handshake();
+}
+
+void SensorInterface::setSensorConnection(const uint8_t &value)
+{
+    uint8_t mask = CONNECTION_SERIAL | CONNECTION_TCP | CONNECTION_NONE;
+
+    if (value == 0 || (mask & value) == 0)
+    {
+        DEBUGF(Logger::DBG_ERROR, "Invalid connection mode %d", value);
+        return;
+    }
+
+    sensorConnection = value;
+}
 }
