@@ -132,6 +132,13 @@ class Telescope : public DefaultDevice
             PEC_ON      = 1
         };
 
+        /*! Dome Locking Policy */
+        enum DomeLockingPolicy
+        {
+            DOME_IGNORED,       /*!< Dome is ignored. Mount can park or unpark irrespective of dome parking status. */
+            DOME_LOCKS,         /*!< Dome locks. Mount can only unpark if dome is already unparked. */
+        };
+
         /**
          * \struct TelescopeConnection
          * \brief Holds the connection mode of the telescope.
@@ -160,7 +167,8 @@ class Telescope : public DefaultDevice
             TELESCOPE_HAS_TRACK_MODE              = 1 << 8,  /** Does the telescope have track modes (sidereal, lunar, solar..etc)? */
             TELESCOPE_CAN_CONTROL_TRACK           = 1 << 9,  /** Can the telescope engage and disengage tracking? */
             TELESCOPE_HAS_TRACK_RATE              = 1 << 10, /** Does the telescope have custom track rates? */
-            TELESCOPE_HAS_PIER_SIDE_SIMULATION     = 1 << 11, /** Does the telescope simulate the pier side property? */
+            TELESCOPE_HAS_PIER_SIDE_SIMULATION    = 1 << 11, /** Does the telescope simulate the pier side property? */
+            TELESCOPE_CAN_TRACK_SATELLITE         = 1 << 12, /** Can the telescope track satellites? */
         } TelescopeCapability;
 
         Telescope();
@@ -261,6 +269,13 @@ class Telescope : public DefaultDevice
         {
             return capability & TELESCOPE_HAS_PIER_SIDE_SIMULATION;
         }
+        /**
+         * @return True if telescope can track satellites
+         */
+        bool CanTrackSatellite()
+        {
+            return capability & TELESCOPE_CAN_TRACK_SATELLITE;
+        }
         /**
          * @return True if telescope supports PEC playback property
          */
@@ -379,8 +394,7 @@ class Telescope : public DefaultDevice
 
         /**
          * @brief isLocked is mount currently locked?
-         * @return true if lock status equals true and DomeClosedLockTP is Dome Locks or Dome Locks and
-         * Dome Parks (both).
+         * @return true if lock status equals true and Dome Policy is Dome Locks.
          */
         bool isLocked() const;
 
@@ -413,7 +427,7 @@ class Telescope : public DefaultDevice
             return currentPECState;
         }
 
-protected:
+    protected:
         virtual bool saveConfigItems(FILE *fp);
 
         /** \brief The child class calls this function when it has updates */
@@ -743,9 +757,9 @@ protected:
         ITextVectorProperty ActiveDeviceTP;
         IText ActiveDeviceT[2] {};
 
-        // Switch to lock if dome is closed, and or force parking if dome parks
-        ISwitchVectorProperty DomeClosedLockTP;
-        ISwitch DomeClosedLockT[4];
+        // Switch to lock if dome is closed.
+        ISwitchVectorProperty DomePolicySP;
+        ISwitch DomePolicyS[2];
 
         // Switch for choosing between motion control by 4-way joystick or two seperate axes
         ISwitchVectorProperty MotionControlModeTP;
@@ -775,6 +789,46 @@ protected:
 
         const char * getPierSideStr(TelescopePierSide ps);
 
+        // Satellite tracking
+        /**
+         * \brief Text Vector property defining the orbital elements of an artificial satellite (TLE).
+         * \ref drivers/telescope/lx200_10micron.cpp "Example implementation"
+         */
+        ITextVectorProperty TLEtoTrackTP;
+        IText TLEtoTrackT[1] {};
+        /**
+         * \struct SatelliteWindow
+         * \brief Satellite pass: window start and end.
+         */
+        enum
+        {
+            SAT_PASS_WINDOW_START, ///< Index for start of the window
+            SAT_PASS_WINDOW_END, ///< Index for end of the window
+            SAT_PASS_WINDOW_COUNT ///< Number of indices
+        } SatelliteWindow;
+        /**
+         * \brief Text Vector property defining the start and end of a satellite pass (window contains pass).
+         * \ref drivers/telescope/lx200_10micron.cpp "Example implementation"
+         */
+        ITextVectorProperty SatPassWindowTP;
+        IText SatPassWindowT[SAT_PASS_WINDOW_COUNT] {};
+        /**
+         * \struct SatelliteTracking
+         * \brief Possible states for the satellite tracking.
+         */
+        enum
+        {
+            SAT_TRACK, ///< Track signal
+            SAT_HALT, ///< Halt signal (abort)
+            SAT_TRACK_COUNT ///< State counter
+        } SatelliteTracking;
+        /**
+         * \brief Switch Vector property defining the state of the satellite tracking of the mount.
+         * \ref drivers/telescope/lx200_10micron.cpp "Example implementation"
+         */
+        ISwitchVectorProperty TrackSatSP;
+        ISwitch TrackSatS[SAT_TRACK_COUNT];
+        
         // PEC State
         ISwitch PECStateS[2];
         ISwitchVectorProperty PECStateSP;
@@ -836,7 +890,7 @@ protected:
         /// The telescope/guide scope configuration file name
         const std::string ScopeConfigFileName;
 
-private:
+    private:
         bool processTimeInfo(const char *utc, const char *offset);
         bool processLocationInfo(double latitude, double longitude, double elevation);
         void triggerSnoop(const char *driverName, const char *propertyName);
