@@ -1,4 +1,5 @@
 /*
+    Copyright (C) 2020 by Pawel Soja <kernel32.pl@gmail.com>
     Copyright (C) 2015 by Jasem Mutlaq <mutlaqja@ikarustech.com>
     Copyright (C) 2014 by geehalel <geehalel@gmail.com>
 
@@ -26,11 +27,12 @@
 #include "indiccd.h"
 #include "indisensorinterface.h"
 #include "indilogger.h"
+#include "indiutility.h"
 
 #include <cerrno>
-#include <signal.h>
 #include <sys/stat.h>
-#include <cmath>
+
+#include <algorithm>
 
 static const char * STREAM_TAB = "Streaming";
 
@@ -44,23 +46,8 @@ StreamManager::StreamManager(DefaultDevice *mainDevice)
     m_isStreaming = false;
     m_isRecording = false;
 
-    prepareGammaLUT();
-
-    // Timer
-    // now use BSD setimer to avoi librt dependency
-    //sevp.sigev_notify=SIGEV_NONE;
-    //timer_create(CLOCK_MONOTONIC, &sevp, &fpstimer);
-    //fpssettings.it_interval.tv_sec=24*3600;
-    //fpssettings.it_interval.tv_nsec=0;
-    //fpssettings.it_value=fpssettings.it_interval;
-    //timer_settime(fpstimer, 0, &fpssettings, nullptr);
-
-    struct itimerval fpssettings;
-    fpssettings.it_interval.tv_sec  = 24 * 3600;
-    fpssettings.it_interval.tv_usec = 0;
-    fpssettings.it_value            = fpssettings.it_interval;
-    signal(SIGALRM, SIG_IGN); //portable
-    setitimer(ITIMER_REAL, &fpssettings, nullptr);
+    m_FPSAverage.setTimeWindow(1000);
+    m_FPSFast.setTimeWindow(50);
 
     recorderManager = new RecorderManager();
     recorder    = recorderManager->getDefaultRecorder();
@@ -74,14 +61,23 @@ StreamManager::StreamManager(DefaultDevice *mainDevice)
     encoder->init(currentDevice);
 
     LOGF_DEBUG("Using default encoder (%s)", encoder->getName());
+
+    m_framesThreadTerminate = false;
+    m_framesThread = std::thread(&StreamManager::asyncStreamThread, this);
+
 }
 
 StreamManager::~StreamManager()
 {
+    if (m_framesThread.joinable())
+    {
+        m_framesThreadTerminate = true;
+        m_framesIncoming.abort();
+        m_framesThread.join();
+    }
+
     delete (recorderManager);
     delete (encoderManager);
-    delete [] downscaleBuffer;
-    delete [] gammaLUT_16_8;
 }
 
 const char * StreamManager::getDeviceName()
@@ -100,14 +96,14 @@ bool StreamManager::initProperties()
     else
         IUFillSwitchVector(&StreamSP, StreamS, NARRAY(StreamS), getDeviceName(), "CCD_VIDEO_STREAM", "Video Stream",
                            STREAM_TAB, IP_RW, ISR_1OFMANY, 0, IPS_IDLE);
-    IUFillNumber(&StreamExposureN[STREAM_EXPOSURE], "STREAMING_EXPOSURE_VALUE", "Duration (s)", "%.6f", 0.000001, 10, 0.1, 0.1);
+    IUFillNumber(&StreamExposureN[STREAM_EXPOSURE], "STREAMING_EXPOSURE_VALUE", "Duration (s)", "%.6f", 0.000001, 60, 0.1, 0.1);
     IUFillNumber(&StreamExposureN[STREAM_DIVISOR], "STREAMING_DIVISOR_VALUE", "Divisor", "%.f", 1, 15, 1, 1);
     IUFillNumberVector(&StreamExposureNP, StreamExposureN, NARRAY(StreamExposureN), getDeviceName(), "STREAMING_EXPOSURE",
                        "Expose", STREAM_TAB, IP_RW, 60, IPS_IDLE);
 
     /* Measured FPS */
-    IUFillNumber(&FpsN[FPS_INSTANT], "EST_FPS", "Instant.", "%3.2f", 0.0, 999.0, 0.0, 30);
-    IUFillNumber(&FpsN[FPS_AVERAGE], "AVG_FPS", "Average (1 sec.)", "%3.2f", 0.0, 999.0, 0.0, 30);
+    IUFillNumber(&FpsN[FPS_INSTANT], "EST_FPS", "Instant.", "%.2f", 0.0, 999.0, 0.0, 30);
+    IUFillNumber(&FpsN[FPS_AVERAGE], "AVG_FPS", "Average (1 sec.)", "%.2f", 0.0, 999.0, 0.0, 30);
     IUFillNumberVector(&FpsNP, FpsN, NARRAY(FpsN), getDeviceName(), "FPS", "FPS", STREAM_TAB, IP_RO, 60, IPS_IDLE);
 
     /* Record Frames */
@@ -119,8 +115,8 @@ bool StreamManager::initProperties()
                      STREAM_TAB, IP_RW, 0, IPS_IDLE);
 
     /* Record Options */
-    IUFillNumber(&RecordOptionsN[0], "RECORD_DURATION", "Duration (sec)", "%6.3f", 0.001, 999999.0, 0.0, 1);
-    IUFillNumber(&RecordOptionsN[1], "RECORD_FRAME_TOTAL", "Frames", "%9.0f", 1.0, 999999999.0, 1.0, 30.0);
+    IUFillNumber(&RecordOptionsN[0], "RECORD_DURATION", "Duration (sec)", "%.3f", 0.001, 999999.0, 0.0, 1);
+    IUFillNumber(&RecordOptionsN[1], "RECORD_FRAME_TOTAL", "Frames", "%.f", 1.0, 999999999.0, 1.0, 30.0);
     IUFillNumberVector(&RecordOptionsNP, RecordOptionsN, NARRAY(RecordOptionsN), getDeviceName(), "RECORD_OPTIONS",
                        "Record Options", STREAM_TAB, IP_RW, 60, IPS_IDLE);
 
@@ -135,10 +131,10 @@ bool StreamManager::initProperties()
     if(currentDevice->getDriverInterface() & INDI::DefaultDevice::CCD_INTERFACE)
     {
         // CCD Streaming Frame
-        IUFillNumber(&StreamFrameN[0], "X", "Left ", "%4.0f", 0, 0.0, 0, 0);
-        IUFillNumber(&StreamFrameN[1], "Y", "Top", "%4.0f", 0, 0, 0, 0);
-        IUFillNumber(&StreamFrameN[2], "WIDTH", "Width", "%4.0f", 0, 0.0, 0, 0.0);
-        IUFillNumber(&StreamFrameN[3], "HEIGHT", "Height", "%4.0f", 0, 0, 0, 0.0);
+        IUFillNumber(&StreamFrameN[0], "X", "Left ", "%.f", 0, 0.0, 0, 0);
+        IUFillNumber(&StreamFrameN[1], "Y", "Top", "%.f", 0, 0, 0, 0);
+        IUFillNumber(&StreamFrameN[2], "WIDTH", "Width", "%.f", 0, 0.0, 0, 0.0);
+        IUFillNumber(&StreamFrameN[3], "HEIGHT", "Height", "%.f", 0, 0, 0, 0.0);
         IUFillNumberVector(&StreamFrameNP, StreamFrameN, 4, getDeviceName(), "CCD_STREAM_FRAME", "Frame", STREAM_TAB, IP_RW,
                            60, IPS_IDLE);
     }
@@ -166,6 +162,12 @@ bool StreamManager::initProperties()
 #ifndef HAVE_THEORA
     RecorderSP.nsp = 1;
 #endif
+
+    // Limits
+    IUFillNumber(&LimitsN[LIMITS_BUFFER_MAX], "LIMITS_BUFFER_MAX", "Maximum Buffer Size (MB)", "%.0f", 1, 1024*64, 1, 512);
+    IUFillNumber(&LimitsN[LIMITS_PREVIEW_FPS], "LIMITS_PREVIEW_FPS", "Maximum Preview FPS", "%.0f", 1, 120, 1, 10);
+    IUFillNumberVector(&LimitsNP, LimitsN, NARRAY(LimitsN), getDeviceName(), "LIMITS",
+                       "Limits", STREAM_TAB, IP_RW, 0, IPS_IDLE);
     return true;
 }
 
@@ -187,6 +189,7 @@ void StreamManager::ISGetProperties(const char * dev)
         currentDevice->defineNumber(&StreamFrameNP);
         currentDevice->defineSwitch(&EncoderSP);
         currentDevice->defineSwitch(&RecorderSP);
+        currentDevice->defineNumber(&LimitsNP);
     }
 }
 
@@ -214,6 +217,7 @@ bool StreamManager::updateProperties()
         currentDevice->defineNumber(&StreamFrameNP);
         currentDevice->defineSwitch(&EncoderSP);
         currentDevice->defineSwitch(&RecorderSP);
+        currentDevice->defineNumber(&LimitsNP);
     }
     else
     {
@@ -227,6 +231,7 @@ bool StreamManager::updateProperties()
         currentDevice->deleteProperty(StreamFrameNP.name);
         currentDevice->deleteProperty(EncoderSP.name);
         currentDevice->deleteProperty(RecorderSP.name);
+        currentDevice->deleteProperty(LimitsNP.name);
     }
 
     return true;
@@ -239,109 +244,200 @@ bool StreamManager::updateProperties()
  * Binned frame must be sent from the camera driver for this to work consistentaly for all drivers.*/
 void StreamManager::newFrame(const uint8_t * buffer, uint32_t nbytes)
 {
-    m_FrameCounterPerSecond += 1;
-    if (StreamExposureN[STREAM_DIVISOR].value > 1
-            && (m_FrameCounterPerSecond % static_cast<int>(StreamExposureN[STREAM_DIVISOR].value)) == 0)
+    // close the data stream on the same thread as the data stream
+    // manually triggered to stop recording.
+    if (m_isRecordingAboutToClose)
+    {
+        stopRecording();
         return;
-
-    double ms1, ms2, deltams;
-    // Measure FPS
-    getitimer(ITIMER_REAL, &tframe2);
-    ms1 = (1000.0 * tframe1.it_value.tv_sec) + (tframe1.it_value.tv_usec / 1000.0);
-    ms2 = (1000.0 * tframe2.it_value.tv_sec) + (tframe2.it_value.tv_usec / 1000.0);
-    if (ms2 > ms1)
-        deltams = ms2 - ms1;
-    else
-        deltams = ms1 - ms2;
-
-    tframe1 = tframe2;
-    mssum += deltams;
-
-    double newFPS = 1000.0 / deltams;
-    if (mssum >= 1000.0)
-    {
-        FpsN[1].value = (m_FrameCounterPerSecond * 1000.0) / mssum;
-        mssum         = 0;
-        m_FrameCounterPerSecond = 0;
     }
 
-    // Only send FPS when there is a substancial update
-    if (fabs(newFPS - FpsN[0].value) > 1 || m_FrameCounterPerSecond == 0)
+    // Discard every N frame.
+    // do not count it to fps statistics
+    // N is StreamExposureN[STREAM_DIVISOR].value
+    ++m_frameCountDivider;
+    if (
+        (StreamExposureN[STREAM_DIVISOR].value > 1) &&
+        (m_frameCountDivider % static_cast<int>(StreamExposureN[STREAM_DIVISOR].value)) == 0
+    )
     {
-        FpsN[0].value = newFPS;
-        IDSetNumber(&FpsNP, nullptr);
+        return;
     }
 
-    //if (m_isStreaming)
-    std::thread(&StreamManager::asyncStream, this, buffer, nbytes, deltams).detach();
-    // JM 2020-03-07: Temporarily disable threading for recording until
-    // callback file descriptor looping issue is figured out.
-    //    else if (m_isRecording)
-    //        asyncStream(buffer, nbytes, deltams);
+    if (m_FPSAverage.newFrame())
+    {
+        FpsN[1].value = m_FPSAverage.framesPerSecond();
+    }
+
+    if (m_FPSFast.newFrame())
+    {
+        FpsN[0].value = m_FPSFast.framesPerSecond();
+        if (m_fastFPSUpdate.try_lock()) // don't block stream thread / record thread
+            std::thread([&](){ IDSetNumber(&FpsNP, nullptr); m_fastFPSUpdate.unlock(); }).detach();
+    }
+
+    if (isStreaming() || isRecording())
+    {
+        size_t allocatedSize = nbytes * m_framesIncoming.size() / 1024 / 1024; // allocated size in MB
+        if (allocatedSize > LimitsN[LIMITS_BUFFER_MAX].value)
+        {
+            LOG_WARN("Frame buffer is full, skipping frame...");
+            return;
+        }
+
+        std::vector<uint8_t> copyBuffer(buffer, buffer + nbytes); // copy the frame
+
+        m_framesIncoming.push(TimeFrame{m_FPSFast.deltaTime(), std::move(copyBuffer)}); // push it into the queue
+    }
+
+    if (isRecording())
+    {
+        m_FPSRecorder.newFrame(); // count frames and total time
+
+        // captured all frames, stream should be close
+        if (
+            (RecordStreamSP.sp[RECORD_FRAME].s == ISS_ON && m_FPSRecorder.totalFrames() >= (RecordOptionsNP.np[1].value)) ||
+            (RecordStreamSP.sp[RECORD_TIME].s  == ISS_ON && m_FPSRecorder.totalTime()   >= (RecordOptionsNP.np[0].value * 1000.0))
+        )
+        {
+            LOG_INFO("Waiting for all buffered frames to be recorded");
+            m_framesIncoming.waitForEmpty();
+            // duplicated message
+#if 0
+            LOGF_INFO(
+                "Ending record after %g millisecs and %d frames",
+                m_FPSRecorder.totalTime(),
+                m_FPSRecorder.totalFrames()
+            );
+#endif
+            RecordStreamSP.sp[RECORD_TIME].s  = ISS_OFF;
+            RecordStreamSP.sp[RECORD_FRAME].s = ISS_OFF;
+            RecordStreamSP.sp[RECORD_OFF].s   = ISS_ON;
+            RecordStreamSP.s = IPS_IDLE;
+            IDSetSwitch(&RecordStreamSP, nullptr);
+
+            stopRecording();
+        }
+    }
 }
 
-void StreamManager::asyncStream(const uint8_t *buffer, uint32_t nbytes, double deltams)
+void StreamManager::asyncStreamThread()
 {
-    std::lock_guard<std::mutex> guard((currentDevice->getDriverInterface() & INDI::DefaultDevice::CCD_INTERFACE) ?
-                                      dynamic_cast<INDI::CCD*>(currentDevice)->ccdBufferLock :
-                                      dynamic_cast<INDI::SensorInterface*>(currentDevice)->detectorBufferLock);
+    TimeFrame sourceTimeFrame;
+    sourceTimeFrame.time = 0;
 
-    // For recording, save immediately.
-    if (isRecording() && recordStream(buffer, nbytes, deltams) == false)
+    std::vector<uint8_t> subframeBuffer;  // Subframe buffer for recording/streaming
+    std::vector<uint8_t> downscaleBuffer; // Downscale buffer for streaming
+
+    double &frameW = StreamFrameN[CCDChip::FRAME_W].value;
+    double &frameH = StreamFrameN[CCDChip::FRAME_H].value;
+    double &frameX = StreamFrameN[CCDChip::FRAME_X].value;
+    double &frameY = StreamFrameN[CCDChip::FRAME_Y].value;
+
+    while(!m_framesThreadTerminate)
     {
-        LOG_ERROR("Recording failed.");
-        stopRecording(true);
-        return;
-    }
+        if (m_framesIncoming.pop(sourceTimeFrame) == false)
+            continue;
 
-    // For streaming, downscale to 8bit if higher than 8bit to reduce bandwidth
-    // When recording is active, send 1 frame every X recorded frames so that the streaming
-    // operation does not adversely affects the recording performance.
-    if (isStreaming())
-    {
-        // N.B. Send one stream frame per 10 recorded frames
-        // The 10 values needs to be fine-tuned and probably set as configurable
-        if (isRecording() && m_RecordingFrameTotal > 0 && ( (m_RecordingFrameTotal - 1) % 10) != 0)
-            return;
 
-        // Downscale to 8bit always for streaming to reduce bandwidth
-        if (m_PixelDepth > 8)
+        const uint8_t *sourceBufferData = sourceTimeFrame.frame.data();
+        uint32_t nbytes                 = sourceTimeFrame.frame.size();
+
+#if 1 // TODO move above the loop
+        int subX = 0, subY = 0, subW = 0, subH = 0;
+        uint32_t npixels = 0;
+        uint8_t components = (m_PixelFormat == INDI_RGB) ? 3 : 1;
+        uint8_t bytesPerPixel = (m_PixelDepth + 7) / 8;
+
+        if(currentDevice->getDriverInterface() & INDI::DefaultDevice::CCD_INTERFACE)
         {
-            uint32_t npixels = 0;
-            if(currentDevice->getDriverInterface() & INDI::DefaultDevice::CCD_INTERFACE)
-            {
-                npixels = (dynamic_cast<INDI::CCD*>(currentDevice)->PrimaryCCD.getSubW() / dynamic_cast<INDI::CCD*>
-                           (currentDevice)->PrimaryCCD.getBinX()) * (dynamic_cast<INDI::CCD*>(currentDevice)->PrimaryCCD.getSubH() /
-                                   dynamic_cast<INDI::CCD*>(currentDevice)->PrimaryCCD.getBinY()) * ((m_PixelFormat == INDI_RGB) ? 3 : 1);
-            }
-            else if(currentDevice->getDriverInterface() & INDI::DefaultDevice::SENSOR_INTERFACE)
-            {
-                npixels = nbytes * 8 / dynamic_cast<INDI::SensorInterface*>(currentDevice)->getBPS();
-            }
-
-            // Allocale new buffer if size changes
-            if (downscaleBufferSize != npixels)
-            {
-                downscaleBufferSize = npixels;
-                delete [] downscaleBuffer;
-                downscaleBuffer = new uint8_t[npixels];
-            }
-
-            const uint16_t * srcBuffer = reinterpret_cast<const uint16_t *>(buffer);
-
-            // Apply gamma
-            for (uint32_t i = 0; i < npixels; i++)
-                downscaleBuffer[i] = gammaLUT_16_8[srcBuffer[i]];
-
-            nbytes /= 2;
-
-            uploadStream(downscaleBuffer, nbytes);
+            INDI::CCD * ccd = dynamic_cast<INDI::CCD*>(currentDevice);
+            subX = ccd->PrimaryCCD.getSubX() / ccd->PrimaryCCD.getBinX();
+            subY = ccd->PrimaryCCD.getSubY() / ccd->PrimaryCCD.getBinY();
+            subW = ccd->PrimaryCCD.getSubW() / ccd->PrimaryCCD.getBinX();
+            subH = ccd->PrimaryCCD.getSubH() / ccd->PrimaryCCD.getBinY();
+            npixels = subW * subH * components;
         }
-        else if (uploadStream(buffer, nbytes) == false)
+        else if(currentDevice->getDriverInterface() & INDI::DefaultDevice::SENSOR_INTERFACE)
         {
-            LOG_ERROR("Streaming failed.");
-            setStream(false);
-            return;
+            INDI::SensorInterface* si = dynamic_cast<INDI::SensorInterface*>(currentDevice);
+            subX = 0;
+            subY = 0;
+            subW = si->getBufferSize() * 8 / si->getBPS();
+            subH = 1;
+            npixels = nbytes * 8 / si->getBPS();
+        }
+
+        // If stream frame was not yet initilized, let's do that now
+        if (frameW == 0 || frameH == 0)
+        {
+            //if (dynamic_cast<INDI::CCD*>(currentDevice)->PrimaryCCD.getNAxis() == 2)
+            //    binFactor = dynamic_cast<INDI::CCD*>(currentDevice)->PrimaryCCD.getBinX();
+            frameX = subX;
+            frameY = subY;
+            frameW = subW;
+            frameH = subH;
+            StreamFrameNP.s = IPS_IDLE;
+            IDSetNumber(&StreamFrameNP, nullptr);
+        }
+#endif
+
+        // Check if we need to subframe
+        if (
+            (m_PixelFormat != INDI_JPG) &&
+            (frameW > 0 && frameH > 0) &&
+            (frameX != subX || frameY != subY || frameW != subW || frameH != subH))
+        {
+            npixels = frameW * frameH * components;
+
+            subframeBuffer.resize(npixels * bytesPerPixel);
+
+            uint32_t srcOffset = components * bytesPerPixel * (frameY * subW + frameX);
+
+            const uint8_t * srcBuffer = sourceBufferData + srcOffset;
+            uint32_t        srcStride = components * bytesPerPixel * subW;
+
+            uint8_t *       dstBuffer = subframeBuffer.data();
+            uint32_t        dstStride = components * bytesPerPixel * frameW;
+
+            // Copy line-by-line
+            for (int i = 0; i < frameH; ++i)
+                memcpy(dstBuffer + i * dstStride, srcBuffer + srcStride * i, dstStride);
+
+            sourceBufferData = dstBuffer;
+            nbytes = frameW * frameH * components * bytesPerPixel;
+        }
+
+        {
+            std::lock_guard<std::mutex> lock(m_recordMutex);
+            // For recording, save immediately.
+            if (isRecording() && recordStream(sourceBufferData, nbytes, sourceTimeFrame.time) == false)
+            {
+                LOG_ERROR("Recording failed.");
+                m_isRecordingAboutToClose = true;
+            }
+        }
+
+        // For streaming, downscale to 8bit if higher than 8bit to reduce bandwidth
+        // You can reduce the number of frames by setting a frame limit.
+        if (isStreaming() && m_FPSPreview.newFrame())
+        {
+            // Downscale to 8bit always for streaming to reduce bandwidth
+            if (m_PixelFormat != INDI_JPG && m_PixelDepth > 8)
+            {
+                // Allocale new buffer if size changes
+                downscaleBuffer.resize(npixels);
+
+                const uint16_t * srcBuffer = reinterpret_cast<const uint16_t *>(sourceBufferData);
+                uint8_t *        dstBuffer = downscaleBuffer.data();
+
+                // Apply gamma
+                m_gammaLut16.apply(srcBuffer, npixels, dstBuffer);
+
+                sourceBufferData = dstBuffer;
+                nbytes /= 2;
+            }
+            uploadStream(sourceBufferData, nbytes);
         }
     }
 }
@@ -381,6 +477,7 @@ void StreamManager::setSize(uint16_t width, uint16_t height)
 
 bool StreamManager::close()
 {
+    std::lock_guard<std::mutex> lock(m_recordMutex);
     return recorder->close();
 }
 
@@ -415,128 +512,34 @@ bool StreamManager::setPixelFormat(INDI_PIXEL_FORMAT pixelFormat, uint8_t pixelD
 
 bool StreamManager::recordStream(const uint8_t * buffer, uint32_t nbytes, double deltams)
 {
+    INDI_UNUSED(deltams);
     if (!m_isRecording)
         return false;
 
-    bool rc = recorder->writeFrame(buffer, nbytes);
-    if (rc == false)
-        return rc;
-
-    m_RecordingFrameDuration += deltams;
-    m_RecordingFrameTotal += 1;
-
-    if ((RecordStreamSP.sp[RECORD_TIME].s == ISS_ON) && (m_RecordingFrameDuration >= (RecordOptionsNP.np[0].value * 1000.0)))
-    {
-        LOGF_INFO("Ending record after %g millisecs", m_RecordingFrameDuration);
-        stopRecording();
-        IUResetSwitch(&RecordStreamSP);
-        RecordStreamSP.sp[RECORD_OFF].s = ISS_ON;
-        RecordStreamSP.s = IPS_IDLE;
-        IDSetSwitch(&RecordStreamSP, nullptr);
-        return true;
-    }
-
-    if ((RecordStreamSP.sp[RECORD_FRAME].s == ISS_ON) && (m_RecordingFrameTotal >= (RecordOptionsNP.np[1].value)))
-    {
-        LOGF_INFO("Ending record after %d frames", m_RecordingFrameTotal);
-        stopRecording();
-        RecordStreamSP.sp[RECORD_FRAME].s = ISS_OFF;
-        RecordStreamSP.sp[RECORD_OFF].s = ISS_ON;
-        RecordStreamSP.s = IPS_IDLE;
-        IDSetSwitch(&RecordStreamSP, nullptr);
-        return true;
-    }
-
-    return true;
-}
-
-int StreamManager::mkpath(std::string s, mode_t mode)
-{
-    size_t pre = 0, pos;
-    std::string dir;
-    int mdret = 0;
-    struct stat st;
-
-    if (s[s.size() - 1] != '/')
-        s += '/';
-
-    while ((pos = s.find_first_of('/', pre)) != std::string::npos)
-    {
-        dir = s.substr(0, pos++);
-        pre = pos;
-        if (dir.size() == 0)
-            continue;
-        if (stat(dir.c_str(), &st))
-        {
-            if (errno != ENOENT || ((mdret = mkdir(dir.c_str(), mode)) && errno != EEXIST))
-            {
-                LOGF_WARN("mkpath: can not create %s", dir.c_str());
-                return mdret;
-            }
-        }
-        else
-        {
-            if (!S_ISDIR(st.st_mode))
-            {
-                LOGF_WARN("mkpath: %s is not a directory", dir.c_str());
-                return -1;
-            }
-        }
-    }
-    return mdret;
+    return recorder->writeFrame(buffer, nbytes);
 }
 
 std::string StreamManager::expand(const std::string &fname, const std::map<std::string, std::string> &patterns)
 {
-    std::string res = fname;
-    std::size_t pos;
-    time_t now;
-    struct tm * tm_now;
-    char val[20];
-    *(val + 19) = '\0';
+    std::string result = fname;
 
-    time(&now);
-    tm_now = gmtime(&now);
+    std::time_t t  =  std::time(nullptr);
+    std::tm     tm = *std::gmtime(&t);
 
-    pos = res.find("_D_");
-    if (pos != std::string::npos)
+    auto extendedPatterns = patterns;
+    extendedPatterns["_D_"] = format_time(tm, "%Y-%m-%d");
+    extendedPatterns["_H_"] = format_time(tm, "%H-%M-%S");
+    extendedPatterns["_T_"] = format_time(tm, "%Y-%m-%d" "@" "%H-%M-%S");
+
+    for(const auto &pattern: extendedPatterns)
     {
-        strftime(val, 11, "%F", tm_now);
-        res.replace(pos, 3, val);
-    }
-
-    pos = res.find("_T_");
-    if (pos != std::string::npos)
-    {
-        strftime(val, 20, "%F@%T", tm_now);
-        res.replace(pos, 3, val);
-    }
-
-    pos = res.find("_H_");
-    if (pos != std::string::npos)
-    {
-        strftime(val, 9, "%T", tm_now);
-        res.replace(pos, 3, val);
-    }
-
-    for (std::map<std::string, std::string>::const_iterator it = patterns.begin(); it != patterns.end(); ++it)
-    {
-        pos = res.find(it->first);
-        if (pos != std::string::npos)
-        {
-            res.replace(pos, it->first.size(), it->second);
-        }
+        replace_all(result, pattern.first, pattern.second);
     }
 
     // Replace all : to - to be valid filename on Windows
-    size_t start_pos = 0;
-    while ((start_pos = res.find(":", start_pos)) != std::string::npos)
-    {
-        res.replace(start_pos, 1, "-");
-        start_pos++;
-    }
+    std::replace(result.begin(), result.end(), ':', '-'); // it's really needed now?
 
-    return res;
+    return result;
 }
 
 bool StreamManager::startRecording()
@@ -611,12 +614,15 @@ bool StreamManager::startRecording()
             recorder->setDefaultColor();
     }
 #endif
-    m_RecordingFrameDuration   = 0.0;
-    m_RecordingFrameTotal = 0;
+    m_FPSRecorder.reset();
+    m_frameCountDivider = 0;
+
+    if (m_isStreaming == false)
+    {
+        m_FPSAverage.reset();
+        m_FPSFast.reset();
+    }
 
-    getitimer(ITIMER_REAL, &tframe1);
-    mssum         = 0;
-    m_FrameCounterPerSecond = 0;
     if(currentDevice->getDriverInterface() & INDI::DefaultDevice::CCD_INTERFACE)
     {
         if (m_isStreaming == false && dynamic_cast<INDI::CCD*>(currentDevice)->StartStreaming() == false)
@@ -647,6 +653,7 @@ bool StreamManager::stopRecording(bool force)
 {
     if (!m_isRecording && force == false)
         return true;
+
     if(currentDevice->getDriverInterface() & INDI::DefaultDevice::CCD_INTERFACE)
     {
         if (!m_isStreaming)
@@ -660,13 +667,21 @@ bool StreamManager::stopRecording(bool force)
     }
 
     m_isRecording = false;
-    recorder->close();
+    m_isRecordingAboutToClose = false;
+
+    {
+        std::lock_guard<std::mutex> lock(m_recordMutex);
+        recorder->close();
+    }
 
     if (force)
         return false;
 
-    LOGF_INFO("Record Duration(millisec): %g -- Frame count: %d", m_RecordingFrameDuration,
-              m_RecordingFrameTotal);
+    LOGF_INFO(
+        "Record Duration: %g millisec / %d frames",
+        m_FPSRecorder.totalTime(),
+        m_FPSRecorder.totalFrames()
+    );
 
     return true;
 }
@@ -739,8 +754,8 @@ bool StreamManager::ISNewSwitch(const char * dev, const char * name, ISState * s
             FpsN[FPS_INSTANT].value = FpsN[FPS_AVERAGE].value = 0;
             if (m_isRecording)
             {
-                LOGF_INFO("Recording stream has been disabled. Frame count %d", m_RecordingFrameTotal);
-                stopRecording();
+                LOG_INFO("Recording stream has been disabled. Closing the stream...");
+                m_isRecordingAboutToClose = true;
             }
         }
 
@@ -840,6 +855,19 @@ bool StreamManager::ISNewNumber(const char * dev, const char * name, double valu
         return true;
     }
 
+    /* Limits */
+    if (!strcmp(LimitsNP.name, name))
+    {
+        IUUpdateNumber(&LimitsNP, values, names, n);
+
+        m_FPSPreview.setTimeWindow(1000.0 / LimitsN[LIMITS_PREVIEW_FPS].value);
+        m_FPSPreview.reset();
+
+        LimitsNP.s = IPS_OK;
+        IDSetNumber(&LimitsNP, nullptr);
+        return true;
+    }
+
     /* Record Options */
     if (!strcmp(RecordOptionsNP.name, name))
     {
@@ -917,9 +945,12 @@ bool StreamManager::setStream(bool enable)
             LOGF_INFO("Starting the video stream with target exposure %.6f s (Max theoritical FPS %.f)", StreamExposureN[0].value,
                       1 / StreamExposureN[0].value);
 
-            getitimer(ITIMER_REAL, &tframe1);
-            mssum         = 0;
-            m_FrameCounterPerSecond = 0;
+            m_FPSAverage.reset();
+            m_FPSFast.reset();
+            m_FPSPreview.reset();
+            m_FPSPreview.setTimeWindow(1000.0 / LimitsN[LIMITS_PREVIEW_FPS].value);
+            m_frameCountDivider = 0;
+            
             if(currentDevice->getDriverInterface() & INDI::DefaultDevice::CCD_INTERFACE)
             {
                 if (dynamic_cast<INDI::CCD*>(currentDevice)->StartStreaming() == false)
@@ -1043,7 +1074,7 @@ bool StreamManager::uploadStream(const uint8_t * buffer, uint32_t nbytes)
         return true;
     }
 
-    // Binning for grayscale frames only for now
+    // Binning for grayscale frames only for now - REMOVE ME
 #if 0
     if (dynamic_cast<INDI::CCD*>(currentDevice)->PrimaryCCD.getNAxis() == 2)
     {
@@ -1053,109 +1084,6 @@ bool StreamManager::uploadStream(const uint8_t * buffer, uint32_t nbytes)
     }
 #endif
 
-    int subX, subY, subW, subH;
-    subX = subY = 0;
-    subW = subH = 0;
-    if(currentDevice->getDriverInterface() & INDI::DefaultDevice::CCD_INTERFACE)
-    {
-        subX = dynamic_cast<INDI::CCD*>(currentDevice)->PrimaryCCD.getSubX() / dynamic_cast<INDI::CCD*>
-               (currentDevice)->PrimaryCCD.getBinX();
-        subY = dynamic_cast<INDI::CCD*>(currentDevice)->PrimaryCCD.getSubY() / dynamic_cast<INDI::CCD*>
-               (currentDevice)->PrimaryCCD.getBinY();
-        subW = dynamic_cast<INDI::CCD*>(currentDevice)->PrimaryCCD.getSubW() / dynamic_cast<INDI::CCD*>
-               (currentDevice)->PrimaryCCD.getBinX();
-        subH = dynamic_cast<INDI::CCD*>(currentDevice)->PrimaryCCD.getSubH() / dynamic_cast<INDI::CCD*>
-               (currentDevice)->PrimaryCCD.getBinY();
-    }
-    else if(currentDevice->getDriverInterface() & INDI::DefaultDevice::SENSOR_INTERFACE)
-    {
-        subX = 0;
-        subY = 0;
-        subW = dynamic_cast<INDI::SensorInterface*>(currentDevice)->getBufferSize() * 8 / dynamic_cast<INDI::SensorInterface*>
-               (currentDevice)->getBPS();
-        subH = 1;
-    }
-
-    // If stream frame was not yet initilized, let's do that now
-    if (StreamFrameN[CCDChip::FRAME_W].value == 0 || StreamFrameN[CCDChip::FRAME_H].value == 0)
-    {
-        //if (dynamic_cast<INDI::CCD*>(currentDevice)->PrimaryCCD.getNAxis() == 2)
-        //    binFactor = dynamic_cast<INDI::CCD*>(currentDevice)->PrimaryCCD.getBinX();
-
-        StreamFrameN[CCDChip::FRAME_X].value = subX;
-        StreamFrameN[CCDChip::FRAME_Y].value = subY;
-        StreamFrameN[CCDChip::FRAME_W].value = subW;
-        StreamFrameN[CCDChip::FRAME_W].value = subH;
-        StreamFrameNP.s                      = IPS_IDLE;
-        IDSetNumber(&StreamFrameNP, nullptr);
-    }
-    // Check if we need to subframe
-    else if ((StreamFrameN[CCDChip::FRAME_W].value > 0 && StreamFrameN[CCDChip::FRAME_H].value > 0) &&
-             (StreamFrameN[CCDChip::FRAME_X].value != subX || StreamFrameN[CCDChip::FRAME_Y].value != subY ||
-              StreamFrameN[CCDChip::FRAME_W].value != subW || StreamFrameN[CCDChip::FRAME_H].value != subH))
-    {
-        uint32_t npixels = StreamFrameN[CCDChip::FRAME_W].value * StreamFrameN[CCDChip::FRAME_H].value * ((
-                               m_PixelFormat == INDI_RGB) ? 3 : 1);
-        if (downscaleBufferSize < npixels)
-        {
-            downscaleBufferSize = npixels;
-            delete [] downscaleBuffer;
-            downscaleBuffer = new uint8_t[npixels];
-        }
-
-        uint32_t sourceOffset = (subW * StreamFrameN[CCDChip::FRAME_Y].value) + StreamFrameN[CCDChip::FRAME_X].value;
-        uint8_t components = (m_PixelFormat == INDI_RGB) ? 3 : 1;
-
-        const uint8_t * srcBuffer  = buffer + sourceOffset * components;
-        uint32_t sourceStride = subW * components;
-
-        uint8_t * destBuffer = downscaleBuffer;
-        uint32_t desStride = StreamFrameN[CCDChip::FRAME_W].value * components;
-
-        // Copy line-by-line
-        for (int i = 0; i < StreamFrameN[CCDChip::FRAME_H].value; i++)
-            memcpy(destBuffer + i * desStride, srcBuffer + sourceStride * i, desStride);
-
-        nbytes = StreamFrameN[CCDChip::FRAME_W].value * StreamFrameN[CCDChip::FRAME_H].value * components;
-
-        if(currentDevice->getDriverInterface() & INDI::DefaultDevice::CCD_INTERFACE)
-        {
-            if (encoder->upload(imageB, downscaleBuffer, nbytes, dynamic_cast<INDI::CCD*>(currentDevice)->PrimaryCCD.isCompressed()))
-            {
-#ifdef HAVE_WEBSOCKET
-                if (dynamic_cast<INDI::CCD*>(currentDevice)->HasWebSocket()
-                        && dynamic_cast<INDI::CCD*>(currentDevice)->WebSocketS[CCD::WEBSOCKET_ENABLED].s == ISS_ON)
-                {
-                    if (m_Format != ".stream")
-                    {
-                        m_Format = ".stream";
-                        dynamic_cast<INDI::CCD*>(currentDevice)->wsServer.send_text(m_Format);
-                    }
-
-                    dynamic_cast<INDI::CCD*>(currentDevice)->wsServer.send_binary(downscaleBuffer, nbytes);
-                    return true;
-                }
-#endif
-                // Upload to client now
-                imageBP->s = IPS_OK;
-                IDSetBLOB(imageBP, nullptr);
-                return true;
-            }
-        }
-        else if(currentDevice->getDriverInterface() & INDI::DefaultDevice::SENSOR_INTERFACE)
-        {
-            if (encoder->upload(imageB, downscaleBuffer, nbytes, false))
-            {
-                // Upload to client now
-                imageBP->s = IPS_OK;
-                IDSetBLOB(imageBP, nullptr);
-                return true;
-            }
-        }
-
-        return false;
-    }
-
     if(currentDevice->getDriverInterface() & INDI::DefaultDevice::CCD_INTERFACE)
     {
         if (encoder->upload(imageB, buffer, nbytes, dynamic_cast<INDI::CCD*>(currentDevice)->PrimaryCCD.isCompressed()))
@@ -1193,21 +1121,4 @@ bool StreamManager::uploadStream(const uint8_t * buffer, uint32_t nbytes)
 
     return false;
 }
-
-void StreamManager::prepareGammaLUT(double gamma, double a, double b, double Ii)
-{
-    if (!gammaLUT_16_8) gammaLUT_16_8 = new uint8_t[65536];
-
-    for (int i = 0; i < 65536; i++)
-    {
-        double I = static_cast<double>(i) / 65535.0;
-        double p;
-        if (I <= Ii)
-            p = a * I;
-        else
-            p = (1 + b) * powf(I, 1.0 / gamma) - b;
-        gammaLUT_16_8[i] = round(255.0 * p);
-    }
-}
-
 }
